'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _uiUtil = require('../utils/uiUtil');

var _envUtil = require('../utils/envUtil');

var _componentUtil = require('../utils/componentUtil');

var _Sequence = require('../utils/Sequence');

var _Sequence2 = _interopRequireDefault(_Sequence);

var _ModalTitle = require('./ModalTitle');

var _ModalTitle2 = _interopRequireDefault(_ModalTitle);

var _ModalContent = require('./ModalContent');

var _ModalContent2 = _interopRequireDefault(_ModalContent);

var _ModalFooter = require('./ModalFooter');

var _ModalFooter2 = _interopRequireDefault(_ModalFooter);

var _ModalPreface = require('./ModalPreface');

var _ModalPreface2 = _interopRequireDefault(_ModalPreface);

var _ModalLoading = require('./ModalLoading');

var _ModalLoading2 = _interopRequireDefault(_ModalLoading);

var _Button = require('../Button');

var _Button2 = _interopRequireDefault(_Button);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var modalList = [];

// ========================================================
// =                          UI                          =
// ========================================================
function refreshWinScrollBar() {
	if (!_envUtil.canUseDOM) return;

	var hasActiveModal = modalList.some(function (_ref) {
		var _ref$state = _ref.state;
		_ref$state = _ref$state === undefined ? {} : _ref$state;
		var animateStatus = _ref$state.animateStatus;
		return animateStatus !== _uiUtil.ANIMATE_STATUS_NONE;
	});

	if (hasActiveModal) {
		(0, _uiUtil.disableWinScrollBar)('Modal');
	} else {
		(0, _uiUtil.enableWinScrollBar)('Modal');
	}
}

// ========================================================
// =                         Modal                        =
// ========================================================

var Modal = function (_React$Component) {
	_inherits(Modal, _React$Component);

	function Modal() {
		_classCallCheck(this, Modal);

		var _this = _possibleConstructorReturn(this, (Modal.__proto__ || Object.getPrototypeOf(Modal)).call(this));

		_this.onBackgroundClick = function () {
			var noMaskClose = _this.props.noMaskClose;

			if (noMaskClose) return;

			// Put close request in next frame
			_this.closeSeq.next(function () {
				var onClose = _this.props.onClose;

				if (onClose) onClose();
			});
		};

		_this.onHolderClick = function () {
			// Occupy the close request if is holder click
			_this.closeSeq.next(function () {}, { priority: 1 });
		};

		_this.onTransitionEnd = function (event) {
			var onClosed = _this.props.onClosed;
			var animateStatus = _this.state.animateStatus;


			if (event.target === _this.$holder && animateStatus === _uiUtil.ANIMATE_STATUS_HIDING) {
				if (onClosed) onClosed(event);

				_this.seq.next(function () {
					_this.setState({
						animateStatus: _uiUtil.ANIMATE_STATUS_NONE
					}, refreshWinScrollBar);
				});
			}
		};

		_this.setHolderRef = function (ele) {
			_this.$holder = ele;
		};

		_this.getTitle = function () {
			var _this$props = _this.props,
			    children = _this$props.children,
			    title = _this$props.title;


			if (title) return _react2.default.createElement(
				_ModalTitle2.default,
				null,
				title
			);
			return (0, _componentUtil.mapChildrenByType)(children, _ModalTitle.BAMBOO_MODAL_TITLE);
		};

		_this.getBody = function () {
			var _this$props2 = _this.props,
			    children = _this$props2.children,
			    content = _this$props2.content;


			if (content) return _react2.default.createElement(
				_ModalContent2.default,
				null,
				content
			);

			var $children = (0, _componentUtil.mapChildrenByType)(children, _ModalContent.BAMBOO_MODAL_CONTENT);
			if ($children.length) return $children;

			return _react2.default.createElement(
				_ModalContent2.default,
				null,
				(0, _componentUtil.mapChildrenByNotType)(children, [_ModalTitle.BAMBOO_MODAL_TITLE, _ModalFooter.BAMBOO_MODAL_FOOTER, _ModalPreface.BAMBOO_MODAL_PREFACE, _ModalLoading.BAMBOO_MODAL_LOADING])
			);
		};

		_this.getFooter = function () {
			var _this$props3 = _this.props,
			    children = _this$props3.children,
			    footer = _this$props3.footer,
			    onClose = _this$props3.onClose,
			    onConfirm = _this$props3.onConfirm,
			    _this$props3$closeTex = _this$props3.closeText,
			    closeText = _this$props3$closeTex === undefined ? 'Close' : _this$props3$closeTex,
			    _this$props3$confirmT = _this$props3.confirmText,
			    confirmText = _this$props3$confirmT === undefined ? 'Confirm' : _this$props3$confirmT,
			    lock = _this$props3.lock;


			if (footer) return _react2.default.createElement(
				_ModalFooter2.default,
				null,
				footer
			);

			var $footer = (0, _componentUtil.mapChildrenByType)(children, _ModalFooter.BAMBOO_MODAL_FOOTER);
			if ($footer.length) return $footer;

			return _react2.default.createElement(
				_ModalFooter2.default,
				null,
				_react2.default.createElement(
					_Button2.default,
					{ type: 'weak', onClick: onClose, disabled: lock, transparent: true },
					closeText
				),
				onConfirm && _react2.default.createElement(
					_Button2.default,
					{ onClick: onConfirm, disabled: lock },
					confirmText
				)
			);
		};

		_this.getLoading = function () {
			var children = _this.props.children;

			var $loading = (0, _componentUtil.mapChildrenByType)(children, _ModalLoading.BAMBOO_MODAL_LOADING);

			if ($loading.length) return $loading;

			return _react2.default.createElement(_ModalLoading2.default, { defaultView: true });
		};

		_this.getPreface = function () {
			var _this$props4 = _this.props,
			    children = _this$props4.children,
			    visible = _this$props4.visible;

			return (0, _componentUtil.mapChildrenByType)(children, _ModalPreface.BAMBOO_MODAL_PREFACE, function (node) {
				return _react2.default.cloneElement(node, { visible: visible });
			});
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			// Process when visibility changed
			if (!prevProps.visible !== !nextProps.visible) {
				if (nextProps.visible) {
					// Show modal
					_this.seq.next(function () {
						_this.setState({
							animateStatus: _uiUtil.ANIMATE_STATUS_SHOWING
						}, refreshWinScrollBar);
					}).next(function () {
						_this.setState({ animateStatus: _uiUtil.ANIMATE_STATUS_SHOWN });
					});
				} else {
					// Hide modal
					_this.seq.next(function () {
						_this.setState({
							animateStatus: _uiUtil.ANIMATE_STATUS_HIDING
						});
					}).next(function () {
						// Clean up when browser not support transitionEnd event
						var onClosed = _this.props.onClosed;

						_this.setState({ animateStatus: _uiUtil.ANIMATE_STATUS_NONE }, refreshWinScrollBar);

						if (onClosed) onClosed();
					}, { delay: 750 });
				}
			}
		};

		_this.state = {
			animateStatus: _uiUtil.ANIMATE_STATUS_NONE
		};

		_this.seq = new _Sequence2.default();
		_this.closeSeq = new _Sequence2.default();
		return _this;
	}

	_createClass(Modal, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			modalList.push(this);
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			var _this2 = this;

			this.seq.destroy();
			modalList = modalList.filter(function (mdl) {
				return mdl !== _this2;
			});
			refreshWinScrollBar();
		}
	}, {
		key: 'render',
		value: function render() {
			var _props = this.props,
			    size = _props.size,
			    type = _props.type,
			    loading = _props.loading,
			    className = _props.className,
			    props = _objectWithoutProperties(_props, ['size', 'type', 'loading', 'className']);

			var animateStatus = this.state.animateStatus;


			if (animateStatus === _uiUtil.ANIMATE_STATUS_NONE) return null;

			var $title = this.getTitle();
			var $body = this.getBody();
			var $footer = this.getFooter();
			var $preface = this.getPreface();
			var $loading = this.getLoading();

			var $modal = _react2.default.createElement(
				'div',
				{ className: 'bmbo-dialog' },
				$title,
				$body,
				$footer,
				$loading
			);

			delete props.visible;
			delete props.lock;
			delete props.title;
			delete props.content;
			delete props.footer;
			delete props.onClose;
			delete props.onClosed;
			delete props.onConfirm;
			delete props.closeText;
			delete props.confirmText;
			delete props.noMaskClose;

			return (0, _uiUtil.createPortal)(_react2.default.createElement(
				'div',
				_extends({}, props, {
					className: (0, _classnames2.default)('bmbo-modal', {
						'bmbo-showing': animateStatus === _uiUtil.ANIMATE_STATUS_SHOWING,
						'bmbo-hidding': animateStatus === _uiUtil.ANIMATE_STATUS_HIDING,
						'bmbo-loading': loading,
						className: className
					}, 'bmbo-modal-' + (size || 'md'), type && 'bmbo-' + type),
					role: 'presentation',
					onClick: this.onBackgroundClick
				}),
				_react2.default.createElement(
					'div',
					{
						className: 'bmbo-dialog-holder',
						role: 'presentation',
						onClick: this.onHolderClick,
						ref: this.setHolderRef,
						onTransitionEnd: this.onTransitionEnd
					},
					$preface,
					$modal
				)
			));
		}
	}]);

	return Modal;
}(_react2.default.Component);

Modal.propTypes = {
	visible: _propTypes2.default.bool,
	children: _propTypes2.default.node,
	className: _propTypes2.default.string,
	size: _propTypes2.default.string,
	type: _propTypes2.default.string,
	loading: _propTypes2.default.bool,
	lock: _propTypes2.default.bool,

	title: _propTypes2.default.node,
	content: _propTypes2.default.node,
	footer: _propTypes2.default.node,

	onClose: _propTypes2.default.func,
	onClosed: _propTypes2.default.func,
	onConfirm: _propTypes2.default.func,

	closeText: _propTypes2.default.func,
	confirmText: _propTypes2.default.func,

	noMaskClose: _propTypes2.default.bool
};

Modal.Title = _ModalTitle2.default;
Modal.Content = _ModalContent2.default;
Modal.Footer = _ModalFooter2.default;
Modal.Preface = _ModalPreface2.default;
Modal.Loading = _ModalLoading2.default;

exports.default = Modal;