'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.mapChildrenByNotType = exports.everyChildrenByType = exports.someChildrenByType = exports.mapChildrenByType = exports.mapChildrenForNode = exports.mapChildren = exports.BAMBOO_INTERNAL_REF = exports.BAMBOO_COMPONENT = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BAMBOO_COMPONENT = exports.BAMBOO_COMPONENT = 'BAMBOO_COMPONENT';

var BAMBOO_INTERNAL_REF = exports.BAMBOO_INTERNAL_REF = 'BAMBOO_INTERNAL_REF';

var mapChildren = exports.mapChildren = function mapChildren(children, func) {
	var list = _react2.default.Children.map(children, function (node, index) {
		return func(node, index);
	}) || [];
	return list.filter(function (node) {
		return node;
	});
};

var mapChildrenForNode = exports.mapChildrenForNode = function mapChildrenForNode(children, func) {
	return mapChildren(children, function (node, index) {
		if (_react2.default.isValidElement(node)) {
			return func(node, index);
		}
		return node;
	});
};

var mapChildrenByType = exports.mapChildrenByType = function mapChildrenByType(children, type, func) {
	var typeList = Array.isArray(type) ? type : [type];

	return mapChildren(children, function (node, index) {
		if (_react2.default.isValidElement(node) && node.type && typeList.includes(node.type[BAMBOO_COMPONENT])) {
			if (func) return func(node, index);
			return node;
		}
		return null;
	});
};

var someChildrenByType = exports.someChildrenByType = function someChildrenByType(children, type, func) {
	var childList = mapChildrenByType(children, type, function (node) {
		return node;
	});
	return childList.some(func);
};

var everyChildrenByType = exports.everyChildrenByType = function everyChildrenByType(children, type, func) {
	var childList = mapChildrenByType(children, type, function (node) {
		return node;
	});
	return childList.every(func);
};

var mapChildrenByNotType = exports.mapChildrenByNotType = function mapChildrenByNotType(children, type, func) {
	var typeList = Array.isArray(type) ? type : [type];

	return mapChildren(children, function (node, index) {
		if (_react2.default.isValidElement(node) && node.type && typeList.includes(node.type[BAMBOO_COMPONENT])) {
			return null;
		}

		if (func) return func(node, index);
		return node;
	});
};