'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.wrapperEventValue = undefined;
exports.addUniqueListener = addUniqueListener;
exports.removeUniqueListener = removeUniqueListener;
exports.isSameSource = isSameSource;

var _envUtil = require('./envUtil');

var events = {};

function addUniqueListener(event, listener) {
	if (!_envUtil.canUseDOM) return;

	if (events[event] === undefined) {
		window.addEventListener(event, function () {
			if (events[event]) {
				events[event].apply(events, arguments);
			}
		});
	}

	events[event] = listener;
}

function removeUniqueListener(event, listener) {
	var currentListener = events[event];

	if (currentListener === listener) {
		events[event] = null;
		return true;
	}

	return false;
}

function isSameSource(event, target) {
	var current = event.target;

	while (current) {
		if (current === target || current._bmbo_ignore) {
			return true;
		}

		current = current.parentElement;
	}

	return false;
}

var wrapperEventValue = exports.wrapperEventValue = function wrapperEventValue(originEvent, target, value) {
	target.value = value; // eslint-disable-line no-param-reassign

	return new Proxy(originEvent, {
		get: function get(tgt, key) {
			if (key === 'target') return target;
			return tgt[key];
		}
	});
};