var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

import { BAMBOO_FORM_RADIO } from './Radio';
import { BAMBOO_FORM_CHECKBOX } from './Checkbox';
import { BAMBOO_FORM_INPUT } from './Input';

import { mapChildrenForNode, BAMBOO_COMPONENT } from '../utils/componentUtil';
import { getValue, updateValue } from '../utils/pathUtil';

var Field = function (_React$Component) {
	_inherits(Field, _React$Component);

	function Field() {
		_classCallCheck(this, Field);

		var _this = _possibleConstructorReturn(this, (Field.__proto__ || Object.getPrototypeOf(Field)).call(this));

		_this.getPath = function () {
			var formPath = _this.context.formPath;
			var name = _this.props.name;


			return [].concat(_toConsumableArray(formPath), [Array.isArray(name) ? name : [name]]);
		};

		_this.getValue = function () {
			var formInstance = _this.context.formInstance;


			if ('value' in _this.props) {
				return _this.props.value;
			}

			if (formInstance) {
				return getValue(formInstance, ['state'].concat(_toConsumableArray(_this.getPath())));
			}
			return _this.state.value;
		};

		_this.getList = function () {
			var _this$context = _this.context,
			    formInstance = _this$context.formInstance,
			    onFormChanged = _this$context.onFormChanged;
			var _this$props = _this.props,
			    title = _this$props.title,
			    children = _this$props.children;

			var myValue = _this.getValue();

			return mapChildrenForNode(children, function (node) {
				var _node$props = node.props,
				    props = _node$props === undefined ? {} : _node$props;


				if (node.type[BAMBOO_COMPONENT] === BAMBOO_FORM_RADIO) {
					// ==================== Radio ====================
					var nodeValue = props.value !== undefined ? props.value : props.children;

					return React.cloneElement(node, {
						'aria-label': props.ariaLabel || title,
						onClick: function onClick() {
							if (props.onClick) props.onClick.apply(props, arguments);

							if (formInstance) {
								formInstance.setState(function (preState) {
									return updateValue(preState, _this.getPath(), function () {
										return nodeValue;
									});
								}, onFormChanged);
							} else {
								_this.setState({ value: nodeValue }, onFormChanged);
							}
						},
						checked: myValue === nodeValue
					});
				} else if (node.type[BAMBOO_COMPONENT] === BAMBOO_FORM_CHECKBOX) {
					// ================== Check Box ==================
					return React.cloneElement(node, {
						'aria-label': props.ariaLabel || title,
						onClick: function onClick() {
							if (props.onClick) props.onClick.apply(props, arguments);

							if (formInstance) {
								formInstance.setState(function (preState) {
									return updateValue(preState, _this.getPath(), function (value) {
										return !value;
									});
								}, onFormChanged);
							} else {
								_this.setState(function (_ref) {
									var value = _ref.value;
									return { value: !value };
								}, onFormChanged);
							}
						},
						checked: myValue
					});
				} else if (node.type[BAMBOO_COMPONENT] === BAMBOO_FORM_INPUT) {
					// ==================== Input ====================
					return React.cloneElement(node, {
						value: props.value !== undefined ? props.value : myValue,
						'aria-label': props.ariaLabel || title,
						onChange: function onChange() {
							for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
								args[_key] = arguments[_key];
							}

							if (props.onChange) props.onChange.apply(props, args);

							var newVal = args[0].target.value;

							if (props.type === 'number') newVal = Number(newVal);

							if (formInstance) {
								formInstance.setState(function (preState) {
									return updateValue(preState, _this.getPath(), function () {
										return newVal;
									});
								}, onFormChanged);
							} else {
								_this.setState({ value: newVal }, onFormChanged);
							}
						}
					});
				}

				// ==================== Rest =====================
				return React.cloneElement(node, {
					value: myValue,
					'aria-label': props.ariaLabel || title,
					onChange: function onChange() {
						for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
							args[_key2] = arguments[_key2];
						}

						if (props.onChange) props.onChange.apply(props, args);

						var newVal = args[0].target.value;

						if (formInstance) {
							formInstance.setState(function (preState) {
								return updateValue(preState, _this.getPath(), function () {
									return newVal;
								});
							}, onFormChanged);
						} else {
							_this.setState({ value: newVal }, onFormChanged);
						}
					}
				});
			});
		};

		_this.state = {
			value: null
		};
		return _this;
	}

	_createClass(Field, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			var defaultValue = this.props.defaultValue;

			this.setState({ value: defaultValue });
		}
	}, {
		key: 'render',
		value: function render() {
			var _props = this.props,
			    name = _props.name,
			    title = _props.title,
			    className = _props.className,
			    props = _objectWithoutProperties(_props, ['name', 'title', 'className']);

			delete props.value;
			delete props.defaultValue;

			var list = this.getList();

			return React.createElement(
				'div',
				_extends({ className: classNames('bmbp-form-field', className) }, props),
				title !== undefined && React.createElement(
					'label',
					{ htmlFor: name },
					title
				),
				list
			);
		}
	}]);

	return Field;
}(React.Component);

Field.propTypes = {
	children: PropTypes.node,
	className: PropTypes.string,

	name: PropTypes.string,
	title: PropTypes.node,
	value: PropTypes.node,
	defaultValue: PropTypes.node
};

Field.contextTypes = {
	formInstance: PropTypes.object,
	formPath: PropTypes.array,
	onFormChanged: PropTypes.func
};

export default Field;