var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

var REGEX = /^(\d+)\/(\d+)$/;

function getMatch(numerator, denominator, total) {
	var multiple = total / denominator;
	if (multiple !== Math.floor(multiple)) return null;

	return numerator * multiple + '-' + total;
}

function getName(prefix, val) {
	if (!val) return '';
	if (val === '0') return prefix + '0';

	var myVal = val === '1' ? '1/1' : val;
	var match = Array.isArray(myVal) ? [null].concat(_toConsumableArray(myVal)) : myVal.match(REGEX);

	// Return full row if not match
	if (!match) return '10-10';

	var numerator = Number(match[1]);
	var denominator = Number(match[2]);

	var cal = getMatch(numerator, denominator, 24) || getMatch(numerator, denominator, 10);
	return '' + prefix + cal;
}

var Col = function Col(props, context) {
	var xs = props.xs,
	    sm = props.sm,
	    md = props.md,
	    lg = props.lg,
	    xl = props.xl,
	    xsOffset = props.xsOffset,
	    smOffset = props.smOffset,
	    mdOffset = props.mdOffset,
	    lgOffset = props.lgOffset,
	    xlOffset = props.xlOffset,
	    children = props.children,
	    className = props.className,
	    rest = _objectWithoutProperties(props, ['xs', 'sm', 'md', 'lg', 'xl', 'xsOffset', 'smOffset', 'mdOffset', 'lgOffset', 'xlOffset', 'children', 'className']);

	var rowGutter = context.rowGutter;


	var xsClassName = getName('bmbo-col-xs-', xs);
	var smClassName = getName('bmbo-col-sm-', sm);
	var mdClassName = getName('bmbo-col-md-', md);
	var lgClassName = getName('bmbo-col-lg-', lg);
	var xlClassName = getName('bmbo-col-xl-', xl);

	var xsOffsetClassName = getName('bmbo-col-xs-offset-', xsOffset);
	var smOffsetClassName = getName('bmbo-col-sm-offset-', smOffset);
	var mdOffsetClassName = getName('bmbo-col-md-offset-', mdOffset);
	var lgOffsetClassName = getName('bmbo-col-lg-offset-', lgOffset);
	var xlOffsetClassName = getName('bmbo-col-xl-offset-', xlOffset);

	var hg = rowGutter / 2;

	return React.createElement(
		'div',
		_extends({
			className: classNames('bmbo-col', xsClassName, smClassName, mdClassName, lgClassName, xlClassName, xsOffsetClassName, smOffsetClassName, mdOffsetClassName, lgOffsetClassName, xlOffsetClassName, className),
			style: { paddingLeft: hg + 'px', paddingRight: hg + 'px' }
		}, rest),
		children
	);
};

Col.propTypes = {
	className: PropTypes.string,

	xs: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	sm: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	md: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	lg: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	xl: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),

	xsOffset: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	smOffset: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	mdOffset: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	lgOffset: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	xlOffset: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),

	children: PropTypes.node
};

Col.contextTypes = {
	rowGutter: PropTypes.number
};

export default Col;