var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

import { ANIMATE_STATUS_NONE, ANIMATE_STATUS_SHOWING, ANIMATE_STATUS_SHOWN, ANIMATE_STATUS_HIDING, createPortal, enableWinScrollBar, disableWinScrollBar } from '../utils/uiUtil';
import { canUseDOM } from '../utils/envUtil';
import { mapChildrenByType, mapChildrenByNotType } from '../utils/componentUtil';
import Sequence from '../utils/Sequence';

import ModalTitle, { BAMBOO_MODAL_TITLE } from './ModalTitle';
import ModalContent, { BAMBOO_MODAL_CONTENT } from './ModalContent';
import ModalFooter, { BAMBOO_MODAL_FOOTER } from './ModalFooter';
import ModalPreface, { BAMBOO_MODAL_PREFACE } from './ModalPreface';
import ModalLoading, { BAMBOO_MODAL_LOADING } from './ModalLoading';

import Button from '../Button';

var modalList = [];

// ========================================================
// =                          UI                          =
// ========================================================
function refreshWinScrollBar() {
	if (!canUseDOM) return;

	var hasActiveModal = modalList.some(function (_ref) {
		var _ref$state = _ref.state;
		_ref$state = _ref$state === undefined ? {} : _ref$state;
		var animateStatus = _ref$state.animateStatus;
		return animateStatus !== ANIMATE_STATUS_NONE;
	});

	if (hasActiveModal) {
		disableWinScrollBar('Modal');
	} else {
		enableWinScrollBar('Modal');
	}
}

// ========================================================
// =                         Modal                        =
// ========================================================

var Modal = function (_React$Component) {
	_inherits(Modal, _React$Component);

	function Modal() {
		_classCallCheck(this, Modal);

		var _this = _possibleConstructorReturn(this, (Modal.__proto__ || Object.getPrototypeOf(Modal)).call(this));

		_this.onBackgroundClick = function () {
			var noMaskClose = _this.props.noMaskClose;

			if (noMaskClose) return;

			// Put close request in next frame
			_this.closeSeq.next(function () {
				var onClose = _this.props.onClose;

				if (onClose) onClose();
			});
		};

		_this.onHolderClick = function () {
			// Occupy the close request if is holder click
			_this.closeSeq.next(function () {}, { priority: 1 });
		};

		_this.onTransitionEnd = function (event) {
			var onClosed = _this.props.onClosed;
			var animateStatus = _this.state.animateStatus;


			if (event.target === _this.$holder && animateStatus === ANIMATE_STATUS_HIDING) {
				if (onClosed) onClosed(event);

				_this.seq.next(function () {
					_this.setState({
						animateStatus: ANIMATE_STATUS_NONE
					}, refreshWinScrollBar);
				});
			}
		};

		_this.setHolderRef = function (ele) {
			_this.$holder = ele;
		};

		_this.getTitle = function () {
			var _this$props = _this.props,
			    children = _this$props.children,
			    title = _this$props.title;


			if (title) return React.createElement(
				ModalTitle,
				null,
				title
			);
			return mapChildrenByType(children, BAMBOO_MODAL_TITLE);
		};

		_this.getBody = function () {
			var _this$props2 = _this.props,
			    children = _this$props2.children,
			    content = _this$props2.content;


			if (content) return React.createElement(
				ModalContent,
				null,
				content
			);

			var $children = mapChildrenByType(children, BAMBOO_MODAL_CONTENT);
			if ($children.length) return $children;

			return React.createElement(
				ModalContent,
				null,
				mapChildrenByNotType(children, [BAMBOO_MODAL_TITLE, BAMBOO_MODAL_FOOTER, BAMBOO_MODAL_PREFACE, BAMBOO_MODAL_LOADING])
			);
		};

		_this.getFooter = function () {
			var _this$props3 = _this.props,
			    children = _this$props3.children,
			    footer = _this$props3.footer,
			    onClose = _this$props3.onClose,
			    onConfirm = _this$props3.onConfirm,
			    _this$props3$closeTex = _this$props3.closeText,
			    closeText = _this$props3$closeTex === undefined ? 'Close' : _this$props3$closeTex,
			    _this$props3$confirmT = _this$props3.confirmText,
			    confirmText = _this$props3$confirmT === undefined ? 'Confirm' : _this$props3$confirmT,
			    lock = _this$props3.lock;


			if (footer) return React.createElement(
				ModalFooter,
				null,
				footer
			);

			var $footer = mapChildrenByType(children, BAMBOO_MODAL_FOOTER);
			if ($footer.length) return $footer;

			return React.createElement(
				ModalFooter,
				null,
				React.createElement(
					Button,
					{ type: 'weak', onClick: onClose, disabled: lock, transparent: true },
					closeText
				),
				onConfirm && React.createElement(
					Button,
					{ onClick: onConfirm, disabled: lock },
					confirmText
				)
			);
		};

		_this.getLoading = function () {
			var children = _this.props.children;

			var $loading = mapChildrenByType(children, BAMBOO_MODAL_LOADING);

			if ($loading.length) return $loading;

			return React.createElement(ModalLoading, { defaultView: true });
		};

		_this.getPreface = function () {
			var _this$props4 = _this.props,
			    children = _this$props4.children,
			    visible = _this$props4.visible;

			return mapChildrenByType(children, BAMBOO_MODAL_PREFACE, function (node) {
				return React.cloneElement(node, { visible: visible });
			});
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			// Process when visibility changed
			if (!prevProps.visible !== !nextProps.visible) {
				if (nextProps.visible) {
					// Show modal
					_this.seq.next(function () {
						_this.setState({
							animateStatus: ANIMATE_STATUS_SHOWING
						}, refreshWinScrollBar);
					}).next(function () {
						_this.setState({ animateStatus: ANIMATE_STATUS_SHOWN });
					});
				} else {
					// Hide modal
					_this.seq.next(function () {
						_this.setState({
							animateStatus: ANIMATE_STATUS_HIDING
						});
					}).next(function () {
						// Clean up when browser not support transitionEnd event
						var onClosed = _this.props.onClosed;

						_this.setState({ animateStatus: ANIMATE_STATUS_NONE }, refreshWinScrollBar);

						if (onClosed) onClosed();
					}, { delay: 750 });
				}
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE
		};

		_this.seq = new Sequence();
		_this.closeSeq = new Sequence();
		return _this;
	}

	_createClass(Modal, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			modalList.push(this);
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			var _this2 = this;

			this.seq.destroy();
			modalList = modalList.filter(function (mdl) {
				return mdl !== _this2;
			});
			refreshWinScrollBar();
		}
	}, {
		key: 'render',
		value: function render() {
			var _props = this.props,
			    size = _props.size,
			    type = _props.type,
			    loading = _props.loading,
			    className = _props.className,
			    props = _objectWithoutProperties(_props, ['size', 'type', 'loading', 'className']);

			var animateStatus = this.state.animateStatus;


			if (animateStatus === ANIMATE_STATUS_NONE) return null;

			var $title = this.getTitle();
			var $body = this.getBody();
			var $footer = this.getFooter();
			var $preface = this.getPreface();
			var $loading = this.getLoading();

			var $modal = React.createElement(
				'div',
				{ className: 'bmbo-dialog' },
				$title,
				$body,
				$footer,
				$loading
			);

			delete props.visible;
			delete props.lock;
			delete props.title;
			delete props.content;
			delete props.footer;
			delete props.onClose;
			delete props.onClosed;
			delete props.onConfirm;
			delete props.closeText;
			delete props.confirmText;
			delete props.noMaskClose;

			return createPortal(React.createElement(
				'div',
				_extends({}, props, {
					className: classNames('bmbo-modal', {
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
						'bmbo-hidding': animateStatus === ANIMATE_STATUS_HIDING,
						'bmbo-loading': loading,
						className: className
					}, 'bmbo-modal-' + (size || 'md'), type && 'bmbo-' + type),
					role: 'presentation',
					onClick: this.onBackgroundClick
				}),
				React.createElement(
					'div',
					{
						className: 'bmbo-dialog-holder',
						role: 'presentation',
						onClick: this.onHolderClick,
						ref: this.setHolderRef,
						onTransitionEnd: this.onTransitionEnd
					},
					$preface,
					$modal
				)
			));
		}
	}]);

	return Modal;
}(React.Component);

Modal.propTypes = {
	visible: PropTypes.bool,
	children: PropTypes.node,
	className: PropTypes.string,
	size: PropTypes.string,
	type: PropTypes.string,
	loading: PropTypes.bool,
	lock: PropTypes.bool,

	title: PropTypes.node,
	content: PropTypes.node,
	footer: PropTypes.node,

	onClose: PropTypes.func,
	onClosed: PropTypes.func,
	onConfirm: PropTypes.func,

	closeText: PropTypes.func,
	confirmText: PropTypes.func,

	noMaskClose: PropTypes.bool
};

Modal.Title = ModalTitle;
Modal.Content = ModalContent;
Modal.Footer = ModalFooter;
Modal.Preface = ModalPreface;
Modal.Loading = ModalLoading;

export default Modal;