var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

import { ANIMATE_STATUS_NONE, ANIMATE_STATUS_INIT, ANIMATE_STATUS_SHOWING, ANIMATE_STATUS_SHOWN, ANIMATE_STATUS_HIDING, createPortal, getEnablePosition } from '../utils/uiUtil';
import Sequence from '../utils/Sequence';

var DRILL_SPEED = 4 / 5;

var NavList = function (_React$Component) {
	_inherits(NavList, _React$Component);

	function NavList() {
		_classCallCheck(this, NavList);

		var _this = _possibleConstructorReturn(this, (NavList.__proto__ || Object.getPrototypeOf(NavList)).call(this));

		_this.onTransitionEnd = function (event) {
			if (_this.state.animateStatus === ANIMATE_STATUS_HIDING && event.target === _this.$list) {
				_this.seq.next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				});
			}
		};

		_this.setListRef = function (ele) {
			var setRef = _this.props.setRef;

			_this.$list = ele;
			setRef(ele);
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (_this.context.navInline) {
				if (!prevProps.open === !nextProps.open) return;
				_this.doDrillAnimation(nextProps);
			} else {
				if (!prevProps.visible === !nextProps.visible) return;
				_this.doFloatAnimation(nextProps);
			}
		};

		_this.doDrillAnimation = function (props) {
			if (props.open) {
				_this.seq.next(function () {
					_this.setState({
						height: 0
					});
				}).next(function () {
					var clientHeight = _this.$list.clientHeight;
					var scrollHeight = _this.$list.scrollHeight;

					var currentHeight = Math.ceil(clientHeight * DRILL_SPEED + scrollHeight * (1 - DRILL_SPEED));
					if (scrollHeight - currentHeight <= 1) {
						currentHeight = scrollHeight;
						_this.setState({ height: null });
						return false;
					}

					_this.setState({ height: currentHeight });

					return null;
				}, {
					loop: true,
					frame: 1
				});
			} else {
				_this.seq.next(function () {
					_this.setState({
						height: null
					});
				}).next(function () {
					var clientHeight = _this.$list.clientHeight;

					var currentHeight = Math.floor(clientHeight * DRILL_SPEED);
					if (clientHeight <= 1) {
						_this.setState({ height: 0 });
						return false;
					}

					_this.setState({ height: currentHeight });

					return null;
				}, {
					loop: true,
					frame: 1
				});
			}
		};

		_this.doFloatAnimation = function (props) {
			if (props.visible) {
				_this.seq.next(function () {
					_this.setState({
						animateStatus: ANIMATE_STATUS_INIT,
						x: 0,
						y: 0
					});
				}).next(function () {
					var rect = _this.props.rect;
					var navVertical = _this.context.navVertical;

					var listRect = _this.$list.getBoundingClientRect();

					var pos = navVertical ? 'r' : 'b';

					// Add offset to let nav list match with parent
					var superRect = {
						left: rect.left,
						top: rect.top,
						width: rect.width,
						height: rect.height
					};
					if (navVertical) {
						superRect.top -= 1;
						superRect.height += 2;
					} else {
						superRect.left -= 1;
						superRect.width += 2;
					}

					_this.setState(_extends({}, getEnablePosition(superRect, listRect, pos), {
						animateStatus: ANIMATE_STATUS_SHOWING
					}));
				}).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_SHOWN });
				});
			} else {
				_this.seq.next(function () {
					var animateStatus = _this.state.animateStatus;

					if (animateStatus === ANIMATE_STATUS_NONE || animateStatus === ANIMATE_STATUS_INIT) {
						return false;
					}
					_this.setState({ animateStatus: ANIMATE_STATUS_HIDING });
					return null;
				}).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				}, { delay: 1000 });
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE,
			x: 0,
			y: 0,
			height: 0
		};
		_this.seq = new Sequence();
		return _this;
	}

	_createClass(NavList, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render() {
			var _props = this.props,
			    children = _props.children,
			    rect = _props.rect;
			var _context = this.context,
			    navType = _context.navType,
			    navVertical = _context.navVertical,
			    navInline = _context.navInline;
			var _state = this.state,
			    animateStatus = _state.animateStatus,
			    x = _state.x,
			    y = _state.y,
			    height = _state.height;


			if (!navVertical || !navInline) {
				// Float display

				if (animateStatus === ANIMATE_STATUS_NONE) return null;

				return createPortal(React.createElement(
					'div',
					{
						className: classNames('bmbo-nav-list', {
							'bmbo-hidden': animateStatus === ANIMATE_STATUS_INIT,
							'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
							'bmbo-hiding': animateStatus === ANIMATE_STATUS_HIDING
						}, 'bmbo-' + (navType || 'lead')),
						ref: this.setListRef,
						style: { left: x + 'px', top: y + 'px', minWidth: rect.width + 'px' },
						onTransitionEnd: this.onTransitionEnd
					},
					children
				));
			}

			// Inline display
			var style = {};
			if (height !== null) {
				style.height = height + 'px';
			}

			return React.createElement(
				'ul',
				{
					className: 'bmbo-nav-list',
					style: style,
					ref: this.setListRef
				},
				children
			);
		}
	}]);

	return NavList;
}(React.Component);

NavList.propTypes = {
	visible: PropTypes.bool,
	open: PropTypes.bool,
	rect: PropTypes.object,

	setRef: PropTypes.func,

	children: PropTypes.node
};

NavList.contextTypes = {
	navType: PropTypes.string,
	navVertical: PropTypes.bool,
	navInline: PropTypes.bool
};

export default NavList;