var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

import { BambooDate } from '../utils/dateUtils';
import { wrapperEventValue } from '../utils/eventUtil';
import { canUseDOM } from '../utils/envUtil';
import { ANIMATE_STATUS_HIDING, ANIMATE_STATUS_NONE, ANIMATE_STATUS_SHOWING, ANIMATE_STATUS_SHOWN } from '../utils/uiUtil';
import Sequence from '../utils/Sequence';

import DatePanel from './DatePanel';

var DATE_LIST = ['SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT'];

var DatePicker = function (_React$Component) {
	_inherits(DatePicker, _React$Component);

	function DatePicker() {
		_classCallCheck(this, DatePicker);

		var _this = _possibleConstructorReturn(this, (DatePicker.__proto__ || Object.getPrototypeOf(DatePicker)).call(this));

		_this.onSwitchDateView = function (nextDate) {
			if (_this.state.animateStatus !== ANIMATE_STATUS_NONE) return;
			_this.switchDateView(nextDate);
		};

		_this.onTransitionEnd = function (event) {
			var _this$state = _this.state,
			    animateStatus = _this$state.animateStatus,
			    nextViewDate = _this$state.nextViewDate;


			if (event.target === _this.$nextView && animateStatus === ANIMATE_STATUS_SHOWN) {
				_this.seq.next(function () {
					_this.setState({
						animateStatus: ANIMATE_STATUS_NONE,
						viewDate: nextViewDate,
						nextViewDate: null
					});
				});
			}
		};

		_this.setRef = function (ele) {
			_this.$ele = ele;
		};

		_this.setViewRef = function (ele) {
			_this.$view = ele;
		};

		_this.setNextViewRef = function (ele) {
			_this.$nextView = ele;
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (prevProps.value === nextProps.value) return;

			var viewDate = _this.state.viewDate;

			var nextDate = new BambooDate(nextProps.value);

			if (!prevProps.value || viewDate && viewDate.format('YYYY-MM') === nextDate.format('YYYY-MM')) {
				_this.setState({
					viewDate: nextDate
				});
			} else {
				var prevDate = new BambooDate(prevProps.value);
				if (prevDate.format('YYYY-MM') === nextDate.format('YYYY-MM')) return;

				_this.switchDateView(nextDate);
			}
		};

		_this.switchDateView = function (nextDate) {
			_this.seq.next(function (done) {
				_this.setState({
					animateStatus: ANIMATE_STATUS_SHOWING,
					nextViewDate: nextDate
				}, done);
			}).next(function (done) {
				_this.setState({
					animateStatus: ANIMATE_STATUS_SHOWN
				}, done);
			});
		};

		_this.triggerChangeEvent = function (event, date) {
			var onChange = _this.props.onChange;

			if (onChange) {
				onChange(wrapperEventValue(event, _this.$ele, date.format('YYYY-MM-DD')));
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE,

			viewDate: null, // This is used for show current year-month panel
			nextViewDate: null // This is used for animation panel
		};

		_this.seq = new Sequence();
		return _this;
	}

	_createClass(DatePicker, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render() {
			var _this2 = this;

			var _state = this.state,
			    animateStatus = _state.animateStatus,
			    viewDate = _state.viewDate,
			    nextViewDate = _state.nextViewDate;

			if (!viewDate) return null;

			var _props = this.props,
			    value = _props.value,
			    className = _props.className,
			    props = _objectWithoutProperties(_props, ['value', 'className']);

			delete props.onChange;

			var date = new BambooDate(value);

			var isShowing = animateStatus === ANIMATE_STATUS_SHOWING;
			var isShown = animateStatus === ANIMATE_STATUS_SHOWN;

			// Current view
			var startDate = viewDate.clone().setDate(1);
			var endDate = viewDate.clone().setDate(1);
			startDate.setDay(0);
			endDate.setMonth(endDate.getMonth() + 1).setDate(0).setDay(6);
			var $view = React.createElement(DatePanel, {
				date: date,
				viewDate: viewDate,
				startDate: startDate,
				endDate: endDate,
				triggerChangeEvent: this.triggerChangeEvent,
				setRef: this.setViewRef,
				className: classNames({
					'bmbo-left': isShown && nextViewDate.getTime() > viewDate.getTime(),
					'bmbo-right': isShown && nextViewDate.getTime() < viewDate.getTime()
				})
			});

			// Next view
			var $nextView = void 0;
			if (animateStatus !== ANIMATE_STATUS_NONE) {
				var nextStartDate = nextViewDate.clone().setDate(1);
				var nextEndDate = nextViewDate.clone().setDate(1);
				nextStartDate.setDay(0);
				nextEndDate.setMonth(nextEndDate.getMonth() + 1).setDate(0).setDay(6);
				$nextView = React.createElement(DatePanel, {
					date: date,
					viewDate: nextViewDate,
					startDate: nextStartDate,
					endDate: nextEndDate,
					setRef: this.setNextViewRef,
					onTransitionEnd: this.onTransitionEnd,
					className: classNames('bmbo-target', {
						'bmbo-left': isShowing && nextViewDate.getTime() < viewDate.getTime(),
						'bmbo-right': isShowing && nextViewDate.getTime() > viewDate.getTime()
					})
				});
			}

			// Content style
			var contentStyle = {};
			if (canUseDOM) {
				if (isShowing && this.$view) {
					contentStyle.height = this.$view.clientHeight + 'px';
				} else if (!isShowing && this.$nextView) {
					contentStyle.height = this.$nextView.clientHeight + 'px';
				}
			}

			return React.createElement(
				'div',
				_extends({
					className: classNames('bmbo-date-picker', className),
					ref: this.setRef
				}, props),
				React.createElement(
					'div',
					{ className: 'bmbo-date-picker-header' },
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Prev Year',
						className: 'bmbo-date-picker-operation bmbo-year-prev',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setFullYear(viewDate.getFullYear() - 1));
						}
					}),
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Prev Month',
						className: 'bmbo-date-picker-operation bmbo-month-prev',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setMonth(viewDate.getMonth() - 1));
						}
					}),
					viewDate.format('YYYY-MM'),
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Next Year',
						className: 'bmbo-date-picker-operation bmbo-year-next',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setFullYear(viewDate.getFullYear() + 1));
						}
					}),
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Next Month',
						className: 'bmbo-date-picker-operation bmbo-month-next',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setMonth(viewDate.getMonth() + 1));
						}
					})
				),
				React.createElement(
					'ul',
					{ className: 'bmbo-date-picker-title' },
					DATE_LIST.map(function (str) {
						return React.createElement(
							'li',
							{ key: str },
							React.createElement(
								'strong',
								null,
								str
							)
						);
					})
				),
				React.createElement(
					'div',
					{
						className: classNames('bmbo-date-picker-content', {
							'bmbo-none': animateStatus === ANIMATE_STATUS_NONE,
							'bmbo-showing': isShowing
						}),
						style: contentStyle
					},
					$view,
					$nextView
				)
			);
		}
	}]);

	return DatePicker;
}(React.Component);

DatePicker.propTypes = {
	value: PropTypes.oneOfType([PropTypes.string, PropTypes.number, PropTypes.object]),
	className: PropTypes.string,
	onChange: PropTypes.func
};

export default DatePicker;