import React from 'react';

export var BAMBOO_COMPONENT = 'BAMBOO_COMPONENT';

export var BAMBOO_INTERNAL_REF = 'BAMBOO_INTERNAL_REF';

export var mapChildren = function mapChildren(children, func) {
	var list = React.Children.map(children, function (node, index) {
		return func(node, index);
	}) || [];
	return list.filter(function (node) {
		return node;
	});
};

export var mapChildrenForNode = function mapChildrenForNode(children, func) {
	return mapChildren(children, function (node, index) {
		if (React.isValidElement(node)) {
			return func(node, index);
		}
		return node;
	});
};

export var mapChildrenByType = function mapChildrenByType(children, type, func) {
	var typeList = Array.isArray(type) ? type : [type];

	return mapChildren(children, function (node, index) {
		if (React.isValidElement(node) && node.type && typeList.includes(node.type[BAMBOO_COMPONENT])) {
			if (func) return func(node, index);
			return node;
		}
		return null;
	});
};

export var someChildrenByType = function someChildrenByType(children, type, func) {
	var childList = mapChildrenByType(children, type, function (node) {
		return node;
	});
	return childList.some(func);
};

export var everyChildrenByType = function everyChildrenByType(children, type, func) {
	var childList = mapChildrenByType(children, type, function (node) {
		return node;
	});
	return childList.every(func);
};

export var mapChildrenByNotType = function mapChildrenByNotType(children, type, func) {
	var typeList = Array.isArray(type) ? type : [type];

	return mapChildren(children, function (node, index) {
		if (React.isValidElement(node) && node.type && typeList.includes(node.type[BAMBOO_COMPONENT])) {
			return null;
		}

		if (func) return func(node, index);
		return node;
	});
};