(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('react-dom')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react', 'react-dom'], factory) :
	(factory((global.BambooUI = {}),global.React,global.ReactDom));
}(this, (function (exports,React,reactDom) { 'use strict';

React = React && React.hasOwnProperty('default') ? React['default'] : React;

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */

function makeEmptyFunction(arg) {
  return function () {
    return arg;
  };
}

/**
 * This function accepts and discards inputs; it has no side effects. This is
 * primarily useful idiomatically for overridable function endpoints which
 * always need to be callable, since JS lacks a null-call idiom ala Cocoa.
 */
var emptyFunction = function emptyFunction() {};

emptyFunction.thatReturns = makeEmptyFunction;
emptyFunction.thatReturnsFalse = makeEmptyFunction(false);
emptyFunction.thatReturnsTrue = makeEmptyFunction(true);
emptyFunction.thatReturnsNull = makeEmptyFunction(null);
emptyFunction.thatReturnsThis = function () {
  return this;
};
emptyFunction.thatReturnsArgument = function (arg) {
  return arg;
};

var emptyFunction_1 = emptyFunction;

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 */

/**
 * Use invariant() to assert state which your program assumes to be true.
 *
 * Provide sprintf-style format (only %s is supported) and arguments
 * to provide information about what broke and what you were
 * expecting.
 *
 * The invariant message will be stripped in production, but the invariant
 * will remain to ensure logic does not differ in production.
 */

var validateFormat = function validateFormat(format) {};

{
  validateFormat = function validateFormat(format) {
    if (format === undefined) {
      throw new Error('invariant requires an error message argument');
    }
  };
}

function invariant(condition, format, a, b, c, d, e, f) {
  validateFormat(format);

  if (!condition) {
    var error;
    if (format === undefined) {
      error = new Error('Minified exception occurred; use the non-minified dev environment ' + 'for the full error message and additional helpful warnings.');
    } else {
      var args = [a, b, c, d, e, f];
      var argIndex = 0;
      error = new Error(format.replace(/%s/g, function () {
        return args[argIndex++];
      }));
      error.name = 'Invariant Violation';
    }

    error.framesToPop = 1; // we don't care about invariant's own frame
    throw error;
  }
}

var invariant_1 = invariant;

/**
 * Similar to invariant but only logs a warning if the condition is not met.
 * This can be used to log issues in development environments in critical
 * paths. Removing the logging code for production environments will keep the
 * same logic and follow the same code paths.
 */

var warning = emptyFunction_1;

{
  var printWarning = function printWarning(format) {
    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var argIndex = 0;
    var message = 'Warning: ' + format.replace(/%s/g, function () {
      return args[argIndex++];
    });
    if (typeof console !== 'undefined') {
      console.error(message);
    }
    try {
      // --- Welcome to debugging React ---
      // This error was thrown as a convenience so that you can use this stack
      // to find the callsite that caused this warning to fire.
      throw new Error(message);
    } catch (x) {}
  };

  warning = function warning(condition, format) {
    if (format === undefined) {
      throw new Error('`warning(condition, format, ...args)` requires a warning ' + 'message argument');
    }

    if (format.indexOf('Failed Composite propType: ') === 0) {
      return; // Ignore CompositeComponent proptype check.
    }

    if (!condition) {
      for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
        args[_key2 - 2] = arguments[_key2];
      }

      printWarning.apply(undefined, [format].concat(args));
    }
  };
}

var warning_1 = warning;

/*
object-assign
(c) Sindre Sorhus
@license MIT
*/

/* eslint-disable no-unused-vars */
var getOwnPropertySymbols = Object.getOwnPropertySymbols;
var hasOwnProperty = Object.prototype.hasOwnProperty;
var propIsEnumerable = Object.prototype.propertyIsEnumerable;

function toObject(val) {
	if (val === null || val === undefined) {
		throw new TypeError('Object.assign cannot be called with null or undefined');
	}

	return Object(val);
}

function shouldUseNative() {
	try {
		if (!Object.assign) {
			return false;
		}

		// Detect buggy property enumeration order in older V8 versions.

		// https://bugs.chromium.org/p/v8/issues/detail?id=4118
		var test1 = new String('abc');  // eslint-disable-line no-new-wrappers
		test1[5] = 'de';
		if (Object.getOwnPropertyNames(test1)[0] === '5') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test2 = {};
		for (var i = 0; i < 10; i++) {
			test2['_' + String.fromCharCode(i)] = i;
		}
		var order2 = Object.getOwnPropertyNames(test2).map(function (n) {
			return test2[n];
		});
		if (order2.join('') !== '0123456789') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test3 = {};
		'abcdefghijklmnopqrst'.split('').forEach(function (letter) {
			test3[letter] = letter;
		});
		if (Object.keys(Object.assign({}, test3)).join('') !==
				'abcdefghijklmnopqrst') {
			return false;
		}

		return true;
	} catch (err) {
		// We don't expect any of the above to throw, but better to be safe.
		return false;
	}
}

var objectAssign = shouldUseNative() ? Object.assign : function (target, source) {
	var from;
	var to = toObject(target);
	var symbols;

	for (var s = 1; s < arguments.length; s++) {
		from = Object(arguments[s]);

		for (var key in from) {
			if (hasOwnProperty.call(from, key)) {
				to[key] = from[key];
			}
		}

		if (getOwnPropertySymbols) {
			symbols = getOwnPropertySymbols(from);
			for (var i = 0; i < symbols.length; i++) {
				if (propIsEnumerable.call(from, symbols[i])) {
					to[symbols[i]] = from[symbols[i]];
				}
			}
		}
	}

	return to;
};

/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var ReactPropTypesSecret = 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED';

var ReactPropTypesSecret_1 = ReactPropTypesSecret;

{
  var invariant$1 = invariant_1;
  var warning$1 = warning_1;
  var ReactPropTypesSecret$1 = ReactPropTypesSecret_1;
  var loggedTypeFailures = {};
}

/**
 * Assert that the values match with the type specs.
 * Error messages are memorized and will only be shown once.
 *
 * @param {object} typeSpecs Map of name to a ReactPropType
 * @param {object} values Runtime values that need to be type-checked
 * @param {string} location e.g. "prop", "context", "child context"
 * @param {string} componentName Name of the component for error messages.
 * @param {?Function} getStack Returns the component stack.
 * @private
 */
function checkPropTypes(typeSpecs, values, location, componentName, getStack) {
  {
    for (var typeSpecName in typeSpecs) {
      if (typeSpecs.hasOwnProperty(typeSpecName)) {
        var error;
        // Prop type validation may throw. In case they do, we don't want to
        // fail the render phase where it didn't fail before. So we log it.
        // After these have been cleaned up, we'll let them throw.
        try {
          // This is intentionally an invariant that gets caught. It's the same
          // behavior as without this statement except with a better message.
          invariant$1(typeof typeSpecs[typeSpecName] === 'function', '%s: %s type `%s` is invalid; it must be a function, usually from ' + 'the `prop-types` package, but received `%s`.', componentName || 'React class', location, typeSpecName, typeof typeSpecs[typeSpecName]);
          error = typeSpecs[typeSpecName](values, typeSpecName, componentName, location, null, ReactPropTypesSecret$1);
        } catch (ex) {
          error = ex;
        }
        warning$1(!error || error instanceof Error, '%s: type specification of %s `%s` is invalid; the type checker ' + 'function must return `null` or an `Error` but returned a %s. ' + 'You may have forgotten to pass an argument to the type checker ' + 'creator (arrayOf, instanceOf, objectOf, oneOf, oneOfType, and ' + 'shape all require an argument).', componentName || 'React class', location, typeSpecName, typeof error);
        if (error instanceof Error && !(error.message in loggedTypeFailures)) {
          // Only monitor this failure once because there tends to be a lot of the
          // same error.
          loggedTypeFailures[error.message] = true;

          var stack = getStack ? getStack() : '';

          warning$1(false, 'Failed %s type: %s%s', location, error.message, stack != null ? stack : '');
        }
      }
    }
  }
}

var checkPropTypes_1 = checkPropTypes;

var factoryWithTypeCheckers = function(isValidElement, throwOnDirectAccess) {
  /* global Symbol */
  var ITERATOR_SYMBOL = typeof Symbol === 'function' && Symbol.iterator;
  var FAUX_ITERATOR_SYMBOL = '@@iterator'; // Before Symbol spec.

  /**
   * Returns the iterator method function contained on the iterable object.
   *
   * Be sure to invoke the function with the iterable as context:
   *
   *     var iteratorFn = getIteratorFn(myIterable);
   *     if (iteratorFn) {
   *       var iterator = iteratorFn.call(myIterable);
   *       ...
   *     }
   *
   * @param {?object} maybeIterable
   * @return {?function}
   */
  function getIteratorFn(maybeIterable) {
    var iteratorFn = maybeIterable && (ITERATOR_SYMBOL && maybeIterable[ITERATOR_SYMBOL] || maybeIterable[FAUX_ITERATOR_SYMBOL]);
    if (typeof iteratorFn === 'function') {
      return iteratorFn;
    }
  }

  /**
   * Collection of methods that allow declaration and validation of props that are
   * supplied to React components. Example usage:
   *
   *   var Props = require('ReactPropTypes');
   *   var MyArticle = React.createClass({
   *     propTypes: {
   *       // An optional string prop named "description".
   *       description: Props.string,
   *
   *       // A required enum prop named "category".
   *       category: Props.oneOf(['News','Photos']).isRequired,
   *
   *       // A prop named "dialog" that requires an instance of Dialog.
   *       dialog: Props.instanceOf(Dialog).isRequired
   *     },
   *     render: function() { ... }
   *   });
   *
   * A more formal specification of how these methods are used:
   *
   *   type := array|bool|func|object|number|string|oneOf([...])|instanceOf(...)
   *   decl := ReactPropTypes.{type}(.isRequired)?
   *
   * Each and every declaration produces a function with the same signature. This
   * allows the creation of custom validation functions. For example:
   *
   *  var MyLink = React.createClass({
   *    propTypes: {
   *      // An optional string or URI prop named "href".
   *      href: function(props, propName, componentName) {
   *        var propValue = props[propName];
   *        if (propValue != null && typeof propValue !== 'string' &&
   *            !(propValue instanceof URI)) {
   *          return new Error(
   *            'Expected a string or an URI for ' + propName + ' in ' +
   *            componentName
   *          );
   *        }
   *      }
   *    },
   *    render: function() {...}
   *  });
   *
   * @internal
   */

  var ANONYMOUS = '<<anonymous>>';

  // Important!
  // Keep this list in sync with production version in `./factoryWithThrowingShims.js`.
  var ReactPropTypes = {
    array: createPrimitiveTypeChecker('array'),
    bool: createPrimitiveTypeChecker('boolean'),
    func: createPrimitiveTypeChecker('function'),
    number: createPrimitiveTypeChecker('number'),
    object: createPrimitiveTypeChecker('object'),
    string: createPrimitiveTypeChecker('string'),
    symbol: createPrimitiveTypeChecker('symbol'),

    any: createAnyTypeChecker(),
    arrayOf: createArrayOfTypeChecker,
    element: createElementTypeChecker(),
    instanceOf: createInstanceTypeChecker,
    node: createNodeChecker(),
    objectOf: createObjectOfTypeChecker,
    oneOf: createEnumTypeChecker,
    oneOfType: createUnionTypeChecker,
    shape: createShapeTypeChecker,
    exact: createStrictShapeTypeChecker,
  };

  /**
   * inlined Object.is polyfill to avoid requiring consumers ship their own
   * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/is
   */
  /*eslint-disable no-self-compare*/
  function is(x, y) {
    // SameValue algorithm
    if (x === y) {
      // Steps 1-5, 7-10
      // Steps 6.b-6.e: +0 != -0
      return x !== 0 || 1 / x === 1 / y;
    } else {
      // Step 6.a: NaN == NaN
      return x !== x && y !== y;
    }
  }
  /*eslint-enable no-self-compare*/

  /**
   * We use an Error-like object for backward compatibility as people may call
   * PropTypes directly and inspect their output. However, we don't use real
   * Errors anymore. We don't inspect their stack anyway, and creating them
   * is prohibitively expensive if they are created too often, such as what
   * happens in oneOfType() for any type before the one that matched.
   */
  function PropTypeError(message) {
    this.message = message;
    this.stack = '';
  }
  // Make `instanceof Error` still work for returned errors.
  PropTypeError.prototype = Error.prototype;

  function createChainableTypeChecker(validate) {
    {
      var manualPropTypeCallCache = {};
      var manualPropTypeWarningCount = 0;
    }
    function checkType(isRequired, props, propName, componentName, location, propFullName, secret) {
      componentName = componentName || ANONYMOUS;
      propFullName = propFullName || propName;

      if (secret !== ReactPropTypesSecret_1) {
        if (throwOnDirectAccess) {
          // New behavior only for users of `prop-types` package
          invariant_1(
            false,
            'Calling PropTypes validators directly is not supported by the `prop-types` package. ' +
            'Use `PropTypes.checkPropTypes()` to call them. ' +
            'Read more at http://fb.me/use-check-prop-types'
          );
        } else if ("development" !== 'production' && typeof console !== 'undefined') {
          // Old behavior for people using React.PropTypes
          var cacheKey = componentName + ':' + propName;
          if (
            !manualPropTypeCallCache[cacheKey] &&
            // Avoid spamming the console because they are often not actionable except for lib authors
            manualPropTypeWarningCount < 3
          ) {
            warning_1(
              false,
              'You are manually calling a React.PropTypes validation ' +
              'function for the `%s` prop on `%s`. This is deprecated ' +
              'and will throw in the standalone `prop-types` package. ' +
              'You may be seeing this warning due to a third-party PropTypes ' +
              'library. See https://fb.me/react-warning-dont-call-proptypes ' + 'for details.',
              propFullName,
              componentName
            );
            manualPropTypeCallCache[cacheKey] = true;
            manualPropTypeWarningCount++;
          }
        }
      }
      if (props[propName] == null) {
        if (isRequired) {
          if (props[propName] === null) {
            return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required ' + ('in `' + componentName + '`, but its value is `null`.'));
          }
          return new PropTypeError('The ' + location + ' `' + propFullName + '` is marked as required in ' + ('`' + componentName + '`, but its value is `undefined`.'));
        }
        return null;
      } else {
        return validate(props, propName, componentName, location, propFullName);
      }
    }

    var chainedCheckType = checkType.bind(null, false);
    chainedCheckType.isRequired = checkType.bind(null, true);

    return chainedCheckType;
  }

  function createPrimitiveTypeChecker(expectedType) {
    function validate(props, propName, componentName, location, propFullName, secret) {
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== expectedType) {
        // `propValue` being instance of, say, date/regexp, pass the 'object'
        // check, but we can offer a more precise error message here rather than
        // 'of type `object`'.
        var preciseType = getPreciseType(propValue);

        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + preciseType + '` supplied to `' + componentName + '`, expected ') + ('`' + expectedType + '`.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createAnyTypeChecker() {
    return createChainableTypeChecker(emptyFunction_1.thatReturnsNull);
  }

  function createArrayOfTypeChecker(typeChecker) {
    function validate(props, propName, componentName, location, propFullName) {
      if (typeof typeChecker !== 'function') {
        return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside arrayOf.');
      }
      var propValue = props[propName];
      if (!Array.isArray(propValue)) {
        var propType = getPropType(propValue);
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an array.'));
      }
      for (var i = 0; i < propValue.length; i++) {
        var error = typeChecker(propValue, i, componentName, location, propFullName + '[' + i + ']', ReactPropTypesSecret_1);
        if (error instanceof Error) {
          return error;
        }
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createElementTypeChecker() {
    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      if (!isValidElement(propValue)) {
        var propType = getPropType(propValue);
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected a single ReactElement.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createInstanceTypeChecker(expectedClass) {
    function validate(props, propName, componentName, location, propFullName) {
      if (!(props[propName] instanceof expectedClass)) {
        var expectedClassName = expectedClass.name || ANONYMOUS;
        var actualClassName = getClassName(props[propName]);
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + actualClassName + '` supplied to `' + componentName + '`, expected ') + ('instance of `' + expectedClassName + '`.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createEnumTypeChecker(expectedValues) {
    if (!Array.isArray(expectedValues)) {
      warning_1(false, 'Invalid argument supplied to oneOf, expected an instance of array.');
      return emptyFunction_1.thatReturnsNull;
    }

    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      for (var i = 0; i < expectedValues.length; i++) {
        if (is(propValue, expectedValues[i])) {
          return null;
        }
      }

      var valuesString = JSON.stringify(expectedValues);
      return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of value `' + propValue + '` ' + ('supplied to `' + componentName + '`, expected one of ' + valuesString + '.'));
    }
    return createChainableTypeChecker(validate);
  }

  function createObjectOfTypeChecker(typeChecker) {
    function validate(props, propName, componentName, location, propFullName) {
      if (typeof typeChecker !== 'function') {
        return new PropTypeError('Property `' + propFullName + '` of component `' + componentName + '` has invalid PropType notation inside objectOf.');
      }
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== 'object') {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type ' + ('`' + propType + '` supplied to `' + componentName + '`, expected an object.'));
      }
      for (var key in propValue) {
        if (propValue.hasOwnProperty(key)) {
          var error = typeChecker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret_1);
          if (error instanceof Error) {
            return error;
          }
        }
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createUnionTypeChecker(arrayOfTypeCheckers) {
    if (!Array.isArray(arrayOfTypeCheckers)) {
      warning_1(false, 'Invalid argument supplied to oneOfType, expected an instance of array.');
      return emptyFunction_1.thatReturnsNull;
    }

    for (var i = 0; i < arrayOfTypeCheckers.length; i++) {
      var checker = arrayOfTypeCheckers[i];
      if (typeof checker !== 'function') {
        warning_1(
          false,
          'Invalid argument supplied to oneOfType. Expected an array of check functions, but ' +
          'received %s at index %s.',
          getPostfixForTypeWarning(checker),
          i
        );
        return emptyFunction_1.thatReturnsNull;
      }
    }

    function validate(props, propName, componentName, location, propFullName) {
      for (var i = 0; i < arrayOfTypeCheckers.length; i++) {
        var checker = arrayOfTypeCheckers[i];
        if (checker(props, propName, componentName, location, propFullName, ReactPropTypesSecret_1) == null) {
          return null;
        }
      }

      return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`.'));
    }
    return createChainableTypeChecker(validate);
  }

  function createNodeChecker() {
    function validate(props, propName, componentName, location, propFullName) {
      if (!isNode(props[propName])) {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` supplied to ' + ('`' + componentName + '`, expected a ReactNode.'));
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createShapeTypeChecker(shapeTypes) {
    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== 'object') {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));
      }
      for (var key in shapeTypes) {
        var checker = shapeTypes[key];
        if (!checker) {
          continue;
        }
        var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret_1);
        if (error) {
          return error;
        }
      }
      return null;
    }
    return createChainableTypeChecker(validate);
  }

  function createStrictShapeTypeChecker(shapeTypes) {
    function validate(props, propName, componentName, location, propFullName) {
      var propValue = props[propName];
      var propType = getPropType(propValue);
      if (propType !== 'object') {
        return new PropTypeError('Invalid ' + location + ' `' + propFullName + '` of type `' + propType + '` ' + ('supplied to `' + componentName + '`, expected `object`.'));
      }
      // We need to check all keys in case some are required but missing from
      // props.
      var allKeys = objectAssign({}, props[propName], shapeTypes);
      for (var key in allKeys) {
        var checker = shapeTypes[key];
        if (!checker) {
          return new PropTypeError(
            'Invalid ' + location + ' `' + propFullName + '` key `' + key + '` supplied to `' + componentName + '`.' +
            '\nBad object: ' + JSON.stringify(props[propName], null, '  ') +
            '\nValid keys: ' +  JSON.stringify(Object.keys(shapeTypes), null, '  ')
          );
        }
        var error = checker(propValue, key, componentName, location, propFullName + '.' + key, ReactPropTypesSecret_1);
        if (error) {
          return error;
        }
      }
      return null;
    }

    return createChainableTypeChecker(validate);
  }

  function isNode(propValue) {
    switch (typeof propValue) {
      case 'number':
      case 'string':
      case 'undefined':
        return true;
      case 'boolean':
        return !propValue;
      case 'object':
        if (Array.isArray(propValue)) {
          return propValue.every(isNode);
        }
        if (propValue === null || isValidElement(propValue)) {
          return true;
        }

        var iteratorFn = getIteratorFn(propValue);
        if (iteratorFn) {
          var iterator = iteratorFn.call(propValue);
          var step;
          if (iteratorFn !== propValue.entries) {
            while (!(step = iterator.next()).done) {
              if (!isNode(step.value)) {
                return false;
              }
            }
          } else {
            // Iterator will provide entry [k,v] tuples rather than values.
            while (!(step = iterator.next()).done) {
              var entry = step.value;
              if (entry) {
                if (!isNode(entry[1])) {
                  return false;
                }
              }
            }
          }
        } else {
          return false;
        }

        return true;
      default:
        return false;
    }
  }

  function isSymbol(propType, propValue) {
    // Native Symbol.
    if (propType === 'symbol') {
      return true;
    }

    // 19.4.3.5 Symbol.prototype[@@toStringTag] === 'Symbol'
    if (propValue['@@toStringTag'] === 'Symbol') {
      return true;
    }

    // Fallback for non-spec compliant Symbols which are polyfilled.
    if (typeof Symbol === 'function' && propValue instanceof Symbol) {
      return true;
    }

    return false;
  }

  // Equivalent of `typeof` but with special handling for array and regexp.
  function getPropType(propValue) {
    var propType = typeof propValue;
    if (Array.isArray(propValue)) {
      return 'array';
    }
    if (propValue instanceof RegExp) {
      // Old webkits (at least until Android 4.0) return 'function' rather than
      // 'object' for typeof a RegExp. We'll normalize this here so that /bla/
      // passes PropTypes.object.
      return 'object';
    }
    if (isSymbol(propType, propValue)) {
      return 'symbol';
    }
    return propType;
  }

  // This handles more types than `getPropType`. Only used for error messages.
  // See `createPrimitiveTypeChecker`.
  function getPreciseType(propValue) {
    if (typeof propValue === 'undefined' || propValue === null) {
      return '' + propValue;
    }
    var propType = getPropType(propValue);
    if (propType === 'object') {
      if (propValue instanceof Date) {
        return 'date';
      } else if (propValue instanceof RegExp) {
        return 'regexp';
      }
    }
    return propType;
  }

  // Returns a string that is postfixed to a warning about an invalid type.
  // For example, "undefined" or "of type array"
  function getPostfixForTypeWarning(value) {
    var type = getPreciseType(value);
    switch (type) {
      case 'array':
      case 'object':
        return 'an ' + type;
      case 'boolean':
      case 'date':
      case 'regexp':
        return 'a ' + type;
      default:
        return type;
    }
  }

  // Returns class name of the object, if any.
  function getClassName(propValue) {
    if (!propValue.constructor || !propValue.constructor.name) {
      return ANONYMOUS;
    }
    return propValue.constructor.name;
  }

  ReactPropTypes.checkPropTypes = checkPropTypes_1;
  ReactPropTypes.PropTypes = ReactPropTypes;

  return ReactPropTypes;
};

var propTypes = createCommonjsModule(function (module) {
/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

{
  var REACT_ELEMENT_TYPE = (typeof Symbol === 'function' &&
    Symbol.for &&
    Symbol.for('react.element')) ||
    0xeac7;

  var isValidElement = function(object) {
    return typeof object === 'object' &&
      object !== null &&
      object.$$typeof === REACT_ELEMENT_TYPE;
  };

  // By explicitly using `prop-types` you are opting into new development behavior.
  // http://fb.me/prop-types-in-prod
  var throwOnDirectAccess = true;
  module.exports = factoryWithTypeCheckers(isValidElement, throwOnDirectAccess);
}
});

var classnames = createCommonjsModule(function (module) {
/*!
  Copyright (c) 2016 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/
/* global define */

(function () {
	var hasOwn = {}.hasOwnProperty;

	function classNames () {
		var classes = [];

		for (var i = 0; i < arguments.length; i++) {
			var arg = arguments[i];
			if (!arg) continue;

			var argType = typeof arg;

			if (argType === 'string' || argType === 'number') {
				classes.push(arg);
			} else if (Array.isArray(arg)) {
				classes.push(classNames.apply(null, arg));
			} else if (argType === 'object') {
				for (var key in arg) {
					if (hasOwn.call(arg, key) && arg[key]) {
						classes.push(key);
					}
				}
			}
		}

		return classes.join(' ');
	}

	if ('object' !== 'undefined' && module.exports) {
		module.exports = classNames;
	} else if (typeof undefined === 'function' && typeof undefined.amd === 'object' && undefined.amd) {
		// register as 'classnames', consistent with npm package name
		undefined('classnames', [], function () {
			return classNames;
		});
	} else {
		window.classNames = classNames;
	}
}());
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};











var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};









var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};



















var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var NavItem = function NavItem(_ref) {
	var children = _ref.children,
	    onClick = _ref.onClick,
	    active = _ref.active,
	    disabled = _ref.disabled,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['children', 'onClick', 'active', 'disabled', 'className']);
	return React.createElement(
		'li',
		_extends({
			className: classnames('bmbo-nav-item', active && 'bmbo-active', disabled && 'bmbo-disabled', className),
			role: 'button',
			tabIndex: -1,
			onClick: onClick
		}, props),
		children
	);
};

NavItem.propTypes = {
	children: propTypes.node,
	onClick: propTypes.func,
	className: propTypes.string,

	active: propTypes.bool,
	disabled: propTypes.bool
};

var HorizontalNav = function HorizontalNav(_ref) {
	var type = _ref.type,
	    children = _ref.children,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['type', 'children', 'className']);
	return React.createElement(
		'ul',
		_extends({
			className: classnames('bmbo-nav', 'bmbo-horizontal-nav', 'bmbo-' + (type || 'lead'), className)
		}, props),
		children
	);
};

HorizontalNav.propTypes = {
	type: propTypes.string,
	children: propTypes.node,
	className: propTypes.string
};

var VerticalNav = function VerticalNav(_ref) {
	var type = _ref.type,
	    children = _ref.children,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['type', 'children', 'className']);
	return React.createElement(
		'ul',
		_extends({
			className: classnames('bmbo-nav', 'bmbo-vertical-nav', 'bmbo-' + (type || 'lead'), className)
		}, props),
		children
	);
};

VerticalNav.propTypes = {
	type: propTypes.string,
	children: propTypes.node,
	className: propTypes.string
};

var canUseDOM = !!(typeof window !== 'undefined' && window.document && window.document.createElement);

var ANIMATE_STATUS_NONE = 0;
var ANIMATE_STATUS_INIT = 1;
var ANIMATE_STATUS_SHOWING = 2;
var ANIMATE_STATUS_SHOWN = 3;
var ANIMATE_STATUS_HIDING = 4;

var BAMBOO_PORTALS_HOLDER = 'bmbo_portals_holder';
var $bambooHolder = void 0;

// =====================================================================================
// =                                   Window Holder                                   =
// =====================================================================================
if (canUseDOM) {
	// Clean up menu holder for hot module reload
	window.document.querySelectorAll('#' + BAMBOO_PORTALS_HOLDER).forEach(function (ele) {
		ele.remove();
	});

	// Set up menu holder
	$bambooHolder = window.document.createElement('div');
	$bambooHolder.id = BAMBOO_PORTALS_HOLDER;
	$bambooHolder.className = 'bmbo-portals-holder';
	// $bambooHolder.dataset.timestamp = Date.now();
	window.document.body.appendChild($bambooHolder);
}


// =====================================================================================
// =                                     Animation                                     =
// =====================================================================================
var ANIMATION_INTERVAL = 1000 / 60;

var delayByFrame = function delayByFrame(func) {
	if (window.requestAnimationFrame) {
		window.requestAnimationFrame(func);
	} else {
		setTimeout(func, ANIMATION_INTERVAL);
	}
};

var requestAnimationFrame = function requestAnimationFrame(func) {
	var delayFrame = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

	var count = delayFrame;

	function doAction() {
		if (count <= 0) {
			func();
			return;
		}

		count -= 1;
		delayByFrame(doAction);
	}

	doAction();
};

// =====================================================================================
// =                                      Scroll                                       =
// =====================================================================================
/**
 * Get Scroll bar size
 * @type {number}
 */
var scrollbarWidth = 0;
function getScrollbarWidth() {
	if (scrollbarWidth || !canUseDOM) return scrollbarWidth;

	var $body = window.document.body;
	var scrollDiv = document.createElement('div');
	scrollDiv.className = 'bmbo-scrollbar-measure';
	$body.appendChild(scrollDiv);
	scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
	scrollDiv.remove();
	return scrollbarWidth;
}

var hasHorizontalScroll = function hasHorizontalScroll() {
	var winWidth = window.innerWidth;
	var bodyWidth = window.document.body.scrollWidth;
	return winWidth < bodyWidth;
};

var hasVerticalScroll = function hasVerticalScroll() {
	var winHeight = window.innerHeight;
	var bodyHeight = window.document.body.scrollHeight;
	return winHeight < bodyHeight;
};

var winScrollLocker = [];

function enableWinScrollBar(name) {
	if (!canUseDOM) return;

	if (name) {
		winScrollLocker = winScrollLocker.filter(function (n) {
			return n !== name;
		});
		if (winScrollLocker.length) return;
	} else {
		winScrollLocker = [];
	}

	document.body.style.paddingRight = '';
	document.body.style.overflowY = '';
}

function disableWinScrollBar(name) {
	if (!canUseDOM) return;
	if (!hasVerticalScroll()) return;

	if (name) {
		winScrollLocker.push(name);
	}

	document.body.style.paddingRight = getScrollbarWidth() + 'px';
	document.body.style.overflowY = 'hidden';
}

// =====================================================================================
// =                                     Position                                      =
// =====================================================================================
var createPortal$1 = function createPortal$$1(node) {
	if (!canUseDOM) return null;

	return reactDom.createPortal(node, $bambooHolder);
};

var getWinWidth = function getWinWidth() {
	var winWidth = window.innerWidth;
	if (hasVerticalScroll()) winWidth -= getScrollbarWidth();
	return winWidth;
};

var getWinHeight = function getWinHeight() {
	var winHeight = window.innerHeight;
	if (hasHorizontalScroll()) winHeight -= getScrollbarWidth();
	return winHeight;
};

/**
 * Get enabled position for element.
 * @param surroundRect
 * @param targetRect
 * @param position
 * @returns {{x: number, y: number}}
 */
var getEnablePosition = function getEnablePosition(surroundRect, targetRect) {
	var position = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'dr';

	if (typeof window === 'undefined') return { x: 0, y: 0 };

	var _surroundRect$width = surroundRect.width,
	    sw = _surroundRect$width === undefined ? 0 : _surroundRect$width,
	    _surroundRect$height = surroundRect.height,
	    sh = _surroundRect$height === undefined ? 0 : _surroundRect$height;

	var sx = surroundRect.left || 0;
	var sy = surroundRect.top || 0;
	var _targetRect$width = targetRect.width,
	    tw = _targetRect$width === undefined ? 0 : _targetRect$width,
	    _targetRect$height = targetRect.height,
	    th = _targetRect$height === undefined ? 0 : _targetRect$height;


	var winWidth = getWinWidth();
	var winHeight = getWinHeight();
	var scrollX = window.scrollX;
	var scrollY = window.scrollY;

	var isR = position.includes('r');
	var isB = position.includes('b');
	var isT = position.includes('t');
	var isL = position.includes('l');
	var isLorR = isL && !isR || !isL && isR;
	var isTorB = isT && !isB || !isT && isB;

	var target = {
		x: sx + scrollX,
		y: sy + scrollY,
		direct: ''
	};

	if (isL && !isR) target.x = sx - tw + scrollX;
	if (isR && !isL) target.x = sx + sw + scrollX;
	if (isT && !isB) target.y = sy - th + scrollY;
	if (isB && !isT) target.y = sy + sh + scrollY;

	if (isL && isR) target.x = sx - (tw - sw) / 2 + scrollX;
	if (isT && isB) target.y = sy - (th - sh) / 2 + scrollY;

	target._x = target.x;
	target._y = target.y;

	// Bottom out of the window
	if (target.y - scrollY + th > winHeight) {
		if (isLorR) {
			target.y = winHeight - th + scrollY;
		} else {
			var ty = sy - th;
			if (ty >= 0) {
				target.y = sy + scrollY - th;
				target.direct = 'up';
			}
		}
	}

	// Top out of the window
	if (target.y - scrollY < 0) {
		if (isLorR) {
			target.y = scrollY;
		} else {
			target.y = sy + sh + scrollY;
		}
	}

	// Left out of the window
	if (target.x - scrollX < 0) {
		if (isTorB) {
			target.x = scrollX;
		} else {
			target.x = sx + sw + scrollX;
		}
	}

	// Right out of the window
	if (target.x - scrollX + tw > winWidth) {
		if (isTorB) {
			target.x = winWidth - tw + scrollX;
		} else {
			target.x = sx + scrollX - tw;
		}
	}

	return target;
};

/**
 * Get enabled position for element.
 * @param surroundRect
 * @param targetRect
 * @param position
 * @param switchPos		Switch position when position has no space left
 * @returns {{x: number, y: number}}
 */
/* export const getEnablePosition2 = (surroundRect, targetRect, position = 'dr', switchPos = true) => {
	if (typeof window === 'undefined') return { x: 0, y: 0 };

	const { width: sw = 0, height: sh = 0 } = surroundRect;
	const sx = surroundRect.left || 0;
	const sy = surroundRect.top || 0;
	const { width: tw = 0, height: th = 0 } = targetRect;

	const winWidth = getWinWidth();
	const winHeight = getWinHeight();
	const scrollX = window.scrollX;
	const scrollY = window.scrollY;

	const isR = position.includes('r');
	const isB = position.includes('b');
	const isT = position.includes('t');
	const isL = position.includes('l');

	const target = {
		x: sx + scrollX,
		y: sy + scrollY,
		direct: '',
	};

	if (isL && !isR) target.x = (sx - tw) + scrollX;
	if (isR && !isL) target.x = (sx + sw) + scrollX;
	if (isT && !isB) target.y = (sy - th) + scrollY;
	if (isB && !isT) target.y = (sy + sh) + scrollY;

	if (isL && isR) target.x = (sx - ((tw - sw) / 2)) + scrollX;
	if (isT && isB) target.y = (sy - ((th - sh) / 2)) + scrollY;

	target._x = target.x;
	target._y = target.y;

	// Bottom out of the window
	if ((target.y - scrollY) + th > winHeight) {
		if (isR) {
			target.y = (winHeight - th) + scrollY;
		} else {
			const ty = (sy + scrollY) - th;
			if (ty >= 0) {
				target.y = (sy + scrollY) - th;
				target.direct = 'up';
			}
		}
	}
	// Top out of the window
	if ((target.y - scrollY) < 0) {
		if (switchPos) {
			target.y = sy + sh + scrollY;
		} else {
			target.y = scrollY;
		}
	}

	// Left out of the window
	if (target.x - scrollX < 0) {
		if (switchPos) {
			target.x = sx + sw;
		} else {
			target.x = scrollX;
		}
	}

	// Right out of the window
	if ((target.x - scrollX) + tw > winWidth) {
		if (switchPos) {
			target.x = sx - tw;
		} else {
			target.x = (winWidth + scrollX) - tw;
		}
	}

	return target;
}; */

function isInRect(x, y, rect) {
	return rect.left < x && x < rect.right && rect.top < y && y < rect.bottom;
}

var Caret = function Caret(_ref) {
	var _ref$direct = _ref.direct,
	    direct = _ref$direct === undefined ? 'right' : _ref$direct,
	    className = _ref.className;
	return React.createElement('span', {
		className: classnames('bmbo-icon bmbo-caret', direct && 'bmbo-caret-' + direct, className)
	});
};

Caret.propTypes = {
	direct: propTypes.string,
	className: propTypes.string
};

var TaskHolder = function () {
	function TaskHolder(config) {
		classCallCheck(this, TaskHolder);

		this.config = config;
		this.list = [];
		this.index = 0;
		this.promise = null;
	}

	createClass(TaskHolder, [{
		key: 'weakThan',
		value: function weakThan() {
			var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			return (this.config.priority || 0) <= (config.priority || 0);
		}
	}, {
		key: 'next',
		value: function next(callback) {
			var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

			this.list.push({ callback: callback, config: config });
			return this;
		}
	}, {
		key: 'getCurrent',
		value: function getCurrent() {
			return this.list[this.index];
		}
	}, {
		key: 'nextStep',
		value: function nextStep() {
			this.index += 1;
		}
	}]);
	return TaskHolder;
}();

var Sequence = function () {
	function Sequence() {
		classCallCheck(this, Sequence);
		this.taskHolder = null;
		this._destroy = false;
	}

	createClass(Sequence, [{
		key: 'next',
		value: function next(callback) {
			var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

			var taskHolder = new TaskHolder(config);
			taskHolder.next(callback, config);

			if (!this.taskHolder || this.taskHolder.weakThan(config)) {
				this.taskHolder = taskHolder;
			}

			taskHolder.promise = this._doAction(taskHolder);

			return taskHolder;
		}
	}, {
		key: 'stop',
		value: function stop() {
			this.taskHolder = null;
		}
	}, {
		key: 'destroy',
		value: function destroy() {
			this._destroy = true;
		}
	}, {
		key: '_doAction',
		value: function _doAction(taskHolder) {
			var _this = this;

			return new Promise(function (resolve, reject) {
				if (_this._destroy) return;

				var step = function step() {
					var task = taskHolder.getCurrent();
					if (!task) {
						_this.taskHolder = null;
						resolve();
						return;
					}

					var exec = function exec() {
						try {
							if (_this.taskHolder !== taskHolder || _this._destroy) return;

							var taskCallback = task.callback;
							var needCallback = taskCallback.length !== 0;

							// Do next
							var doNext = function doNext() {
								// If is a looper
								if (task.config.loop) {
									step();
									return;
								}

								// Next task
								taskHolder.nextStep();
								step();
							};

							// Execute task
							var taskResult = taskCallback(doNext);
							if (taskResult === false) {
								_this.taskHolder = null;
								resolve();
								return;
							}

							if (!needCallback) {
								doNext();
							}
						} catch (err) {
							reject(err);
						}
					};

					if (task.config.delay !== undefined) {
						setTimeout(exec, task.config.delay);
					} else {
						requestAnimationFrame(exec, task.config.frame || 2);
					}
				};

				step();
			});
		}
	}]);
	return Sequence;
}();

var DRILL_SPEED = 4 / 5;

var NavList = function (_React$Component) {
	inherits(NavList, _React$Component);

	function NavList() {
		classCallCheck(this, NavList);

		var _this = possibleConstructorReturn(this, (NavList.__proto__ || Object.getPrototypeOf(NavList)).call(this));

		_this.onTransitionEnd = function (event) {
			if (_this.state.animateStatus === ANIMATE_STATUS_HIDING && event.target === _this.$list) {
				_this.seq.next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				});
			}
		};

		_this.setListRef = function (ele) {
			var setRef = _this.props.setRef;

			_this.$list = ele;
			setRef(ele);
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (_this.context.navInline) {
				if (!prevProps.open === !nextProps.open) return;
				_this.doDrillAnimation(nextProps);
			} else {
				if (!prevProps.visible === !nextProps.visible) return;
				_this.doFloatAnimation(nextProps);
			}
		};

		_this.doDrillAnimation = function (props) {
			if (props.open) {
				_this.seq.next(function () {
					_this.setState({
						height: 0
					});
				}).next(function () {
					var clientHeight = _this.$list.clientHeight;
					var scrollHeight = _this.$list.scrollHeight;

					var currentHeight = Math.ceil(clientHeight * DRILL_SPEED + scrollHeight * (1 - DRILL_SPEED));
					if (scrollHeight - currentHeight <= 1) {
						currentHeight = scrollHeight;
						_this.setState({ height: null });
						return false;
					}

					_this.setState({ height: currentHeight });

					return null;
				}, {
					loop: true,
					frame: 1
				});
			} else {
				_this.seq.next(function () {
					_this.setState({
						height: null
					});
				}).next(function () {
					var clientHeight = _this.$list.clientHeight;

					var currentHeight = Math.floor(clientHeight * DRILL_SPEED);
					if (clientHeight <= 1) {
						_this.setState({ height: 0 });
						return false;
					}

					_this.setState({ height: currentHeight });

					return null;
				}, {
					loop: true,
					frame: 1
				});
			}
		};

		_this.doFloatAnimation = function (props) {
			if (props.visible) {
				_this.seq.next(function () {
					_this.setState({
						animateStatus: ANIMATE_STATUS_INIT,
						x: 0,
						y: 0
					});
				}).next(function () {
					var rect = _this.props.rect;
					var navVertical = _this.context.navVertical;

					var listRect = _this.$list.getBoundingClientRect();

					var pos = navVertical ? 'r' : 'b';

					// Add offset to let nav list match with parent
					var superRect = {
						left: rect.left,
						top: rect.top,
						width: rect.width,
						height: rect.height
					};
					if (navVertical) {
						superRect.top -= 1;
						superRect.height += 2;
					} else {
						superRect.left -= 1;
						superRect.width += 2;
					}

					_this.setState(_extends({}, getEnablePosition(superRect, listRect, pos), {
						animateStatus: ANIMATE_STATUS_SHOWING
					}));
				}).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_SHOWN });
				});
			} else {
				_this.seq.next(function () {
					var animateStatus = _this.state.animateStatus;

					if (animateStatus === ANIMATE_STATUS_NONE || animateStatus === ANIMATE_STATUS_INIT) {
						return false;
					}
					_this.setState({ animateStatus: ANIMATE_STATUS_HIDING });
					return null;
				}).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				}, { delay: 1000 });
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE,
			x: 0,
			y: 0,
			height: 0
		};
		_this.seq = new Sequence();
		return _this;
	}

	createClass(NavList, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    rect = _props.rect;
			var _context = this.context,
			    navType = _context.navType,
			    navVertical = _context.navVertical,
			    navInline = _context.navInline;
			var _state = this.state,
			    animateStatus = _state.animateStatus,
			    x = _state.x,
			    y = _state.y,
			    height = _state.height;


			if (!navVertical || !navInline) {
				// Float display

				if (animateStatus === ANIMATE_STATUS_NONE) return null;

				return createPortal$1(React.createElement(
					'div',
					{
						className: classnames('bmbo-nav-list', {
							'bmbo-hidden': animateStatus === ANIMATE_STATUS_INIT,
							'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
							'bmbo-hiding': animateStatus === ANIMATE_STATUS_HIDING
						}, 'bmbo-' + (navType || 'lead')),
						ref: this.setListRef,
						style: { left: x + 'px', top: y + 'px', minWidth: rect.width + 'px' },
						onTransitionEnd: this.onTransitionEnd
					},
					children
				));
			}

			// Inline display
			var style = {};
			if (height !== null) {
				style.height = height + 'px';
			}

			return React.createElement(
				'ul',
				{
					className: 'bmbo-nav-list',
					style: style,
					ref: this.setListRef
				},
				children
			);
		}
	}]);
	return NavList;
}(React.Component);

NavList.propTypes = {
	visible: propTypes.bool,
	open: propTypes.bool,
	rect: propTypes.object,

	setRef: propTypes.func,

	children: propTypes.node
};

NavList.contextTypes = {
	navType: propTypes.string,
	navVertical: propTypes.bool,
	navInline: propTypes.bool
};

var NavGroup = function (_React$Component) {
	inherits(NavGroup, _React$Component);

	function NavGroup() {
		classCallCheck(this, NavGroup);

		var _this = possibleConstructorReturn(this, (NavGroup.__proto__ || Object.getPrototypeOf(NavGroup)).call(this));

		_this.onMouseEnter = function () {
			var rect = _this.$item.getBoundingClientRect();
			_this.setState({ hover: true, rect: rect });
		};

		_this.onMouseLeave = function (event) {
			var clientX = event.clientX,
			    clientY = event.clientY;


			if (_this.$list) {
				var rect = _this.$list.getBoundingClientRect();

				// Chrome has the bug of click event will trigger mouse leave event
				// ref: https://stackoverflow.com/questions/45266854/mouseleave-triggered-by-click
				if (isInRect(clientX, clientY, rect)) {
					return;
				}
			}

			_this.setState({ hover: false });
		};

		_this.onClick = function () {
			_this.setState(function (_ref) {
				var open = _ref.open;
				return {
					open: !open
				};
			});
		};

		_this.setItemRef = function (ele) {
			_this.$item = ele;
		};

		_this.setListRef = function (ele) {
			_this.$list = ele;
		};

		_this.state = {
			hover: false,
			rect: null,

			open: false
		};
		return _this;
	}

	createClass(NavGroup, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    title = _props.title,
			    children = _props.children,
			    active = _props.active,
			    disabled = _props.disabled,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['title', 'children', 'active', 'disabled', 'className']);
			var _context = this.context,
			    navVertical = _context.navVertical,
			    navInline = _context.navInline;
			var _state = this.state,
			    hover = _state.hover,
			    rect = _state.rect,
			    open = _state.open;


			return React.createElement(
				'li',
				_extends({
					className: classnames('bmbo-nav-group', hover && 'bmbo-hover', active && 'bmbo-active', disabled && 'bmbo-disabled', className),
					ref: this.setItemRef,
					onMouseEnter: this.onMouseEnter,
					onMouseLeave: this.onMouseLeave
				}, props),
				React.createElement(
					'div',
					{
						className: 'bmbo-nav-title',
						role: 'button',
						tabIndex: -1,
						onClick: this.onClick
					},
					title,
					navVertical && React.createElement(Caret, {
						direct: navInline && open ? 'down' : 'right'
					})
				),
				React.createElement(
					NavList,
					{
						visible: hover,
						rect: rect,
						setRef: this.setListRef,
						open: open
					},
					children
				)
			);
		}
	}]);
	return NavGroup;
}(React.Component);

NavGroup.propTypes = {
	className: propTypes.string,
	title: propTypes.node,
	children: propTypes.node,
	active: propTypes.bool,
	disabled: propTypes.bool
};

NavGroup.contextTypes = {
	navVertical: propTypes.bool,
	navInline: propTypes.bool
};

var Nav = function (_React$Component) {
	inherits(Nav, _React$Component);

	function Nav() {
		classCallCheck(this, Nav);
		return possibleConstructorReturn(this, (Nav.__proto__ || Object.getPrototypeOf(Nav)).apply(this, arguments));
	}

	createClass(Nav, [{
		key: 'getChildContext',
		value: function getChildContext() {
			return {
				navType: this.props.type,
				navVertical: this.props.vertical,
				navInline: this.props.inline
			};
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    vertical = _props.vertical,
			    props = objectWithoutProperties(_props, ['vertical']);

			delete props.inline;

			if (vertical) return React.createElement(VerticalNav, props);
			return React.createElement(HorizontalNav, props);
		}
	}]);
	return Nav;
}(React.Component);

Nav.propTypes = {
	vertical: propTypes.bool,
	type: propTypes.string,
	inline: propTypes.bool
};

Nav.childContextTypes = {
	navType: propTypes.string,
	navVertical: propTypes.bool,
	navInline: propTypes.bool
};

Nav.Group = NavGroup;
Nav.Item = NavItem;

var REGEX = /^(\d+)\/(\d+)$/;

function getMatch(numerator, denominator, total) {
	var multiple = total / denominator;
	if (multiple !== Math.floor(multiple)) return null;

	return numerator * multiple + '-' + total;
}

function getName(prefix, val) {
	if (!val) return '';
	if (val === '0') return prefix + '0';

	var myVal = val === '1' ? '1/1' : val;
	var match = Array.isArray(myVal) ? [null].concat(toConsumableArray(myVal)) : myVal.match(REGEX);

	// Return full row if not match
	if (!match) return '10-10';

	var numerator = Number(match[1]);
	var denominator = Number(match[2]);

	var cal = getMatch(numerator, denominator, 24) || getMatch(numerator, denominator, 10);
	return '' + prefix + cal;
}

var Col = function Col(props, context) {
	var xs = props.xs,
	    sm = props.sm,
	    md = props.md,
	    lg = props.lg,
	    xl = props.xl,
	    xsOffset = props.xsOffset,
	    smOffset = props.smOffset,
	    mdOffset = props.mdOffset,
	    lgOffset = props.lgOffset,
	    xlOffset = props.xlOffset,
	    children = props.children,
	    className = props.className,
	    rest = objectWithoutProperties(props, ['xs', 'sm', 'md', 'lg', 'xl', 'xsOffset', 'smOffset', 'mdOffset', 'lgOffset', 'xlOffset', 'children', 'className']);
	var rowGutter = context.rowGutter;


	var xsClassName = getName('bmbo-col-xs-', xs);
	var smClassName = getName('bmbo-col-sm-', sm);
	var mdClassName = getName('bmbo-col-md-', md);
	var lgClassName = getName('bmbo-col-lg-', lg);
	var xlClassName = getName('bmbo-col-xl-', xl);

	var xsOffsetClassName = getName('bmbo-col-xs-offset-', xsOffset);
	var smOffsetClassName = getName('bmbo-col-sm-offset-', smOffset);
	var mdOffsetClassName = getName('bmbo-col-md-offset-', mdOffset);
	var lgOffsetClassName = getName('bmbo-col-lg-offset-', lgOffset);
	var xlOffsetClassName = getName('bmbo-col-xl-offset-', xlOffset);

	var hg = rowGutter / 2;

	return React.createElement(
		'div',
		_extends({
			className: classnames('bmbo-col', xsClassName, smClassName, mdClassName, lgClassName, xlClassName, xsOffsetClassName, smOffsetClassName, mdOffsetClassName, lgOffsetClassName, xlOffsetClassName, className),
			style: { paddingLeft: hg + 'px', paddingRight: hg + 'px' }
		}, rest),
		children
	);
};

Col.propTypes = {
	className: propTypes.string,

	xs: propTypes.oneOfType([propTypes.string, propTypes.array]),
	sm: propTypes.oneOfType([propTypes.string, propTypes.array]),
	md: propTypes.oneOfType([propTypes.string, propTypes.array]),
	lg: propTypes.oneOfType([propTypes.string, propTypes.array]),
	xl: propTypes.oneOfType([propTypes.string, propTypes.array]),

	xsOffset: propTypes.oneOfType([propTypes.string, propTypes.array]),
	smOffset: propTypes.oneOfType([propTypes.string, propTypes.array]),
	mdOffset: propTypes.oneOfType([propTypes.string, propTypes.array]),
	lgOffset: propTypes.oneOfType([propTypes.string, propTypes.array]),
	xlOffset: propTypes.oneOfType([propTypes.string, propTypes.array]),

	children: propTypes.node
};

Col.contextTypes = {
	rowGutter: propTypes.number
};

var DEFAULT_GUTTER = 15;

var Row = function (_React$PureComponent) {
	inherits(Row, _React$PureComponent);

	function Row() {
		classCallCheck(this, Row);
		return possibleConstructorReturn(this, (Row.__proto__ || Object.getPrototypeOf(Row)).apply(this, arguments));
	}

	createClass(Row, [{
		key: 'getChildContext',
		value: function getChildContext() {
			var _props$gutter = this.props.gutter,
			    gutter = _props$gutter === undefined ? DEFAULT_GUTTER : _props$gutter;


			return {
				rowGutter: gutter
			};
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    _props$gutter2 = _props.gutter,
			    gutter = _props$gutter2 === undefined ? DEFAULT_GUTTER : _props$gutter2,
			    children = _props.children,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['gutter', 'children', 'className']);

			var hg = gutter / 2;

			return React.createElement(
				'div',
				_extends({ className: classnames('bmbo-row', className) }, props),
				React.createElement(
					'div',
					{ style: { marginLeft: '-' + hg + 'px', marginRight: '-' + hg + 'px' } },
					children
				)
			);
		}
	}]);
	return Row;
}(React.PureComponent);

Row.propTypes = {
	gutter: propTypes.number,
	children: propTypes.node,
	className: propTypes.string
};

Row.childContextTypes = {
	rowGutter: propTypes.number
};

Row.Col = Col;

var Button = function (_React$Component) {
	inherits(Button, _React$Component);

	function Button() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Button);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Button.__proto__ || Object.getPrototypeOf(Button)).call.apply(_ref, [this].concat(args))), _this), _initialiseProps.call(_this), _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Button, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    className = _props.className,
			    disabled = _props.disabled,
			    size = _props.size,
			    transparent = _props.transparent,
			    checked = _props.checked,
			    props = objectWithoutProperties(_props, ['children', 'className', 'disabled', 'size', 'transparent', 'checked']);

			var type = this.props.type || 'primary';

			delete props.type;
			delete props.onClick;

			return React.createElement(
				'button',
				_extends({
					className: classnames('bmbo-button', type && 'bmbo-' + type, disabled && 'bmbo-disabled', 'bmbo-' + (size || 'md'), transparent && 'bmbo-transparent', checked && 'bmbo-checked', className),
					onClick: this.onClick

				}, props),
				children
			);
		}
	}]);
	return Button;
}(React.Component);

var _initialiseProps = function _initialiseProps() {
	var _this2 = this;

	this.onClick = function () {
		var _props2 = _this2.props,
		    onClick = _props2.onClick,
		    disabled = _props2.disabled;


		if (disabled) return;
		if (onClick) onClick.apply(undefined, arguments);
	};
};

Button.propTypes = {
	children: propTypes.node,

	type: propTypes.string,
	className: propTypes.string,
	size: propTypes.string,
	disabled: propTypes.bool,
	onClick: propTypes.func,
	transparent: propTypes.bool,
	checked: propTypes.bool
};

var Loading = function Loading(_ref) {
	var id = _ref.id,
	    className = _ref.className;
	return React.createElement('span', {
		className: classnames('bmbo-icon bmbo-loading', id && 'bmbo-loading-' + id, className)
	});
};

Loading.propTypes = {
	id: propTypes.number,
	className: propTypes.string
};

var Search = function Search(_ref) {
	var className = _ref.className;
	return React.createElement('span', {
		className: classnames('bmbo-icon bmbo-search', className)
	});
};

Search.propTypes = {
	className: propTypes.string
};

var Icon = function Icon(_ref) {
	var name = _ref.name,
	    spin = _ref.spin,
	    className = _ref.className;
	return React.createElement('span', { className: classnames('fa fa-' + name + ' bmbo-icon', spin && 'fa-spin', className) });
};

Icon.propTypes = {
	name: propTypes.string,
	spin: propTypes.bool,
	className: propTypes.string
};

Icon.Loading = Loading;
Icon.Caret = Caret;
Icon.Search = Search;

var A = function A(_ref) {
	var className = _ref.className,
	    target = _ref.target,
	    props = objectWithoutProperties(_ref, ['className', 'target']);

	var myProps = { target: target };
	if (target === '_blank') {
		myProps.rel = 'noopener noreferrer';
	}

	return React.createElement('a', _extends({
		className: classnames('bmbo-a', className)
	}, myProps, props));
};

A.propTypes = {
	className: propTypes.string,
	target: propTypes.string
};

var BAMBOO_COMPONENT = 'BAMBOO_COMPONENT';

var BAMBOO_INTERNAL_REF = 'BAMBOO_INTERNAL_REF';

var mapChildren = function mapChildren(children, func) {
	var list = React.Children.map(children, function (node, index) {
		return func(node, index);
	}) || [];
	return list.filter(function (node) {
		return node;
	});
};

var mapChildrenForNode = function mapChildrenForNode(children, func) {
	return mapChildren(children, function (node, index) {
		if (React.isValidElement(node)) {
			return func(node, index);
		}
		return node;
	});
};

var mapChildrenByType = function mapChildrenByType(children, type, func) {
	var typeList = Array.isArray(type) ? type : [type];

	return mapChildren(children, function (node, index) {
		if (React.isValidElement(node) && node.type && typeList.includes(node.type[BAMBOO_COMPONENT])) {
			if (func) return func(node, index);
			return node;
		}
		return null;
	});
};





var mapChildrenByNotType = function mapChildrenByNotType(children, type, func) {
	var typeList = Array.isArray(type) ? type : [type];

	return mapChildren(children, function (node, index) {
		if (React.isValidElement(node) && node.type && typeList.includes(node.type[BAMBOO_COMPONENT])) {
			return null;
		}

		if (func) return func(node, index);
		return node;
	});
};

var events = {};

function addUniqueListener(event, listener) {
	if (!canUseDOM) return;

	if (events[event] === undefined) {
		window.addEventListener(event, function () {
			if (events[event]) {
				events[event].apply(events, arguments);
			}
		});
	}

	events[event] = listener;
}

function removeUniqueListener(event, listener) {
	var currentListener = events[event];

	if (currentListener === listener) {
		events[event] = null;
		return true;
	}

	return false;
}

function isSameSource(event, target) {
	var current = event.target;

	while (current) {
		if (current === target || current._bmbo_ignore) {
			return true;
		}

		current = current.parentElement;
	}

	return false;
}

var wrapperEventValue = function wrapperEventValue(originEvent, target, value) {
	target.value = value; // eslint-disable-line no-param-reassign

	return new Proxy(originEvent, {
		get: function get(tgt, key) {
			if (key === 'target') return target;
			return tgt[key];
		}
	});
};

var BAMBOO_FORM_RADIO = 'BAMBOO_FORM_RADIO';

var Radio = function (_React$Component) {
	inherits(Radio, _React$Component);

	function Radio() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Radio);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Radio.__proto__ || Object.getPrototypeOf(Radio)).call.apply(_ref, [this].concat(args))), _this), _initialiseProps$1.call(_this), _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Radio, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    checked = _props.checked,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['children', 'checked', 'className']);


			delete props.checked;
			delete props.onClick;
			delete props.onChange;
			delete props.value;

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-radio', checked && 'bmbo-checked', className),
					role: 'button',
					tabIndex: 0,
					onClick: this.onClick
				}, props),
				React.createElement('input', { type: 'radio', checked: checked, readOnly: true }),
				React.createElement('span', { className: 'bmbo-radio-check' }),
				React.createElement(
					'div',
					{ className: 'bmbo-radio-content' },
					children
				)
			);
		}
	}]);
	return Radio;
}(React.Component);

var _initialiseProps$1 = function _initialiseProps() {
	var _this2 = this;

	this.onClick = function () {
		var _props2 = _this2.props,
		    onClick = _props2.onClick,
		    onChange = _props2.onChange,
		    value = _props2.value,
		    children = _props2.children;

		var event = arguments.length <= 0 ? undefined : arguments[0];

		if (onClick) onClick.apply(undefined, arguments);
		if (onChange) {
			onChange(wrapperEventValue(event, event.target, value !== undefined ? value : children));
		}
	};
};

Radio.propTypes = {
	children: propTypes.node,
	className: propTypes.string,

	checked: propTypes.bool,
	onClick: propTypes.func,
	onChange: propTypes.func,
	value: propTypes.node
};

Radio[BAMBOO_COMPONENT] = BAMBOO_FORM_RADIO;

var BAMBOO_FORM_CHECKBOX = 'BAMBOO_FORM_CHECKBOX';

var Checkbox = function Checkbox(_ref) {
	var children = _ref.children,
	    checked = _ref.checked,
	    disabled = _ref.disabled,
	    size = _ref.size,
	    onClick = _ref.onClick,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['children', 'checked', 'disabled', 'size', 'onClick', 'className']);
	return React.createElement(
		'div',
		_extends({
			className: classnames('bmbo-checkbox', 'bmbo-' + (size || 'md'), checked && 'bmbo-checked', disabled && 'bmbo-disabled', className),
			role: 'button',
			tabIndex: 0,
			onClick: disabled ? null : onClick
		}, props),
		React.createElement('input', { type: 'checkbox', checked: checked, disabled: disabled, readOnly: true }),
		React.createElement('span', { className: 'bmbo-checkbox-check' }),
		React.createElement(
			'div',
			{ className: 'bmbo-checkbox-content' },
			children
		)
	);
};

Checkbox.propTypes = {
	children: propTypes.node,
	className: propTypes.string,
	size: propTypes.string,
	disabled: propTypes.bool,

	checked: propTypes.bool,
	onClick: propTypes.func
};

Checkbox[BAMBOO_COMPONENT] = BAMBOO_FORM_CHECKBOX;

var BAMBOO_FORM_INPUT = 'BAMBOO_FORM_INPUT';

var Input = function (_React$PureComponent) {
	inherits(Input, _React$PureComponent);

	function Input() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Input);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Input.__proto__ || Object.getPrototypeOf(Input)).call.apply(_ref, [this].concat(args))), _this), _this.setRef = function (ele) {
			var setRef = _this.props[BAMBOO_INTERNAL_REF];
			_this.$input = ele;

			if (setRef) setRef(ele);
		}, _this.focus = function () {
			if (_this.$input) {
				_this.$input.focus();
			}
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Input, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    size = _props.size,
			    type = _props.type,
			    icon = _props.icon,
			    className = _props.className,
			    rest = objectWithoutProperties(_props, ['size', 'type', 'icon', 'className']);

			delete rest[BAMBOO_INTERNAL_REF];

			var $input = React.createElement('input', _extends({
				className: classnames('bmbo-input', icon && 'bmbo-with-icon', 'bmbo-' + (size || 'md'), type && 'bmbo-' + type, className),
				ref: this.setRef
			}, rest));

			if (icon) {
				return React.createElement(
					'div',
					{
						className: classnames('bmbo-input-container', 'bmbo-' + (size || 'md'))
					},
					$input,
					React.createElement(Icon, { name: icon, size: size, className: 'bmbo-input-icon' })
				);
			}

			return $input;
		}
	}]);
	return Input;
}(React.PureComponent);

Input.propTypes = {
	size: propTypes.string,
	type: propTypes.string,
	icon: propTypes.string,
	className: propTypes.string
};

Input[BAMBOO_FORM_INPUT] = BAMBOO_FORM_INPUT;

var getValue = function getValue(unit, path, defaultValue) {
	if (!path) return unit;

	var desList = Array.isArray(path) ? path : [path];
	var current = unit;
	var len = desList.length;

	for (var i = 0; i < len; i += 1) {
		var des = desList[i];
		current = current[des];

		if (current === undefined || i !== len - 1 && current === null) {
			current = defaultValue;
			break;
		}
	}

	return current;
};





var updateValue = function updateValue(unit, path, updateFunc) {
	var desList = Array.isArray(path) ? path : [path];
	var preLen = desList.length - 1;
	var instance = Array.isArray(unit) ? [].concat(toConsumableArray(unit)) : _extends({}, unit);
	var current = instance;
	var des = void 0;

	for (var i = 0; i < preLen; i += 1) {
		des = desList[i];
		var value = current[des];
		current = current[des] = Array.isArray(value) ? [].concat(toConsumableArray(value)) : _extends({}, value);
	}

	current[desList[preLen]] = typeof updateFunc === 'function' ? updateFunc(current[desList[preLen]]) : updateFunc;

	return instance;
};

var Field = function (_React$Component) {
	inherits(Field, _React$Component);

	function Field() {
		classCallCheck(this, Field);

		var _this = possibleConstructorReturn(this, (Field.__proto__ || Object.getPrototypeOf(Field)).call(this));

		_this.getPath = function () {
			var formPath = _this.context.formPath;
			var name = _this.props.name;


			return [].concat(toConsumableArray(formPath), [Array.isArray(name) ? name : [name]]);
		};

		_this.getValue = function () {
			var formInstance = _this.context.formInstance;


			if ('value' in _this.props) {
				return _this.props.value;
			}

			if (formInstance) {
				return getValue(formInstance, ['state'].concat(toConsumableArray(_this.getPath())));
			}
			return _this.state.value;
		};

		_this.getList = function () {
			var _this$context = _this.context,
			    formInstance = _this$context.formInstance,
			    onFormChanged = _this$context.onFormChanged;
			var _this$props = _this.props,
			    title = _this$props.title,
			    children = _this$props.children;

			var myValue = _this.getValue();

			return mapChildrenForNode(children, function (node) {
				var _node$props = node.props,
				    props = _node$props === undefined ? {} : _node$props;


				if (node.type[BAMBOO_COMPONENT] === BAMBOO_FORM_RADIO) {
					// ==================== Radio ====================
					var nodeValue = props.value !== undefined ? props.value : props.children;

					return React.cloneElement(node, {
						'aria-label': props.ariaLabel || title,
						onClick: function onClick() {
							if (props.onClick) props.onClick.apply(props, arguments);

							if (formInstance) {
								formInstance.setState(function (preState) {
									return updateValue(preState, _this.getPath(), function () {
										return nodeValue;
									});
								}, onFormChanged);
							} else {
								_this.setState({ value: nodeValue }, onFormChanged);
							}
						},
						checked: myValue === nodeValue
					});
				} else if (node.type[BAMBOO_COMPONENT] === BAMBOO_FORM_CHECKBOX) {
					// ================== Check Box ==================
					return React.cloneElement(node, {
						'aria-label': props.ariaLabel || title,
						onClick: function onClick() {
							if (props.onClick) props.onClick.apply(props, arguments);

							if (formInstance) {
								formInstance.setState(function (preState) {
									return updateValue(preState, _this.getPath(), function (value) {
										return !value;
									});
								}, onFormChanged);
							} else {
								_this.setState(function (_ref) {
									var value = _ref.value;
									return { value: !value };
								}, onFormChanged);
							}
						},
						checked: myValue
					});
				} else if (node.type[BAMBOO_COMPONENT] === BAMBOO_FORM_INPUT) {
					// ==================== Input ====================
					return React.cloneElement(node, {
						value: props.value !== undefined ? props.value : myValue,
						'aria-label': props.ariaLabel || title,
						onChange: function onChange() {
							for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
								args[_key] = arguments[_key];
							}

							if (props.onChange) props.onChange.apply(props, args);

							var newVal = args[0].target.value;

							if (props.type === 'number') newVal = Number(newVal);

							if (formInstance) {
								formInstance.setState(function (preState) {
									return updateValue(preState, _this.getPath(), function () {
										return newVal;
									});
								}, onFormChanged);
							} else {
								_this.setState({ value: newVal }, onFormChanged);
							}
						}
					});
				}

				// ==================== Rest =====================
				return React.cloneElement(node, {
					value: myValue,
					'aria-label': props.ariaLabel || title,
					onChange: function onChange() {
						for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
							args[_key2] = arguments[_key2];
						}

						if (props.onChange) props.onChange.apply(props, args);

						var newVal = args[0].target.value;

						if (formInstance) {
							formInstance.setState(function (preState) {
								return updateValue(preState, _this.getPath(), function () {
									return newVal;
								});
							}, onFormChanged);
						} else {
							_this.setState({ value: newVal }, onFormChanged);
						}
					}
				});
			});
		};

		_this.state = {
			value: null
		};
		return _this;
	}

	createClass(Field, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			var defaultValue = this.props.defaultValue;

			this.setState({ value: defaultValue });
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    name = _props.name,
			    title = _props.title,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['name', 'title', 'className']);


			delete props.value;
			delete props.defaultValue;

			var list = this.getList();

			return React.createElement(
				'div',
				_extends({ className: classnames('bmbp-form-field', className) }, props),
				title !== undefined && React.createElement(
					'label',
					{ htmlFor: name },
					title
				),
				list
			);
		}
	}]);
	return Field;
}(React.Component);

Field.propTypes = {
	children: propTypes.node,
	className: propTypes.string,

	name: propTypes.string,
	title: propTypes.node,
	value: propTypes.node,
	defaultValue: propTypes.node
};

Field.contextTypes = {
	formInstance: propTypes.object,
	formPath: propTypes.array,
	onFormChanged: propTypes.func
};

var Form = function (_React$Component) {
	inherits(Form, _React$Component);

	function Form() {
		classCallCheck(this, Form);

		var _this = possibleConstructorReturn(this, (Form.__proto__ || Object.getPrototypeOf(Form)).call(this));

		_this.onFormChanged = function () {
			var onChanged = _this.props.onChanged;

			if (onChanged) onChanged();
		};

		_this.getInstance = function () {
			return _this.props.instance || _this;
		};

		_this.state = {};
		return _this;
	}

	createClass(Form, [{
		key: 'getChildContext',
		value: function getChildContext() {
			var _props$path = this.props.path,
			    path = _props$path === undefined ? [] : _props$path;


			return {
				formInstance: this.getInstance(),
				formPath: Array.isArray(path) ? path : [path],
				onFormChanged: this.onFormChanged
			};
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['children', 'className']);


			delete props.instance;
			delete props.path;
			delete props.onChanged;

			return React.createElement(
				'div',
				_extends({ className: classnames('bmbo-form', className) }, props),
				children
			);
		}
	}]);
	return Form;
}(React.Component);

Form.propTypes = {
	children: propTypes.node,
	className: propTypes.string,

	instance: propTypes.object,
	path: propTypes.oneOfType([propTypes.string, propTypes.array]),
	onChanged: propTypes.func
};

Form.childContextTypes = {
	formInstance: propTypes.object,
	formPath: propTypes.array,
	onFormChanged: propTypes.func
};

Form.Field = Field;

var currentSlider = void 0;

function getEventPercentage(holder, event) {
	var clientX = event.clientX;

	var _holder$getBoundingCl = holder.getBoundingClientRect(),
	    left = _holder$getBoundingCl.left,
	    width = _holder$getBoundingCl.width;

	var ptg = (clientX - left) / width;
	if (ptg < 0) ptg = 0;
	if (ptg > 1) ptg = 1;

	return ptg;
}

function getLeft(value, min, max) {
	var offset = value - min;
	var len = max - min;

	return offset / len * 100 + '%';
}

function valueSort(a, b) {
	return a - b;
}

if (canUseDOM) {
	window.addEventListener('mousemove', function (event) {
		if (!currentSlider) return;

		var ptg = getEventPercentage(currentSlider.$pinHolder, event);

		currentSlider.setValuePtg(ptg, event);
		event.preventDefault();
	});

	window.addEventListener('mouseup', function () {
		if (currentSlider) {
			currentSlider.pinIndex = undefined;
			currentSlider.finishUpdate();
			currentSlider = null;
		}
	});
}

var Pin = function (_React$Component) {
	inherits(Pin, _React$Component);

	function Pin() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Pin);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Pin.__proto__ || Object.getPrototypeOf(Pin)).call.apply(_ref, [this].concat(args))), _this), _this.onMouseDown = function (event) {
			var _this$props = _this.props,
			    onMouseDown = _this$props.onMouseDown,
			    index = _this$props.index;

			if (onMouseDown) onMouseDown(event, index);
		}, _this.setRef = function (ele) {
			var _this$props2 = _this.props,
			    setRef = _this$props2.setRef,
			    index = _this$props2.index;

			if (setRef) setRef(ele, index);
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Pin, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    left = _props.left,
			    index = _props.index,
			    isMark = _props.isMark,
			    isActive = _props.isActive;


			return React.createElement('div', {
				className: classnames('bmbo-slider-pin', isMark && 'bmbo-mark', isActive && 'bmbo-active'),
				onMouseDown: this.onMouseDown,
				style: { left: left },
				role: 'button',
				tabIndex: -1,
				ref: this.setRef,
				'data-index': index
			});
		}
	}]);
	return Pin;
}(React.Component);

Pin.propTypes = {
	setRef: propTypes.func,
	left: propTypes.string,
	index: propTypes.number,
	onMouseDown: propTypes.func,
	isMark: propTypes.bool,
	isActive: propTypes.bool
};

var Slider = function (_React$Component2) {
	inherits(Slider, _React$Component2);

	function Slider() {
		classCallCheck(this, Slider);

		var _this2 = possibleConstructorReturn(this, (Slider.__proto__ || Object.getPrototypeOf(Slider)).call(this));

		_this2.onBarMouseDown = function () {
			for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
				args[_key2] = arguments[_key2];
			}

			var _this2$props = _this2.props,
			    disabled = _this2$props.disabled,
			    onMouseDown = _this2$props.onMouseDown,
			    value = _this2$props.value;

			if (disabled || _this2.getPinCount() > 1) return;

			if (onMouseDown) onMouseDown.apply(undefined, args);

			var event = args[0].nativeEvent;

			_this2.setState({
				value: value
			}, function () {
				_this2.pinIndex = 0;
				currentSlider = _this2;
				_this2.setValuePtg(getEventPercentage(_this2.$pinHolder, event), event);
			});
		};

		_this2.onPinMouseDown = function (event, index) {
			var _this2$props2 = _this2.props,
			    disabled = _this2$props2.disabled,
			    value = _this2$props2.value;

			if (disabled) return;

			_this2.setState({
				value: value
			}, function () {
				_this2.pinIndex = index;
				currentSlider = _this2;
			});

			event.stopPropagation();
		};

		_this2.setPinRef = function (ele, index) {
			_this2.$pins[index] = ele;
		};

		_this2.setPinHolderRef = function (ele) {
			_this2.$pinHolder = ele;
		};

		_this2.setValuePtg = function (ptg, event) {
			if (_this2.pinIndex === undefined) return;

			var onChange = _this2.props.onChange;

			var value = _this2.getValue();
			var _this2$props3 = _this2.props,
			    _this2$props3$min = _this2$props3.min,
			    min = _this2$props3$min === undefined ? 0 : _this2$props3$min,
			    _this2$props3$max = _this2$props3.max,
			    max = _this2$props3$max === undefined ? 100 : _this2$props3$max,
			    _this2$props3$step = _this2$props3.step,
			    step = _this2$props3$step === undefined ? 1 : _this2$props3$step;

			// Format number

			min = Number(min);
			max = Number(max);

			var len = max - min;
			var offset = void 0;
			var pinValue = void 0;

			var markValues = _this2.getMarkValues();

			if (step !== null) {
				step = Number(step);
				offset = Math.round(len * ptg / step) * step;
				pinValue = offset + min;

				// Stick mark if needed
				markValues.some(function (_ref2) {
					var markValue = _ref2.value,
					    stick = _ref2.stick;

					if (!stick) return false;

					if (markValue - stick <= pinValue && pinValue <= markValue + stick) {
						pinValue = markValue;
						return null;
					}

					return false;
				});
			} else {
				pinValue = min + len * ptg;
				var match = false;
				for (var i = markValues.length - 1; i > 0; i -= 1) {
					var cur = markValues[i].value;
					var prev = markValues[i - 1].value;
					if (pinValue >= prev) {
						match = true;
						pinValue = pinValue > (cur + prev) / 2 ? cur : prev;
						break;
					}
				}

				if (!match && markValues.length > 0) {
					pinValue = markValues[0].value;
				}
			}

			var newValue = void 0;
			if (Array.isArray(value)) {
				newValue = value.concat();
				newValue[_this2.pinIndex] = pinValue;
			} else if (_this2.getPinCount() === 1) {
				newValue = pinValue;
			} else {
				newValue = [];
				newValue[_this2.pinIndex] = pinValue;
			}

			var outputValue = newValue;

			// Clean up empty value
			if (Array.isArray(newValue)) {
				for (var _i = newValue.length - 1; _i >= 0; _i -= 1) {
					var val = newValue[_i];
					if (val === null || val === undefined) {
						newValue[_i] = min;
					}
				}

				outputValue = newValue.concat().sort(valueSort);
			}

			// Sync local value
			_this2.setState({
				value: newValue
			});

			var newEvent = wrapperEventValue(event, _this2.$pins[_this2.pinIndex], outputValue);

			if (onChange) {
				onChange(newEvent);
			}
		};

		_this2.getMarkValues = function () {
			var _this2$props4 = _this2.props,
			    _this2$props4$min = _this2$props4.min,
			    min = _this2$props4$min === undefined ? 0 : _this2$props4$min,
			    _this2$props4$max = _this2$props4.max,
			    max = _this2$props4$max === undefined ? 100 : _this2$props4$max,
			    _this2$props4$marks = _this2$props4.marks,
			    marks = _this2$props4$marks === undefined ? {} : _this2$props4$marks;

			// No need to sort since number key is always in order

			return Object.keys(marks).map(function (markValue) {
				var value = Number(markValue);
				if (isNaN(value) || value < min || value > max) {
					console.warn('[Bamboo - Slider] Mark is out of range:', markValue, '(Range: ' + min + ' ~ ' + max);
					return null;
				}

				var config = marks[value];
				if (typeof config === 'string') {
					config = {
						title: config
					};
				}

				return _extends({}, config, {
					value: value
				});
			}).filter(function (n) {
				return n !== null;
			});
		};

		_this2.getPinCount = function () {
			var multi = _this2.props.multi;

			if (!multi) return 1;
			return multi === true ? 2 : Math.max(multi, 1);
		};

		_this2.getValue = function () {
			if (_this2.isOperating()) {
				return _this2.state.value;
			}
			return _this2.props.value;
		};

		_this2.getPinValue = function (index) {
			var value = _this2.getValue();
			var _this2$props5 = _this2.props,
			    _this2$props5$min = _this2$props5.min,
			    min = _this2$props5$min === undefined ? 0 : _this2$props5$min,
			    _this2$props5$max = _this2$props5.max,
			    max = _this2$props5$max === undefined ? 100 : _this2$props5$max;

			var current = void 0;

			if (Array.isArray(value)) {
				current = value[index];
			} else if (_this2.getPinCount() === 1) {
				current = value;
			} else {
				current = min;
			}

			min = Number(min);
			max = Number(max);
			current = Number(current);

			if (current === null || current === undefined || isNaN(current) || current < min) current = min;
			if (current > max) current = max;

			return current;
		};

		_this2.isOperating = function () {
			return currentSlider === _this2;
		};

		_this2.finishUpdate = function () {
			// Sync slider value with props value when user finish dragging
			Promise.resolve().then(function () {
				_this2.forceUpdate();
			});
		};

		_this2.$pins = {};
		_this2.state = {
			value: null // Component handle value when user is operating
		};
		return _this2;
	}

	createClass(Slider, [{
		key: 'render',
		value: function render$$1() {
			var _props2 = this.props,
			    _props2$min = _props2.min,
			    min = _props2$min === undefined ? 0 : _props2$min,
			    _props2$max = _props2.max,
			    max = _props2$max === undefined ? 100 : _props2$max,
			    disabled = _props2.disabled,
			    transparent = _props2.transparent,
			    type = _props2.type,
			    size = _props2.size,
			    className = _props2.className,
			    props = objectWithoutProperties(_props2, ['min', 'max', 'disabled', 'transparent', 'type', 'size', 'className']);


			delete props.value;
			delete props.step;
			delete props.multi;
			delete props.marks;
			delete props.onMouseDown;
			delete props.onChange;

			var pinCount = this.getPinCount();

			var $pinList = [];
			var rangeMin = Number.MAX_VALUE;
			var rangeMax = Number.MIN_VALUE;

			for (var i = 0; i < pinCount; i += 1) {
				var value = this.getPinValue(i);

				// Pin list
				var left = getLeft(value, min, max);

				$pinList.push(React.createElement(Pin, {
					key: 'pin_' + i,
					index: i,
					setRef: this.setPinRef,
					onMouseDown: this.onPinMouseDown,
					left: left
				}));

				// Get limitation
				if (pinCount > 1) {
					rangeMin = Math.min(rangeMin, value);
				} else {
					rangeMin = min;
				}
				rangeMax = Math.max(rangeMax, value);
			}

			// Pin range
			var rangeWidthPtg = (rangeMax - rangeMin) / (max - min);

			// Marks
			var $markList = [];
			var $markTitleList = [];

			this.getMarkValues().forEach(function (_ref3) {
				var value = _ref3.value,
				    title = _ref3.title;

				var left = getLeft(value, min, max);

				// Mark
				$markList.push(React.createElement(Pin, {
					key: 'mark_' + value,
					left: left,
					isMark: true,
					isActive: rangeMin <= value && value <= rangeMax
				}));

				// Title
				$markTitleList.push(React.createElement(
					'div',
					{ key: 'title_' + value, className: 'bmbo-title-holder', style: { left: left } },
					React.createElement(
						'div',
						{ className: 'bmbo-title' },
						title
					)
				));
			});

			return React.createElement(
				'div',
				{
					className: classnames('bmbo-slider', pinCount === 1 && 'bmbo-single', 'bmbo-' + (type || 'primary'), 'bmbo-' + (size || 'md'), disabled && 'bmbo-disabled', transparent && 'bmbo-transparent', className),

					onMouseDown: this.onBarMouseDown
				},
				React.createElement(
					'div',
					{ className: 'bmbo-slider-content' },
					React.createElement('div', { className: 'bmbo-slider-bar', ref: this.setPinHolderRef }),
					React.createElement('div', {
						className: 'bmbo-slider-range',
						style: {
							left: getLeft(rangeMin, min, max),
							width: rangeWidthPtg * 100 + '%'
						}
					}),
					$markList,
					$pinList
				),
				React.createElement(
					'div',
					{ className: 'bmbo-slider-title' },
					$markTitleList
				)
			);
		}
	}]);
	return Slider;
}(React.Component);

Slider.propTypes = {
	className: propTypes.string,
	value: propTypes.oneOfType([propTypes.number, propTypes.array]),
	min: propTypes.number,
	max: propTypes.number,
	step: propTypes.number,
	type: propTypes.string,
	size: propTypes.string,
	transparent: propTypes.bool,
	disabled: propTypes.bool,
	multi: propTypes.oneOfType([propTypes.bool, propTypes.number]),
	marks: propTypes.object,

	onMouseDown: propTypes.func,
	onChange: propTypes.func
};

var equals = function equals() {
	var list1 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
	var list2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
	var path = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

	var len = list1.length;
	if (len !== list2.length) return false;

	for (var i = 0; i < len; i += 1) {
		var item1 = getValue(list1[i], path);
		var item2 = getValue(list2[i], path);
		if (item1 !== item2) return false;
	}

	return true;
};

var toArray$1 = function toArray(value) {
	if (Array.isArray(value)) return value;
	if (!value) return [];
	return [value];
};

var EmptyWrapper = function (_React$Component) {
	inherits(EmptyWrapper, _React$Component);

	function EmptyWrapper() {
		classCallCheck(this, EmptyWrapper);
		return possibleConstructorReturn(this, (EmptyWrapper.__proto__ || Object.getPrototypeOf(EmptyWrapper)).apply(this, arguments));
	}

	createClass(EmptyWrapper, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    props = objectWithoutProperties(_props, ['children']);

			return React.cloneElement(children, props);
		}
	}]);
	return EmptyWrapper;
}(React.Component);

EmptyWrapper.propTypes = {
	children: propTypes.node
};

var Box = function (_React$Component) {
	inherits(Box, _React$Component);

	function Box() {
		classCallCheck(this, Box);

		var _this = possibleConstructorReturn(this, (Box.__proto__ || Object.getPrototypeOf(Box)).call(this));

		_this.onTransitionEnd = function (event) {
			if (_this.state.animateStatus === ANIMATE_STATUS_HIDING && event.target === _this.$ele) {
				_this.seq.next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				});
			}
		};

		_this.setRef = function (ele) {
			_this.$ele = ele;

			// Let window event handler ignore this element
			if (_this.$ele) _this.$ele._bmbo_ignore = true;
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (!prevProps.visible === !nextProps.visible) return;

			var onVisibilityChanged = nextProps.onVisibilityChanged;


			if (nextProps.visible) {
				_this.seq.next(function (done) {
					var rect = nextProps.rect,
					    stretch = nextProps.stretch;


					_this.setState({
						animateStatus: ANIMATE_STATUS_INIT,
						x: 0,
						y: 0,
						width: stretch ? rect.width : null
					}, done);
				}, { frame: 1 }).next(function (done) {
					var rect = nextProps.rect;

					var pinRect = _this.$ele.getBoundingClientRect();

					if (onVisibilityChanged) {
						onVisibilityChanged(true);
					}

					_this.setState(_extends({
						animateStatus: ANIMATE_STATUS_SHOWING
					}, getEnablePosition(rect, pinRect, 'b')), done);
				}, { frame: 1 }).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_SHOWN });
				});
			} else {
				_this.seq.next(function (done) {
					_this.setState({
						animateStatus: ANIMATE_STATUS_HIDING
					}, done);
				}, { frame: 1 }).next(function () {
					if (onVisibilityChanged) {
						onVisibilityChanged(false);
					}
				}, { delay: 0 }).next(function () {
					/** Reset to none status.
      ** This will not trigger if `onTransitionEnd` works.
      */
					_this.setState({
						animateStatus: ANIMATE_STATUS_NONE
					});
				}, { delay: 3000 });
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE,
			x: 0,
			y: 0,
			width: null
		};

		_this.seq = new Sequence();
		return _this;
	}

	createClass(Box, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.seq.destroy();
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _state = this.state,
			    animateStatus = _state.animateStatus,
			    x = _state.x,
			    y = _state.y,
			    width = _state.width,
			    direct = _state.direct;

			if (animateStatus === ANIMATE_STATUS_NONE) return null;

			var _props = this.props,
			    children = _props.children,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['children', 'className']);

			delete props.visible;
			delete props.rect;
			delete props.stretch;
			delete props.onVisibilityChanged;

			var style = {
				left: x + 'px',
				top: y + 'px'
			};

			if (width !== null) {
				style.width = width + 'px';
			}

			return createPortal$1(React.createElement(
				'div',
				_extends({}, props, {

					style: style,
					className: classnames('bmbo-pin-box', {
						'bmbo-init': animateStatus === ANIMATE_STATUS_INIT,
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
						'bmbo-hiding': animateStatus === ANIMATE_STATUS_HIDING
					}, 'bmbo-' + direct, className),
					onTransitionEnd: this.onTransitionEnd,
					ref: this.setRef
				}),
				children
			));
		}
	}]);
	return Box;
}(React.Component);

Box.propTypes = {
	visible: propTypes.bool,
	rect: propTypes.object,
	stretch: propTypes.bool,
	onVisibilityChanged: propTypes.func,

	children: propTypes.node,
	className: propTypes.string
};

var PinBox = function (_React$Component) {
	inherits(PinBox, _React$Component);

	function PinBox() {
		classCallCheck(this, PinBox);

		var _this = possibleConstructorReturn(this, (PinBox.__proto__ || Object.getPrototypeOf(PinBox)).call(this));

		_this.onBackdrop = function (event) {
			if (isSameSource(event, reactDom.findDOMNode(_this.$ele))) return;

			_this.setState({ visible: false }, _this.onVisibilityChange);
			removeUniqueListener('mousedown', _this.onBackdrop);
		};

		_this.onClick = function () {
			var _this$props = _this.props,
			    backdrop = _this$props.backdrop,
			    disabled = _this$props.disabled;


			if (disabled) return;

			_this.setState(function (_ref) {
				var visible = _ref.visible;

				var newState = { visible: !visible };

				if (newState.visible && _this.$ele) {
					/** Use `findDOMNode` to support customize component.
      ** To avoid user need to support setRef for this component.
      **/
					newState.rect = reactDom.findDOMNode(_this.$ele) // eslint-disable-line react/no-find-dom-node
					.getBoundingClientRect();

					if (backdrop) {
						addUniqueListener('mousedown', _this.onBackdrop);
					}
				}

				return newState;
			}, _this.onVisibilityChange);
		};

		_this.onVisibilityChange = function () {
			var onVisibilityChange = _this.props.onVisibilityChange;
			var visible = _this.state.visible;


			if (onVisibilityChange) onVisibilityChange(visible);
		};

		_this.setRef = function (ele) {
			_this.$ele = ele;
		};

		_this.setVisible = function (visible) {
			_this.setState({ visible: visible }, _this.onVisibilityChange);
		};

		_this.state = {
			visible: false
		};
		return _this;
	}

	createClass(PinBox, [{
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			removeUniqueListener('mousedown', this.onBackdrop);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var _props = this.props,
			    trigger = _props.trigger,
			    children = _props.children,
			    pin = _props.pin,
			    stretch = _props.stretch,
			    onVisibilityChanged = _props.onVisibilityChanged,
			    props = objectWithoutProperties(_props, ['trigger', 'children', 'pin', 'stretch', 'onVisibilityChanged']);
			var _state = this.state,
			    visible = _state.visible,
			    rect = _state.rect;


			delete props.onVisibilityChange;
			delete props.backdrop;

			var $children = mapChildrenForNode(children, function (node) {
				var onClick = node.props.onClick,
				    _ref2 = node.ref;

				var newProps = {
					ref: function ref() {
						if (_ref2) _ref2.apply(undefined, arguments);
						_this2.setRef.apply(_this2, arguments);
					}
				};

				if (!trigger || trigger === 'click') {
					// Click Trigger
					newProps.onClick = function () {
						if (onClick) onClick.apply(undefined, arguments);
						_this2.onClick.apply(_this2, arguments);
					};
				} else {
					console.warn('[PinBox] Trigger not realize:', trigger);
				}

				// Transform children to class component to avoid stateless function can't use ref
				return React.createElement(
					EmptyWrapper,
					newProps,
					node
				);
			});

			return [].concat(toConsumableArray($children), [pin ? React.createElement(
				Box,
				_extends({
					key: '__BMBO_PIN_BOX_BOX__'
				}, props, {
					visible: visible,
					rect: rect,
					stretch: stretch,
					onVisibilityChanged: onVisibilityChanged
				}),
				pin
			) : null]);
		}
	}]);
	return PinBox;
}(React.Component);

PinBox.propTypes = {
	trigger: propTypes.string,
	backdrop: propTypes.bool,
	stretch: propTypes.bool,
	children: propTypes.node,
	pin: propTypes.node,
	disabled: propTypes.bool,

	onVisibilityChange: propTypes.func,
	onVisibilityChanged: propTypes.func
};

var BAMBOO_FORM_SELECT_OPTION = 'BAMBOO_FORM_SELECT_OPTION';

var SelectOption = function (_React$Component) {
	inherits(SelectOption, _React$Component);

	function SelectOption() {
		classCallCheck(this, SelectOption);

		var _this = possibleConstructorReturn(this, (SelectOption.__proto__ || Object.getPrototypeOf(SelectOption)).call(this));

		_this.onClick = function () {
			for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
				args[_key] = arguments[_key];
			}

			var _this$props = _this.props,
			    onClick = _this$props.onClick,
			    value = _this$props.value,
			    children = _this$props.children;
			var bmboOnSelectValue = _this.context.bmboOnSelectValue;


			var myVal = value !== undefined ? value : children;

			if (bmboOnSelectValue) bmboOnSelectValue.apply(undefined, [myVal].concat(args));
			if (onClick) onClick.apply(undefined, args);
		};

		_this.state = {};
		return _this;
	}

	createClass(SelectOption, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    disabled = _props.disabled,
			    value = _props.value,
			    children = _props.children,
			    props = objectWithoutProperties(_props, ['disabled', 'value', 'children']);
			var _context = this.context,
			    bmboSelectMulti = _context.bmboSelectMulti,
			    bmboSelectDisabled = _context.bmboSelectDisabled,
			    bmboSelectSize = _context.bmboSelectSize,
			    bmboSelectIsChecked = _context.bmboSelectIsChecked;


			var myDisabled = void 0;
			if (disabled !== undefined) {
				myDisabled = disabled;
			} else if (bmboSelectDisabled !== undefined) {
				myDisabled = bmboSelectDisabled;
			}

			var $children = children;
			if (bmboSelectMulti) {
				$children = React.createElement(
					Checkbox,
					{
						size: bmboSelectSize,
						checked: bmboSelectIsChecked(value === undefined ? children : value),
						disabled: myDisabled
					},
					children
				);
			}

			return React.createElement(
				'li',
				_extends({
					role: 'button',
					tabIndex: -1,
					className: classnames('bmbo-select-item bmbo-padding', myDisabled && 'bmbo-disabled')
				}, props, {
					onClick: myDisabled ? null : this.onClick
				}),
				$children
			);
		}
	}]);
	return SelectOption;
}(React.Component);

SelectOption.propTypes = {
	onClick: propTypes.func,
	children: propTypes.node,
	value: propTypes.node,
	disabled: propTypes.bool
};

SelectOption.contextTypes = {
	bmboSelectSize: propTypes.string,
	bmboSelectMulti: propTypes.bool,
	bmboSelectDisabled: propTypes.bool,
	bmboOnSelectValue: propTypes.func,
	bmboSelectIsChecked: propTypes.func
};

SelectOption[BAMBOO_COMPONENT] = BAMBOO_FORM_SELECT_OPTION;

var BAMBOO_FORM_SELECT_GROUP = 'BAMBOO_FORM_SELECT_GROUP';

function getNodeList() {
	var children = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
	var parentProps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

	var valueList = [];

	mapChildrenByType(children, BAMBOO_FORM_SELECT_OPTION, function (node) {
		var _node$props = node.props,
		    nodeVal = _node$props.value,
		    nodeChildren = _node$props.children,
		    disabled = _node$props.disabled;

		var myVal = nodeVal !== undefined ? nodeVal : nodeChildren;
		valueList.push({
			title: nodeChildren,
			value: myVal,
			disabled: disabled !== undefined ? disabled : parentProps.disabled
		});
	});

	mapChildrenByType(children, BAMBOO_FORM_SELECT_GROUP, function (node) {
		valueList = valueList.concat(getNodeList(node.props.children, node.props || {}));
	});

	return valueList;
}

function getValueList() {
	var children = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

	return getNodeList(children).map(function (_ref) {
		var value = _ref.value;
		return value;
	});
}

var SelectGroup = function (_React$Component) {
	inherits(SelectGroup, _React$Component);

	function SelectGroup() {
		var _ref2;

		var _temp, _this, _ret;

		classCallCheck(this, SelectGroup);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref2 = SelectGroup.__proto__ || Object.getPrototypeOf(SelectGroup)).call.apply(_ref2, [this].concat(args))), _this), _this.selectAllValue = function (event) {
			var children = _this.props.children;
			var bmboOnSelectValues = _this.context.bmboOnSelectValues;

			var valueList = getValueList(children);

			bmboOnSelectValues(event, valueList, !_this.isAllSelected());
		}, _this.isAllSelected = function () {
			var children = _this.props.children;
			var bmboSelectIsChecked = _this.context.bmboSelectIsChecked;

			var valueList = getValueList(children);

			return valueList.every(bmboSelectIsChecked);
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(SelectGroup, [{
		key: 'getChildContext',
		value: function getChildContext() {
			var bmboSelectDisabled = this.context.bmboSelectDisabled;


			return {
				bmboSelectDisabled: this.props.disabled !== undefined ? this.props.disabled : bmboSelectDisabled
			};
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    title = _props.title,
			    children = _props.children,
			    noSelectAll = _props.noSelectAll,
			    disabled = _props.disabled,
			    props = objectWithoutProperties(_props, ['title', 'children', 'noSelectAll', 'disabled']);
			var _context = this.context,
			    bmboSelectMulti = _context.bmboSelectMulti,
			    bmboSelectSize = _context.bmboSelectSize,
			    bmboSelectDisabled = _context.bmboSelectDisabled;


			var myDisabled = void 0;
			if (disabled !== undefined) {
				myDisabled = disabled;
			} else if (bmboSelectDisabled !== undefined) {
				myDisabled = bmboSelectDisabled;
			}

			var $title = title;
			var selectAllProps = {};

			if (bmboSelectMulti && !noSelectAll) {
				$title = React.createElement(
					Checkbox,
					{ size: bmboSelectSize, disabled: myDisabled, checked: this.isAllSelected() },
					title
				);

				selectAllProps = {
					role: 'button',
					tabIndex: -1,
					onClick: myDisabled ? null : this.selectAllValue
				};
			} else {
				$title = React.createElement(
					'span',
					null,
					title
				);
			}

			return React.createElement(
				'li',
				_extends({
					className: 'bmbo-' + (bmboSelectSize || 'md') + ' bmbo-select-group'
				}, props),
				React.createElement(
					'div',
					_extends({
						className: 'bmbo-select-group-title bmbo-padding'
					}, selectAllProps),
					$title
				),
				React.createElement(
					'ul',
					null,
					children
				)
			);
		}
	}]);
	return SelectGroup;
}(React.Component);

SelectGroup.propTypes = {
	children: propTypes.node,
	title: propTypes.node,
	disabled: propTypes.bool,
	noSelectAll: propTypes.bool
};

SelectGroup.childContextTypes = {
	bmboSelectDisabled: propTypes.bool
};

SelectGroup.contextTypes = {
	bmboSelectSize: propTypes.string,
	bmboSelectMulti: propTypes.bool,
	bmboSelectDisabled: propTypes.bool,
	bmboOnSelectValue: propTypes.func,
	bmboOnSelectValues: propTypes.func,
	bmboSelectIsChecked: propTypes.func
};

SelectGroup[BAMBOO_COMPONENT] = BAMBOO_FORM_SELECT_GROUP;

function getTitle(optionList, value) {
	var option = optionList.find(function (opt) {
		return opt.value === value;
	});
	if (!option) return null;

	return option.title;
}

var Select = function (_React$Component) {
	inherits(Select, _React$Component);

	function Select() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Select);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Select.__proto__ || Object.getPrototypeOf(Select)).call.apply(_ref, [this].concat(args))), _this), _this.onSelectValue = function (value, event) {
			var _this$props = _this.props,
			    onChange = _this$props.onChange,
			    multi = _this$props.multi,
			    oriValue = _this$props.value;


			var newValue = void 0;
			if (multi) {
				newValue = oriValue;
				if (!Array.isArray(newValue)) {
					newValue = newValue ? [newValue] : [];
				}

				if (newValue.includes(value)) {
					newValue = newValue.filter(function (v) {
						return v !== value;
					});
				} else {
					newValue = [].concat(toConsumableArray(newValue), [value]);
				}
			} else {
				newValue = value;
			}

			if (onChange) {
				onChange(wrapperEventValue(event, _this.$ele, newValue));
			}

			if (!multi && _this.$pin) {
				_this.$pin.setVisible(false);
			}
		}, _this.onSelectValues = function (event, valueList, checked) {
			var _this$props2 = _this.props,
			    children = _this$props2.children,
			    value = _this$props2.value,
			    onChange = _this$props2.onChange;

			var myList = valueList || getValueList(children);
			var myChecked = checked !== undefined ? checked : !_this.isAllValueChecked();
			var newValue = [].concat(toConsumableArray(toArray$1(value)));

			if (myChecked) {
				myList.forEach(function (val) {
					if (!newValue.includes(val)) {
						newValue.push(val);
					}
				});
			} else {
				newValue = newValue.filter(function (val) {
					return !myList.includes(val);
				});
			}

			if (onChange) {
				onChange(wrapperEventValue(event, _this.$ele, newValue));
			}
		}, _this.onSelectAllValue = function (event) {
			_this.onSelectValues(event);
		}, _this.setRef = function (ele) {
			_this.$ele = ele;
		}, _this.setPinRef = function (ele) {
			_this.$pin = ele;
		}, _this.isValueChecked = function (val) {
			var value = _this.props.value;

			return toArray$1(value).includes(val);
		}, _this.isAllValueChecked = function () {
			var _this$props3 = _this.props,
			    value = _this$props3.value,
			    children = _this$props3.children;

			var myValue = toArray$1(value);

			var allValueList = getValueList(children);
			return allValueList.every(function (val) {
				return myValue.includes(val);
			});
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Select, [{
		key: 'getChildContext',
		value: function getChildContext() {
			return {
				bmboSelectSize: this.props.size,
				bmboSelectMulti: this.props.multi,
				bmboSelectDisabled: this.props.disabled,
				bmboOnSelectValue: this.onSelectValue,
				bmboOnSelectValues: this.onSelectValues,
				bmboSelectIsChecked: this.isValueChecked,
				bmboSelectIsAllChecked: this.isAllValueChecked
			};
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    disabled = _props.disabled,
			    size = _props.size,
			    title = _props.title,
			    className = _props.className,
			    value = _props.value,
			    multi = _props.multi,
			    flatten = _props.flatten,
			    noSelectAll = _props.noSelectAll,
			    children = _props.children,
			    props = objectWithoutProperties(_props, ['disabled', 'size', 'title', 'className', 'value', 'multi', 'flatten', 'noSelectAll', 'children']);

			delete props.onChange;

			// =============================== Value ===============================
			var optionList = getNodeList(children);

			var $value = void 0;
			if (title !== undefined) {
				$value = title || '\xA0';
			} else if (multi) {
				var valueList = toArray$1(value);
				$value = React.createElement(
					'ul',
					{ className: 'bmbo-list-inline' },
					valueList.map(function (val) {
						return React.createElement(
							'li',
							{ key: val },
							getTitle(optionList, val)
						);
					}),
					!valueList.length && React.createElement(
						'li',
						null,
						'\xA0'
					)
				);
			} else {
				$value = getTitle(optionList, value) || '\xA0';
			}

			// ================================ List ===============================
			var canSelectAll = !noSelectAll && optionList.every(function (opt) {
				return !opt.disabled;
			});

			var $list = React.createElement(
				'ul',
				{
					className: classnames('bmbo-select-list', 'bmbo-list-unstyled', 'bmbo-' + (size || 'md'))
				},
				multi && canSelectAll && React.createElement(
					'li',
					{
						className: 'bmbo-select-item bmbo-padding bmbo-select-item-all',
						onClick: this.onSelectAllValue,
						role: 'button',
						tabIndex: -1
					},
					React.createElement(
						Checkbox,
						{ size: size, checked: this.isAllValueChecked() },
						'Select All'
					)
				),
				children
			);

			// =============================== Render ==============================
			if (multi && flatten) {
				return React.createElement(
					'div',
					_extends({
						className: classnames('bmbo-select', 'bmbo-select-flatten', disabled && 'bmbo-disabled')
					}, props, {
						ref: this.setRef
					}),
					$list
				);
			}

			return React.createElement(
				PinBox,
				{ pin: $list, ref: this.setPinRef, disabled: disabled, backdrop: true, stretch: true },
				React.createElement(
					'div',
					_extends({
						className: classnames('bmbo-select', disabled && 'bmbo-disabled')
					}, props, {
						ref: this.setRef
					}),
					React.createElement(
						'div',
						{
							className: classnames('bmbo-select-value', 'bmbo-' + (size || 'md'), className),

							role: 'button',
							tabIndex: 0
						},
						$value,
						React.createElement(Caret, { direct: 'down' })
					)
				)
			);
		}
	}]);
	return Select;
}(React.Component);

Select.propTypes = {
	onChange: propTypes.func,
	size: propTypes.string,
	className: propTypes.string,
	title: propTypes.node,
	value: propTypes.any,
	children: propTypes.node,
	multi: propTypes.bool,
	disabled: propTypes.bool,
	noSelectAll: propTypes.bool,
	flatten: propTypes.bool
};

Select.childContextTypes = {
	bmboSelectSize: propTypes.string,
	bmboSelectMulti: propTypes.bool,
	bmboSelectDisabled: propTypes.bool,
	bmboSelectIsChecked: propTypes.func,
	bmboSelectIsAllChecked: propTypes.func,
	bmboOnSelectValue: propTypes.func,
	bmboOnSelectValues: propTypes.func
};

Select.Option = SelectOption;
Select.Group = SelectGroup;

var MenuItem = function (_React$Component) {
	inherits(MenuItem, _React$Component);

	function MenuItem() {
		classCallCheck(this, MenuItem);

		var _this = possibleConstructorReturn(this, (MenuItem.__proto__ || Object.getPrototypeOf(MenuItem)).call(this));

		_this.onClick = function () {
			var _this$props$item = _this.props.item,
			    disabled = _this$props$item.disabled,
			    onClick = _this$props$item.onClick;


			if (disabled) return;

			var hideMenu = _this.context.hideMenu;


			if (hideMenu) hideMenu();
			if (onClick) onClick.apply(undefined, arguments);
		};

		_this.onMouseDown = function (event) {
			event.preventDefault();
		};

		_this.onMouseEnter = function () {
			var list = _this.props.item.list;


			_this.setState({
				active: true
			});

			if (!list) return;

			var rect = _this.$item.getBoundingClientRect();

			_this.setState({
				drillDown: list,
				rect: rect
			});
		};

		_this.onMouseLeave = function () {
			_this.setState({
				active: false,
				drillDown: null
			});
		};

		_this.setRef = function (ele) {
			_this.$item = ele;
		};

		_this.state = {
			drillDown: undefined,
			rect: undefined,
			active: false
		};
		return _this;
	}

	createClass(MenuItem, [{
		key: 'render',
		value: function render$$1() {
			var _props$item = this.props.item,
			    separator = _props$item.separator,
			    title = _props$item.title,
			    disabled = _props$item.disabled,
			    list = _props$item.list;
			var _state = this.state,
			    drillDown = _state.drillDown,
			    _state$rect = _state.rect;
			_state$rect = _state$rect === undefined ? {} : _state$rect;
			var left = _state$rect.left,
			    top = _state$rect.top,
			    width = _state$rect.width,
			    height = _state$rect.height,
			    active = _state.active;


			if (separator) {
				return React.createElement('li', { className: 'bmbo-separator' });
			}

			return React.createElement(
				'li',
				{
					className: classnames('bmbo-menu-item', disabled && 'bmbo-disabled', !disabled && active && 'bmbo-active'),
					role: 'button',
					tabIndex: -1,
					ref: this.setRef,
					onClick: this.onClick,
					onMouseDown: this.onMouseDown,
					onMouseEnter: this.onMouseEnter,
					onMouseLeave: this.onMouseLeave
				},
				title,
				list && React.createElement(Caret, null),
				drillDown && React.createElement(MenuList, {
					list: drillDown,
					x: left,
					y: top - 1,
					width: width,
					height: height
				})
			);
		}
	}]);
	return MenuItem;
}(React.Component);

MenuItem.propTypes = {
	item: propTypes.object
};

MenuItem.contextTypes = {
	hideMenu: propTypes.func
};

var MenuList = function (_React$Component) {
	inherits(MenuList, _React$Component);

	function MenuList() {
		classCallCheck(this, MenuList);

		var _this = possibleConstructorReturn(this, (MenuList.__proto__ || Object.getPrototypeOf(MenuList)).call(this));

		_this.onMouseDown = function (event) {
			event.stopPropagation();
		};

		_this.setRef = function (ele) {
			_this.$list = ele;
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE
		};
		_this.seq = new Sequence();
		return _this;
	}

	createClass(MenuList, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			var _this2 = this;

			var _props = this.props,
			    x = _props.x,
			    y = _props.y,
			    _props$width = _props.width,
			    width = _props$width === undefined ? 0 : _props$width,
			    _props$height = _props.height,
			    height = _props$height === undefined ? 0 : _props$height;

			var rect = this.$list.getBoundingClientRect();

			this.seq.next(function () {
				_this2.setState(_extends({}, getEnablePosition({ left: x, top: y, width: width, height: height }, rect, 'r'), {
					animateStatus: ANIMATE_STATUS_SHOWING
				}));
			}).next(function () {
				_this2.setState({
					animateStatus: ANIMATE_STATUS_SHOWN
				});
			});
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.seq.destroy();
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props$list = this.props.list,
			    list = _props$list === undefined ? [] : _props$list;
			var menuSize = this.context.menuSize;
			var _state = this.state,
			    x = _state.x,
			    y = _state.y,
			    animateStatus = _state.animateStatus;


			return createPortal$1(React.createElement(
				'ul',
				{
					ref: this.setRef,
					className: classnames('bmbo-box-absolute', 'bmbo-menu-list', {
						'bmbo-hidden': animateStatus === ANIMATE_STATUS_NONE,
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING
					}, 'bmbo-' + (menuSize || 'md')),
					style: { left: x + 'px', top: y + 'px' },
					role: 'presentation',
					tabIndex: -1,
					onMouseDown: this.onMouseDown
				},
				list.map(function (item, index) {
					if (!item) return null;
					return React.createElement(MenuItem, { key: index, item: item });
				})
			));
		}
	}]);
	return MenuList;
}(React.Component);

MenuList.propTypes = {
	list: propTypes.array,
	x: propTypes.number,
	y: propTypes.number,
	width: propTypes.number,
	height: propTypes.number
};

MenuList.contextTypes = {
	menuSize: propTypes.string
};

var $currentMenu = void 0;

if (canUseDOM) {
	window.addEventListener('mousedown', function () {
		if ($currentMenu) {
			$currentMenu.hideMenu();
			$currentMenu = null;
		}
	});
}

var Menu = function (_React$Component) {
	inherits(Menu, _React$Component);

	function Menu() {
		classCallCheck(this, Menu);

		var _this = possibleConstructorReturn(this, (Menu.__proto__ || Object.getPrototypeOf(Menu)).call(this));

		_this.onContextMenu = function (event) {
			event.preventDefault();

			_this.setState({
				show: true,
				x: event.clientX,
				y: event.clientY
			});

			$currentMenu = _this;
		};

		_this.hideMenu = function (callback) {
			_this.setState({
				show: false
			}, callback);
		};

		_this.state = {
			show: false,
			x: 0,
			y: 0
		};
		return _this;
	}

	createClass(Menu, [{
		key: 'getChildContext',
		value: function getChildContext() {
			return {
				hideMenu: this.hideMenu,
				menuSize: this.props.size
			};
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    _props$menu = _props.menu,
			    menu = _props$menu === undefined ? [] : _props$menu,
			    children = _props.children,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['menu', 'children', 'className']);
			var _state = this.state,
			    show = _state.show,
			    x = _state.x,
			    y = _state.y;


			delete props.size;

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-menu-wrapper', className),
					onContextMenu: this.onContextMenu
				}, props),
				children,
				show && React.createElement(MenuList, { list: menu, x: x, y: y })
			);
		}
	}]);
	return Menu;
}(React.Component);

Menu.propTypes = {
	children: propTypes.node,
	className: propTypes.string,
	menu: propTypes.array,
	size: propTypes.string
};

Menu.childContextTypes = {
	hideMenu: propTypes.func,
	menuSize: propTypes.string
};

var BAMBOO_GROUP_ITEM = 'BAMBOO_GROUP_ITEM';

var Item = function (_React$Component) {
	inherits(Item, _React$Component);

	function Item() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Item);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Item.__proto__ || Object.getPrototypeOf(Item)).call.apply(_ref, [this].concat(args))), _this), _this.setRef = function (ele) {
			_this.$ele = ele;
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Item, [{
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var _props = this.props,
			    _props$style = _props.style,
			    style = _props$style === undefined ? {} : _props$style,
			    width = _props.width,
			    children = _props.children,
			    isFirst = _props.isFirst,
			    isLast = _props.isLast,
			    value = _props.value,
			    onChange = _props.onChange,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['style', 'width', 'children', 'isFirst', 'isLast', 'value', 'onChange', 'className']);

			if (width) style.width = width;

			var $child = React.Children.only(children);

			if (React.isValidElement($child) && $child.type && $child.type[BAMBOO_COMPONENT] === BAMBOO_FORM_RADIO) {
				var _$child$props = $child.props,
				    _onClick = _$child$props.onClick,
				    type = _$child$props.type,
				    transparent = _$child$props.transparent,
				    restProps = objectWithoutProperties(_$child$props, ['onClick', 'type', 'transparent']);

				var checked = restProps.value === value;

				$child = React.createElement(Button, _extends({
					checked: checked,
					type: checked || !transparent ? type : 'weak',
					transparent: transparent
				}, restProps, {
					ref: this.setRef,
					onClick: function onClick() {
						var event = arguments.length <= 0 ? undefined : arguments[0];
						if (onChange) onChange(wrapperEventValue(event, _this2.$ele, restProps.value));
						if (_onClick) _onClick.apply(undefined, arguments);
					}
				}));
			}

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-group-item', {
						'bmbo-first': isFirst && !isLast,
						'bmbo-last': isLast && !isFirst,
						'bmbo-middle': !isFirst && !isLast,
						className: className
					}),
					style: style
				}, props),
				$child
			);
		}
	}]);
	return Item;
}(React.Component);

Item.propTypes = {
	children: propTypes.node,
	className: propTypes.string,
	style: propTypes.object,
	width: propTypes.string,
	isFirst: propTypes.bool,
	isLast: propTypes.bool,
	value: propTypes.node,
	onChange: propTypes.func
};

Item[BAMBOO_COMPONENT] = BAMBOO_GROUP_ITEM;

var Group = function Group(_ref) {
	var children = _ref.children,
	    value = _ref.value,
	    onChange = _ref.onChange,
	    _ref$style = _ref.style,
	    style = _ref$style === undefined ? {} : _ref$style,
	    width = _ref.width,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['children', 'value', 'onChange', 'style', 'width', 'className']);

	var myStyle = _extends({}, style);
	if (width) myStyle.width = width;

	// Wrap component with Item
	var itemList = mapChildrenForNode(children, function (node) {
		var item = node;
		if (!node.type || node.type[BAMBOO_COMPONENT] !== BAMBOO_GROUP_ITEM) {
			item = React.createElement(
				Item,
				null,
				node
			);
		}

		return item;
	}).filter(function (n) {
		return n;
	});
	var count = itemList.length;

	// Mark first & last Item
	var $children = itemList.map(function (node, index) {
		return React.cloneElement(node, {
			isFirst: index === 0,
			isLast: index === count - 1,
			value: value,
			onChange: onChange
		});
	});

	return React.createElement(
		'div',
		_extends({
			className: classnames('bmbo-group', className),
			style: myStyle
		}, props),
		$children
	);
};

Group.propTypes = {
	children: propTypes.node,
	className: propTypes.string,
	value: propTypes.node,
	onChange: propTypes.func,
	style: propTypes.object,
	width: propTypes.string
};

Group.Item = Item;

var BAMBOO_MODAL_TITLE = 'BAMBOO_MODAL_TITLE';

var ModalTitle = function ModalTitle(_ref) {
	var children = _ref.children,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['children', 'className']);
	return React.createElement(
		'div',
		_extends({ className: classnames('bmbo-modal-title', className) }, props),
		children
	);
};

ModalTitle.propTypes = {
	children: propTypes.node,
	className: propTypes.string
};

ModalTitle[BAMBOO_COMPONENT] = BAMBOO_MODAL_TITLE;

var BAMBOO_MODAL_CONTENT = 'BAMBOO_MODAL_CONTENT';

var ModalContent = function ModalContent(_ref) {
	var children = _ref.children,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['children', 'className']);
	return React.createElement(
		'div',
		_extends({ className: classnames('bmbo-modal-body', className) }, props),
		children
	);
};

ModalContent.propTypes = {
	children: propTypes.node,
	className: propTypes.string
};

ModalContent[BAMBOO_COMPONENT] = BAMBOO_MODAL_CONTENT;

var BAMBOO_MODAL_FOOTER = 'BAMBOO_MODAL_FOOTER';

var ModalFooter = function ModalFooter(_ref) {
	var children = _ref.children,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['children', 'className']);
	return React.createElement(
		'div',
		_extends({ className: classnames('bmbo-modal-footer', className) }, props),
		children
	);
};

ModalFooter.propTypes = {
	children: propTypes.node,
	className: propTypes.string
};

ModalFooter[BAMBOO_COMPONENT] = BAMBOO_MODAL_FOOTER;

var BAMBOO_MODAL_PREFACE = 'BAMBOO_MODAL_PREFACE';

var ModalPreface = function (_React$Component) {
	inherits(ModalPreface, _React$Component);

	function ModalPreface() {
		classCallCheck(this, ModalPreface);

		var _this = possibleConstructorReturn(this, (ModalPreface.__proto__ || Object.getPrototypeOf(ModalPreface)).call(this));

		_this.setContentRef = function (ele) {
			_this.$content = ele;
		};

		_this.checkUpdate = function (preProps, nextProps) {
			if (!preProps.visible !== !nextProps.visible) {
				if (nextProps.visible) {
					var _nextProps$delay = nextProps.delay,
					    delay = _nextProps$delay === undefined ? 400 : _nextProps$delay;

					// Show preface

					_this.seq.next(function () {
						_this.setState({
							animateStatus: ANIMATE_STATUS_SHOWING,
							contentWidth: 0
						});
					}).next(function () {
						var newState = { animateStatus: ANIMATE_STATUS_SHOWN };

						if (_this.$content) {
							newState.contentWidth = _this.$content.scrollWidth;
						}

						_this.setState(newState);
					}, { delay: delay });
				} else {
					// Hide preface
					_this.seq.next(function () {
						_this.setState({
							animateStatus: ANIMATE_STATUS_NONE
						});
					});
				}
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE,
			contentWidth: 0
		};
		_this.seq = new Sequence();
		return _this;
	}

	createClass(ModalPreface, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.seq.destroy();
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['children', 'className']);
			var _state = this.state,
			    animateStatus = _state.animateStatus,
			    contentWidth = _state.contentWidth;


			delete props.delay;
			delete props.visible;

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-modal-preface', {
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
						className: className
					}),
					style: { width: contentWidth ? contentWidth + 'px' : null }
				}, props),
				React.createElement(
					'div',
					{ className: 'bmbo-modal-preface-content', ref: this.setContentRef },
					children
				)
			);
		}
	}]);
	return ModalPreface;
}(React.Component);

ModalPreface.propTypes = {
	delay: propTypes.number,
	children: propTypes.node,
	className: propTypes.string,
	visible: propTypes.bool // eslint-disable-line react/no-unused-prop-types
};

ModalPreface[BAMBOO_COMPONENT] = BAMBOO_MODAL_PREFACE;

var BAMBOO_MODAL_LOADING = 'BAMBOO_MODAL_LOADING';

var ModalLoading = function ModalLoading(_ref) {
	var children = _ref.children,
	    loading = _ref.loading,
	    defaultView = _ref.defaultView,
	    className = _ref.className,
	    props = objectWithoutProperties(_ref, ['children', 'loading', 'defaultView', 'className']);

	var $content = children;

	if (defaultView) {
		$content = React.createElement(
			'div',
			{ className: classnames('bmbo-default-loading') },
			React.createElement(Icon.Loading, null)
		);
	}

	return React.createElement(
		'div',
		_extends({
			className: classnames('bmbo-modal-loading', {
				'bmbo-shown': loading,
				className: className
			})
		}, props),
		$content
	);
};

ModalLoading.propTypes = {
	loading: propTypes.bool,
	children: propTypes.node,
	className: propTypes.string,
	defaultView: propTypes.bool
};

ModalLoading[BAMBOO_COMPONENT] = BAMBOO_MODAL_LOADING;

var modalList = [];

// ========================================================
// =                          UI                          =
// ========================================================
function refreshWinScrollBar() {
	if (!canUseDOM) return;

	var hasActiveModal = modalList.some(function (_ref) {
		var _ref$state = _ref.state;
		_ref$state = _ref$state === undefined ? {} : _ref$state;
		var animateStatus = _ref$state.animateStatus;
		return animateStatus !== ANIMATE_STATUS_NONE;
	});

	if (hasActiveModal) {
		disableWinScrollBar('Modal');
	} else {
		enableWinScrollBar('Modal');
	}
}

// ========================================================
// =                         Modal                        =
// ========================================================

var Modal$1 = function (_React$Component) {
	inherits(Modal, _React$Component);

	function Modal() {
		classCallCheck(this, Modal);

		var _this = possibleConstructorReturn(this, (Modal.__proto__ || Object.getPrototypeOf(Modal)).call(this));

		_this.onBackgroundClick = function () {
			var noMaskClose = _this.props.noMaskClose;

			if (noMaskClose) return;

			// Put close request in next frame
			_this.closeSeq.next(function () {
				var onClose = _this.props.onClose;

				if (onClose) onClose();
			});
		};

		_this.onHolderClick = function () {
			// Occupy the close request if is holder click
			_this.closeSeq.next(function () {}, { priority: 1 });
		};

		_this.onTransitionEnd = function (event) {
			var onClosed = _this.props.onClosed;
			var animateStatus = _this.state.animateStatus;


			if (event.target === _this.$holder && animateStatus === ANIMATE_STATUS_HIDING) {
				if (onClosed) onClosed(event);

				_this.seq.next(function () {
					_this.setState({
						animateStatus: ANIMATE_STATUS_NONE
					}, refreshWinScrollBar);
				});
			}
		};

		_this.setHolderRef = function (ele) {
			_this.$holder = ele;
		};

		_this.getTitle = function () {
			var _this$props = _this.props,
			    children = _this$props.children,
			    title = _this$props.title;


			if (title) return React.createElement(
				ModalTitle,
				null,
				title
			);
			return mapChildrenByType(children, BAMBOO_MODAL_TITLE);
		};

		_this.getBody = function () {
			var _this$props2 = _this.props,
			    children = _this$props2.children,
			    content = _this$props2.content;


			if (content) return React.createElement(
				ModalContent,
				null,
				content
			);

			var $children = mapChildrenByType(children, BAMBOO_MODAL_CONTENT);
			if ($children.length) return $children;

			return React.createElement(
				ModalContent,
				null,
				mapChildrenByNotType(children, [BAMBOO_MODAL_TITLE, BAMBOO_MODAL_FOOTER, BAMBOO_MODAL_PREFACE, BAMBOO_MODAL_LOADING])
			);
		};

		_this.getFooter = function () {
			var _this$props3 = _this.props,
			    children = _this$props3.children,
			    footer = _this$props3.footer,
			    onClose = _this$props3.onClose,
			    onConfirm = _this$props3.onConfirm,
			    _this$props3$closeTex = _this$props3.closeText,
			    closeText = _this$props3$closeTex === undefined ? 'Close' : _this$props3$closeTex,
			    _this$props3$confirmT = _this$props3.confirmText,
			    confirmText = _this$props3$confirmT === undefined ? 'Confirm' : _this$props3$confirmT,
			    lock = _this$props3.lock;


			if (footer) return React.createElement(
				ModalFooter,
				null,
				footer
			);

			var $footer = mapChildrenByType(children, BAMBOO_MODAL_FOOTER);
			if ($footer.length) return $footer;

			return React.createElement(
				ModalFooter,
				null,
				React.createElement(
					Button,
					{ type: 'weak', onClick: onClose, disabled: lock, transparent: true },
					closeText
				),
				onConfirm && React.createElement(
					Button,
					{ onClick: onConfirm, disabled: lock },
					confirmText
				)
			);
		};

		_this.getLoading = function () {
			var children = _this.props.children;

			var $loading = mapChildrenByType(children, BAMBOO_MODAL_LOADING);

			if ($loading.length) return $loading;

			return React.createElement(ModalLoading, { defaultView: true });
		};

		_this.getPreface = function () {
			var _this$props4 = _this.props,
			    children = _this$props4.children,
			    visible = _this$props4.visible;

			return mapChildrenByType(children, BAMBOO_MODAL_PREFACE, function (node) {
				return React.cloneElement(node, { visible: visible });
			});
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			// Process when visibility changed
			if (!prevProps.visible !== !nextProps.visible) {
				if (nextProps.visible) {
					// Show modal
					_this.seq.next(function () {
						_this.setState({
							animateStatus: ANIMATE_STATUS_SHOWING
						}, refreshWinScrollBar);
					}).next(function () {
						_this.setState({ animateStatus: ANIMATE_STATUS_SHOWN });
					});
				} else {
					// Hide modal
					_this.seq.next(function () {
						_this.setState({
							animateStatus: ANIMATE_STATUS_HIDING
						});
					}).next(function () {
						// Clean up when browser not support transitionEnd event
						var onClosed = _this.props.onClosed;

						_this.setState({ animateStatus: ANIMATE_STATUS_NONE }, refreshWinScrollBar);

						if (onClosed) onClosed();
					}, { delay: 750 });
				}
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE
		};

		_this.seq = new Sequence();
		_this.closeSeq = new Sequence();
		return _this;
	}

	createClass(Modal, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			modalList.push(this);
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			var _this2 = this;

			this.seq.destroy();
			modalList = modalList.filter(function (mdl) {
				return mdl !== _this2;
			});
			refreshWinScrollBar();
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    size = _props.size,
			    type = _props.type,
			    loading = _props.loading,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['size', 'type', 'loading', 'className']);
			var animateStatus = this.state.animateStatus;


			if (animateStatus === ANIMATE_STATUS_NONE) return null;

			var $title = this.getTitle();
			var $body = this.getBody();
			var $footer = this.getFooter();
			var $preface = this.getPreface();
			var $loading = this.getLoading();

			var $modal = React.createElement(
				'div',
				{ className: 'bmbo-dialog' },
				$title,
				$body,
				$footer,
				$loading
			);

			delete props.visible;
			delete props.lock;
			delete props.title;
			delete props.content;
			delete props.footer;
			delete props.onClose;
			delete props.onClosed;
			delete props.onConfirm;
			delete props.closeText;
			delete props.confirmText;
			delete props.noMaskClose;

			return createPortal$1(React.createElement(
				'div',
				_extends({}, props, {
					className: classnames('bmbo-modal', {
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
						'bmbo-hidding': animateStatus === ANIMATE_STATUS_HIDING,
						'bmbo-loading': loading,
						className: className
					}, 'bmbo-modal-' + (size || 'md'), type && 'bmbo-' + type),
					role: 'presentation',
					onClick: this.onBackgroundClick
				}),
				React.createElement(
					'div',
					{
						className: 'bmbo-dialog-holder',
						role: 'presentation',
						onClick: this.onHolderClick,
						ref: this.setHolderRef,
						onTransitionEnd: this.onTransitionEnd
					},
					$preface,
					$modal
				)
			));
		}
	}]);
	return Modal;
}(React.Component);

Modal$1.propTypes = {
	visible: propTypes.bool,
	children: propTypes.node,
	className: propTypes.string,
	size: propTypes.string,
	type: propTypes.string,
	loading: propTypes.bool,
	lock: propTypes.bool,

	title: propTypes.node,
	content: propTypes.node,
	footer: propTypes.node,

	onClose: propTypes.func,
	onClosed: propTypes.func,
	onConfirm: propTypes.func,

	closeText: propTypes.func,
	confirmText: propTypes.func,

	noMaskClose: propTypes.bool
};

Modal$1.Title = ModalTitle;
Modal$1.Content = ModalContent;
Modal$1.Footer = ModalFooter;
Modal$1.Preface = ModalPreface;
Modal$1.Loading = ModalLoading;

var ModalFlexHolder = function (_React$Component) {
	inherits(ModalFlexHolder, _React$Component);

	function ModalFlexHolder() {
		classCallCheck(this, ModalFlexHolder);

		var _this = possibleConstructorReturn(this, (ModalFlexHolder.__proto__ || Object.getPrototypeOf(ModalFlexHolder)).call(this));

		_this.showDialog = function (config) {
			_this.setState(function (_ref) {
				var list = _ref.list;
				return {
					list: [].concat(toConsumableArray(list), [{
						visible: true,
						config: config
					}])
				};
			});
		};

		_this.lockDialog = function (index) {
			var lock = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

			_this.setState(function (_ref2) {
				var list = _ref2.list;

				var newList = list.concat();
				newList[index] = _extends({}, list[index], {
					lock: lock
				});

				return { list: newList };
			});
		};

		_this.hideDialog = function (index) {
			_this.setState(function (_ref3) {
				var list = _ref3.list;

				var newList = list.concat();
				newList[index] = _extends({}, list[index], {
					visible: false
				});

				return { list: newList };
			});
		};

		_this.removeDialog = function (index) {
			_this.setState(function (_ref4) {
				var list = _ref4.list;

				var dialog = list[index];
				return {
					list: list.filter(function (d) {
						return d !== dialog;
					})
				};
			});
		};

		_this.state = {
			list: []
		};
		return _this;
	}

	createClass(ModalFlexHolder, [{
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var list = this.state.list;


			return React.createElement(
				'div',
				null,
				list.map(function (dialog, index) {
					var config = dialog.config,
					    visible = dialog.visible,
					    lock = dialog.lock;
					var _onClose = config.onClose,
					    onConfirm = config.onConfirm;


					var closeWrapper = function closeWrapper(func) {
						var ret = void 0;
						if (func) ret = func();

						_this2.lockDialog(index, true);

						Promise.resolve(ret).then(function (close) {
							if (close === false) {
								_this2.lockDialog(index, false);
								return;
							}

							_this2.hideDialog(index);
						}).catch(function () {
							_this2.lockDialog(index, false);
						});
					};

					var props = {
						onClose: function onClose() {
							closeWrapper(_onClose);
						},
						onClosed: function onClosed() {
							_this2.removeDialog(index);
						}
					};

					if (onConfirm) {
						props.onConfirm = function () {
							closeWrapper(onConfirm);
						};
					}

					return React.createElement(Modal$1, _extends({
						key: index
					}, config, {

						visible: visible,
						lock: lock
					}, props));
				})
			);
		}
	}]);
	return ModalFlexHolder;
}(React.Component);

ModalFlexHolder.propTypes = {};

var BAMBOO_MODAL_FLEX_HOLDER = 'bmbo_modal_flex_holder';

var $modalFlexHolder = void 0;
if (canUseDOM) {
	// Clean up menu holder for hot module reload
	window.document.querySelectorAll('#' + BAMBOO_MODAL_FLEX_HOLDER).forEach(function (ele) {
		ele.remove();
	});

	// Set up menu holder
	var $flexHolder = window.document.createElement('div');
	$flexHolder.id = BAMBOO_MODAL_FLEX_HOLDER;

	reactDom.render(React.createElement(ModalFlexHolder, {
		ref: function ref(node) {
			$modalFlexHolder = node;
		}
	}), $flexHolder);
}

Modal$1.showDialog = function (config) {
	if (!$modalFlexHolder) {
		console.error('[Modal] Flex holder not defined.(Isomorphic render?)');
		return;
	}

	$modalFlexHolder.showDialog(config);
};

var Curtain = function (_React$Component) {
	inherits(Curtain, _React$Component);

	function Curtain() {
		classCallCheck(this, Curtain);

		var _this = possibleConstructorReturn(this, (Curtain.__proto__ || Object.getPrototypeOf(Curtain)).call(this));

		_this.onTransitionEnd = function (event) {
			if (_this.state.animateStatus === ANIMATE_STATUS_HIDING && _this.$curtain === event.target) {
				_this.seq.next(function () {
					enableWinScrollBar('Curtain');
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				});
			}
		};

		_this.setCurtainRef = function (ele) {
			_this.$curtain = ele;
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (!prevProps.visible === !nextProps.visible) return;

			if (nextProps.visible) {
				_this.seq.next(function () {
					disableWinScrollBar('Curtain');
					_this.setState({ animateStatus: ANIMATE_STATUS_SHOWING });
				}).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_SHOWN });
				});
			} else {
				_this.seq.next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_HIDING });
				}).next(function () {
					enableWinScrollBar('Curtain');
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				}, { delay: 1000 });
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE
		};
		_this.seq = new Sequence();
		return _this;
	}

	createClass(Curtain, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    className = _props.className,
			    onClose = _props.onClose,
			    props = objectWithoutProperties(_props, ['children', 'className', 'onClose']);
			var animateStatus = this.state.animateStatus;


			if (animateStatus === ANIMATE_STATUS_NONE) return null;

			delete props.visible;

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-curtain', {
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
						'bmbo-hiding': animateStatus === ANIMATE_STATUS_HIDING
					}, className),

					ref: this.setCurtainRef,
					onTransitionEnd: this.onTransitionEnd
				}, props),
				React.createElement(
					'div',
					{ className: 'bmbo-curtain-content-holder' },
					React.createElement(
						'div',
						{ className: 'bmbo-curtain-content' },
						children,
						onClose && React.createElement('a', {
							role: 'button',
							tabIndex: -1,
							onClick: onClose,
							className: 'bmbo-curtain-close bmbo-times'
						})
					)
				)
			);
		}
	}]);
	return Curtain;
}(React.Component);

Curtain.propTypes = {
	visible: propTypes.bool,
	children: propTypes.node,
	className: propTypes.string,
	onClose: propTypes.func
};

var Progress = function (_React$Component) {
	inherits(Progress, _React$Component);

	function Progress() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Progress);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Progress.__proto__ || Object.getPrototypeOf(Progress)).call.apply(_ref, [this].concat(args))), _this), _this.getPercentage = function () {
			var _this$props = _this.props,
			    min = _this$props.min,
			    max = _this$props.max,
			    value = _this$props.value;

			var myValue = value;
			if (myValue > max) myValue = max;
			if (myValue < min) myValue = min;
			return Math.floor((myValue - min) * 100 / (max - min));
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Progress, [{
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    type = _props.type,
			    size = _props.size,
			    active = _props.active,
			    showMark = _props.showMark,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['type', 'size', 'active', 'showMark', 'className']);

			var ptg = this.getPercentage();

			delete props.min;
			delete props.max;
			delete props.value;

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-progress', 'bmbo-' + (type || 'primary'), 'bmbo-' + (size || 'md'), active && 'bmbo-active', className),
					role: 'progressbar'
				}, props),
				React.createElement(
					'div',
					{ className: 'bmbo-progress-bar', style: { width: ptg + '%' } },
					showMark && React.createElement(
						'span',
						{ className: 'bmbo-mark' },
						ptg,
						'%'
					)
				)
			);
		}
	}]);
	return Progress;
}(React.Component);

Progress.propTypes = {
	className: propTypes.string,
	min: propTypes.number,
	max: propTypes.number,
	value: propTypes.number,

	type: propTypes.string,
	size: propTypes.string,
	active: propTypes.bool,
	showMark: propTypes.bool
};

Progress.defaultProps = {
	min: 0,
	max: 100,
	value: 100
};

var ARROW_DES = 5;

var Tooltip = function (_React$Component) {
	inherits(Tooltip, _React$Component);

	function Tooltip() {
		classCallCheck(this, Tooltip);

		var _this = possibleConstructorReturn(this, (Tooltip.__proto__ || Object.getPrototypeOf(Tooltip)).call(this));

		_this.onTransitionEnd = function (event) {
			if (_this.state.animateStatus === ANIMATE_STATUS_HIDING && event.target === _this.$holder) {
				_this.seq.next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				});
			}
		};

		_this.setHolderRef = function (ele) {
			_this.$holder = ele;
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (!prevProps.visible === !nextProps.visible) return;

			if (nextProps.visible) {
				_this.seq.next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_INIT, x: 0, y: 0 });
				}).next(function () {
					var rect = _this.props.rect;

					if (!_this.$holder) return;

					var ps = void 0;
					switch (nextProps.placement) {
						case 'bottom':
							ps = 'b,lr';
							break;
						case 'left':
							ps = 'l,tb';
							break;
						case 'right':
							ps = 'r,tb';
							break;
						default:
							ps = 't,lr';
					}

					var left = rect.left,
					    width = rect.width,
					    top = rect.top,
					    height = rect.height;

					var tgtRect = _this.$holder.getBoundingClientRect();

					var _getEnablePosition = getEnablePosition({
						left: left - ARROW_DES,
						top: top - ARROW_DES,
						width: width + 2 * ARROW_DES,
						height: height + 2 * ARROW_DES
					}, tgtRect, ps),
					    _x = _getEnablePosition._x,
					    _y = _getEnablePosition._y,
					    pos = objectWithoutProperties(_getEnablePosition, ['_x', '_y']);

					var arrowOffsetX = _x - pos.x;
					var arrowOffsetY = _y - pos.y;

					_this.setState(_extends({
						animateStatus: ANIMATE_STATUS_SHOWING
					}, pos, {
						arrowOffsetX: arrowOffsetX,
						arrowOffsetY: arrowOffsetY
					}));
				}).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_SHOWN });
				});
			} else {
				_this.seq.next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_HIDING });
				}).next(function () {
					_this.setState({ animateStatus: ANIMATE_STATUS_NONE });
				}, { delay: 1000 });
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE,
			x: 0,
			y: 0
		};
		_this.seq = new Sequence();
		return _this;
	}

	createClass(Tooltip, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    children = _props.children,
			    placement = _props.placement,
			    maxWidth = _props.maxWidth;
			var _state = this.state,
			    animateStatus = _state.animateStatus,
			    x = _state.x,
			    y = _state.y,
			    arrowOffsetX = _state.arrowOffsetX,
			    arrowOffsetY = _state.arrowOffsetY;


			if (animateStatus === ANIMATE_STATUS_NONE) return null;

			return createPortal$1(React.createElement(
				'div',
				{
					className: classnames('bmbo-tooltip', {
						'bmbo-hidden': animateStatus === ANIMATE_STATUS_INIT,
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
						'bmbo-hiding': animateStatus === ANIMATE_STATUS_HIDING
					}, 'bmbo-' + (placement || 'top')),
					style: { left: x + 'px', top: y + 'px' },
					ref: this.setHolderRef,
					onTransitionEnd: this.onTransitionEnd
				},
				React.createElement(
					'div',
					{
						className: 'bmbo-tooltip-title',
						style: { maxWidth: maxWidth && maxWidth + 'px' }
					},
					children
				),
				React.createElement('div', {
					className: 'bmbo-tooltip-arrow',
					style: { marginLeft: arrowOffsetX + 'px', marginTop: arrowOffsetY + 'px' }
				})
			));
		}
	}]);
	return Tooltip;
}(React.Component);

Tooltip.propTypes = {
	visible: propTypes.bool,
	maxWidth: propTypes.number,
	rect: propTypes.object,
	children: propTypes.node,
	placement: propTypes.string
};

var TooltipHolder = function (_React$Component) {
	inherits(TooltipHolder, _React$Component);

	function TooltipHolder() {
		classCallCheck(this, TooltipHolder);

		var _this = possibleConstructorReturn(this, (TooltipHolder.__proto__ || Object.getPrototypeOf(TooltipHolder)).call(this));

		_this.onMouseEnter = function (event) {
			var rect = event.currentTarget.getBoundingClientRect();
			_this.setState({ hover: true, rect: rect });
		};

		_this.onMouseLeave = function () {
			_this.setState({ hover: false });
		};

		_this.state = {
			hover: false
		};
		return _this;
	}

	createClass(TooltipHolder, [{
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var _props = this.props,
			    title = _props.title,
			    children = _props.children,
			    placement = _props.placement,
			    maxWidth = _props.maxWidth,
			    className = _props.className,
			    inject = _props.inject,
			    props = objectWithoutProperties(_props, ['title', 'children', 'placement', 'maxWidth', 'className', 'inject']);
			var _state = this.state,
			    hover = _state.hover,
			    rect = _state.rect;


			if (inject) {
				var childList = React.Children.map(children, function (node) {
					return React.cloneElement(node, {
						onMouseEnter: _this2.onMouseEnter,
						onMouseLeave: _this2.onMouseLeave,
						ref: _this2.setHolderRef
					});
				});

				return childList.concat(React.createElement(
					Tooltip,
					{ key: 'bmbo-tooltip', visible: hover, rect: rect, placement: placement, maxWidth: maxWidth },
					title
				));
			}

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-tooltip-holder', className),
					onMouseEnter: this.onMouseEnter,
					onMouseLeave: this.onMouseLeave
				}, props),
				children,
				React.createElement(
					Tooltip,
					{ visible: hover, rect: rect, placement: placement, maxWidth: maxWidth },
					title
				)
			);
		}
	}]);
	return TooltipHolder;
}(React.Component);

TooltipHolder.propTypes = {
	className: propTypes.string,
	title: propTypes.node,
	children: propTypes.node,
	placement: propTypes.string,
	maxWidth: propTypes.number,
	inject: propTypes.bool
};

var DynamicNumber = function (_React$Component) {
	inherits(DynamicNumber, _React$Component);

	function DynamicNumber() {
		classCallCheck(this, DynamicNumber);

		var _this = possibleConstructorReturn(this, (DynamicNumber.__proto__ || Object.getPrototypeOf(DynamicNumber)).call(this));

		_this.doAnimation = function (props) {
			var value = props.value,
			    duration = props.duration,
			    step = props.step,
			    decimal = props.decimal;

			var currentValue = Number(_this.state.current);

			// Ignore if value not change
			if (value === currentValue) {
				_this.seq.stop();
				return;
			}

			var startValue = isNaN(currentValue) ? 0 : currentValue;

			if (step) {
				// Use step
				var myStep = Math.abs(step);
				if (isNaN(myStep) || myStep === 0) myStep = 1;

				var frameStep = currentValue < value ? myStep : -myStep;
				var current = currentValue;

				_this.seq.next(function () {
					if (Math.abs(current - value) <= myStep) {
						_this.setState({ current: value.toFixed(decimal) });
						return false;
					}

					current += frameStep;
					_this.setState({ current: current.toFixed(decimal) });

					return null;
				}, {
					loop: true
				});
			} else {
				// Use duration
				var startTime = Date.now();

				_this.seq.next(function () {
					var currentTime = Date.now();
					var distance = currentTime - startTime;

					if (distance >= duration) {
						_this.setState({ current: value.toFixed(decimal) });
						return false;
					}

					var current = (startValue + (value - startValue) * (distance / duration)).toFixed(decimal);
					_this.setState({ current: current });

					return null;
				}, {
					loop: true
				});
			}
		};

		_this.state = {
			current: 0
		};

		_this.seq = new Sequence();
		return _this;
	}

	createClass(DynamicNumber, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.doAnimation(this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.doAnimation(nextProps);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.seq.destroy();
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var current = this.state.current;

			var props = _extends({}, this.props);
			delete props.value;
			delete props.duration;
			delete props.decimal;
			delete props.step;

			return React.createElement(
				'span',
				props,
				current
			);
		}
	}]);
	return DynamicNumber;
}(React.Component);

DynamicNumber.propTypes = {
	value: propTypes.number,
	duration: propTypes.number,
	decimal: propTypes.number,
	step: propTypes.number
};

DynamicNumber.defaultProps = {
	duration: 1000,
	decimal: 0
};

var DynamicString = function (_React$Component) {
	inherits(DynamicString, _React$Component);

	function DynamicString() {
		classCallCheck(this, DynamicString);

		var _this = possibleConstructorReturn(this, (DynamicString.__proto__ || Object.getPrototypeOf(DynamicString)).call(this));

		_this.doAnimation = function (props) {
			var value = props.value,
			    duration = props.duration,
			    step = props.step;

			var currentValue = _this.state.current;

			// Ignore if value not change
			if (value === currentValue) {
				_this.seq.stop();
				return;
			}

			var totalLen = currentValue.length + value.length;
			if (totalLen === 0) {
				// Skip if not word left
				_this.seq.stop();
				_this.setState({ current: '' });
				return;
			}

			var hidePtg = currentValue.length / totalLen;

			if (step) {
				// Use step
				var hiding = true;
				var current = currentValue;
				_this.seq.next(function () {
					if (hiding) {
						var len = Math.max(0, current.length - step);
						current = current.slice(0, len);
						_this.setState({ current: current });

						if (len === 0) hiding = false;
					} else {
						current = value.slice(0, current.length + step);
						_this.setState({ current: current });

						if (current.length === value.length) return false;
					}

					return null;
				}, {
					loop: true
				});
			} else {
				// Use duration
				var startTime = Date.now();
				_this.seq.next(function () {
					var currentTime = Date.now();
					var distance = currentTime - startTime;

					if (distance >= duration) {
						_this.setState({ current: value });
						return false;
					}

					var ptg = distance / duration;
					if (ptg < hidePtg) {
						var desPtg = (hidePtg - ptg) / hidePtg;
						_this.setState({
							current: currentValue.slice(0, Math.floor(currentValue.length * desPtg))
						});
					} else {
						var _desPtg = (ptg - hidePtg) / (1 - hidePtg);
						_this.setState({ current: value.slice(0, Math.floor(value.length * _desPtg)) });
					}

					return null;
				}, {
					loop: true
				});
			}
		};

		_this.state = {
			current: ''
		};

		_this.seq = new Sequence();
		return _this;
	}

	createClass(DynamicString, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.doAnimation(this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.doAnimation(nextProps);
		}
	}, {
		key: 'componentWillUnmount',
		value: function componentWillUnmount() {
			this.seq.destroy();
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var current = this.state.current;
			var _props = this.props,
			    cursor = _props.cursor,
			    props = objectWithoutProperties(_props, ['cursor']);

			delete props.value;
			delete props.duration;
			delete props.step;

			return React.createElement(
				'span',
				props,
				current,
				cursor && React.createElement('span', { className: 'bmbo-text-cursor' })
			);
		}
	}]);
	return DynamicString;
}(React.Component);

DynamicString.propTypes = {
	value: propTypes.string,
	duration: propTypes.number,
	step: propTypes.number,
	cursor: propTypes.bool
};

DynamicString.defaultProps = {
	duration: 1000
};

var List = function (_React$PureComponent) {
	inherits(List, _React$PureComponent);

	function List() {
		classCallCheck(this, List);

		var _this = possibleConstructorReturn(this, (List.__proto__ || Object.getPrototypeOf(List)).call(this));

		_this.onContainerTransitionEnd = function (event) {
			// Set state to NONE when hide animation done
			if (event.target !== _this.$container || _this.state.animateStatus !== ANIMATE_STATUS_HIDING) return;
			_this.seq.next(function () {
				_this.setState({
					animateStatus: ANIMATE_STATUS_NONE
				});
			});
		};

		_this.setContainerRef = function (ele) {
			_this.$container = ele;
		};

		_this.setListRef = function (ele) {
			_this.$list = ele;
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			var list = nextProps.list,
			    rect = nextProps.rect;

			if (prevProps.list === list || !prevProps.list && !list) return;
			if (prevProps.list && list && equals(prevProps.list, list, ['value'])) return;

			if (!list || list.length === 0) {
				// Hide list
				_this.seq.next(function () {
					_this.setState({
						height: 0,
						animateStatus: ANIMATE_STATUS_HIDING
					});
				});
			} else {
				// Show list
				var height = void 0;

				_this.seq.next(function (done) {
					_this.setState(function (preState) {
						var myState = {
							currentList: list,
							width: rect.width
						};

						if (preState.animateStatus === ANIMATE_STATUS_NONE) {
							myState.animateStatus = ANIMATE_STATUS_INIT;
						} else {
							myState.animateStatus = ANIMATE_STATUS_SHOWING;
						}

						return myState;
					}, done);
				}).next(function (done) {
					var listRect = _this.$list.getBoundingClientRect();
					height = listRect.height;

					_this.setState(_extends({}, getEnablePosition(rect, listRect, 'b')), done);
				}).next(function () {
					_this.setState({
						height: height,
						animateStatus: ANIMATE_STATUS_SHOWN
					});
				});
			}
		};

		_this.state = {
			x: 0,
			y: 0,
			animateStatus: ANIMATE_STATUS_NONE,

			height: 0,
			width: 0,
			currentList: null
		};

		_this.seq = new Sequence();
		return _this;
	}

	createClass(List, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _state = this.state,
			    currentList = _state.currentList,
			    height = _state.height,
			    width = _state.width,
			    animateStatus = _state.animateStatus,
			    x = _state.x,
			    y = _state.y;


			if (animateStatus === ANIMATE_STATUS_NONE) return null;

			var selectedIndex = this.props.selectedIndex;


			return createPortal$1(React.createElement(
				'div',
				{
					className: classnames('bmbo-auto-container', {
						'bmbo-init': animateStatus === ANIMATE_STATUS_INIT,
						'bmbo-showing': animateStatus === ANIMATE_STATUS_SHOWING,
						'bmbo-hiding': animateStatus === ANIMATE_STATUS_HIDING
					}),
					style: {
						height: height + 'px',
						width: width + 'px',
						left: x + 'px',
						top: y + 'px'
					},

					ref: this.setContainerRef,
					onTransitionEnd: this.onContainerTransitionEnd
				},
				React.createElement(
					'ul',
					{ className: 'bmbo-auto-list', ref: this.setListRef },
					(currentList || []).map(function (_ref, index) {
						var value = _ref.value;
						return React.createElement(
							'li',
							{ key: index, className: classnames(selectedIndex === index && 'bmbo-active') },
							value
						);
					})
				)
			));
		}
	}]);
	return List;
}(React.PureComponent);

List.propTypes = {
	list: propTypes.array,
	selectedIndex: propTypes.number,
	rect: propTypes.object
};

var KEY_UP = 38;
var KEY_DOWN = 40;
var KEY_ENTER = 13;
var OP_KEY_LIST = [KEY_UP, KEY_DOWN, KEY_ENTER];
var id = 0;

var AutoComplete = function (_React$PureComponent) {
	inherits(AutoComplete, _React$PureComponent);

	function AutoComplete() {
		classCallCheck(this, AutoComplete);

		var _this = possibleConstructorReturn(this, (AutoComplete.__proto__ || Object.getPrototypeOf(AutoComplete)).call(this));

		_this.onKeyDown = function (nodeOnChange) {
			for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
				args[_key - 1] = arguments[_key];
			}

			var event = args[0];

			// Ignore if list is empty
			var _this$state = _this.state,
			    selectedIndex = _this$state.selectedIndex,
			    list = _this$state.list;

			if (!list || !list.length) return;

			// Check if is direct key
			var which = event.which;

			if (!OP_KEY_LIST.includes(which)) return;
			event.preventDefault();

			if (which === KEY_UP || which === KEY_DOWN) {
				// Move operation
				var myIndex = selectedIndex;
				if (which === 38) {
					myIndex -= 1;
				} else if (which === 40) {
					myIndex += 1;
				}
				myIndex = (myIndex + list.length) % list.length;

				_this.setState({
					selectedIndex: myIndex
				});
			} else if (which === KEY_ENTER) {
				// Select item
				event.target.value = list[selectedIndex].value;
				nodeOnChange.apply(undefined, args);

				_this.setState({
					selectedIndex: 0,
					list: null
				});
			}
		};

		_this.onKeyUp = function (event) {
			var which = event.which,
			    target = event.target;


			if (OP_KEY_LIST.includes(which)) return;

			var list = _this.getMatchList(target.value);

			_this.setState({
				list: list,
				rect: target.getBoundingClientRect()
			});
		};

		_this.onBlur = function () {
			_this.setState({
				selectedIndex: 0,
				list: null
			});
		};

		_this.getMatchList = function (str) {
			var matcher = _this.props.matcher;

			if (!matcher || !str) return null;

			var myStr = str.toUpperCase();

			if (Array.isArray(matcher)) {
				var len = matcher.length;
				var list = [];
				for (var i = 0; i < len; i += 1) {
					var cur = matcher[i];

					// String match
					if (typeof cur === 'string') {
						if (cur.toUpperCase().includes(myStr)) {
							list.push({ value: cur });
						}
					} else {
						var value = cur.value;

						if (String(value).toUpperCase().includes(myStr)) {
							list.push(cur);
						}
					}

					if (list.length >= 10) break;
				}

				return list;
			}

			return [];
		};

		_this.state = {
			list: null,
			selectedIndex: 0
		};

		_this.id = id;
		id += 1;
		return _this;
	}

	createClass(AutoComplete, [{
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var _props = this.props,
			    children = _props.children,
			    props = objectWithoutProperties(_props, ['children']);
			var _state = this.state,
			    list = _state.list,
			    rect = _state.rect,
			    selectedIndex = _state.selectedIndex;


			delete props.matcher;

			var myChildren = children || React.createElement(Input, null);

			var $children = React.Children.map(myChildren, function (node) {
				var nodeOnKeyDown = getValue(node, ['props', 'onKeyDown']);
				var nodeOnKeyUp = getValue(node, ['props', 'onKeyUp']);
				var nodeOnBlur = getValue(node, ['props', 'onBlur']);
				var nodeOnChange = getValue(node, ['props', 'onChange']) || _this2.props.onChange;

				return React.cloneElement(node, _extends({}, props, {
					onKeyDown: function onKeyDown() {
						for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
							args[_key2] = arguments[_key2];
						}

						if (nodeOnKeyDown) nodeOnKeyDown.apply(undefined, args);
						_this2.onKeyDown.apply(_this2, [nodeOnChange].concat(args));
					},
					onKeyUp: function onKeyUp() {
						if (nodeOnKeyUp) nodeOnKeyUp.apply(undefined, arguments);
						_this2.onKeyUp.apply(_this2, arguments);
					},
					onBlur: function onBlur() {
						if (nodeOnBlur) nodeOnBlur.apply(undefined, arguments);
						_this2.onBlur.apply(_this2, arguments);
					}
				}));
			});

			return [].concat(toConsumableArray($children), [React.createElement(List, { key: 'list_' + this.id, list: list, rect: rect, selectedIndex: selectedIndex })]);
		}
	}]);
	return AutoComplete;
}(React.PureComponent);

AutoComplete.propTypes = {
	children: propTypes.node,
	matcher: propTypes.oneOfType([propTypes.array, propTypes.func]),
	onChange: propTypes.func
};

var BAMBOO_TABLE_COLUMN = 'BAMBOO_TABLE_COLUMN';

var Column = function Column() {
	return null;
};

Column.propTypes = {
	title: propTypes.node,
	render: propTypes.func
};

Column[BAMBOO_COMPONENT] = BAMBOO_TABLE_COLUMN;

var Pagination = function (_React$Component) {
	inherits(Pagination, _React$Component);

	function Pagination() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Pagination);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Pagination.__proto__ || Object.getPrototypeOf(Pagination)).call.apply(_ref, [this].concat(args))), _this), _this.gotoPage = function (page) {
			var gotoPage = _this.props.gotoPage;

			if (gotoPage) gotoPage(page);
		}, _this.goPrevPage = function () {
			var page = _this.props.page;

			if (page > 1) _this.gotoPage(page - 1);
		}, _this.goNextPage = function () {
			var _this$props = _this.props,
			    page = _this$props.page,
			    count = _this$props.count,
			    pageSize = _this$props.pageSize;

			var totalPages = Math.ceil(count / pageSize);
			if (page < totalPages) _this.gotoPage(page + 1);
		}, _this.goFirstPage = function () {
			_this.gotoPage(1);
		}, _this.goLastPage = function () {
			var _this$props2 = _this.props,
			    count = _this$props2.count,
			    pageSize = _this$props2.pageSize;

			var totalPages = Math.ceil(count / pageSize);
			_this.gotoPage(totalPages);
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Pagination, [{
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var _props = this.props,
			    count = _props.count,
			    pageSize = _props.pageSize,
			    _props$page = _props.page,
			    page = _props$page === undefined ? 1 : _props$page,
			    _props$range = _props.range,
			    range = _props$range === undefined ? 9 : _props$range,
			    disabled = _props.disabled,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['count', 'pageSize', 'page', 'range', 'disabled', 'className']);

			delete props.gotoPage;

			var totalPages = Math.ceil(count / pageSize);

			var lRange = Math.floor((range - 1) / 2);
			var rRange = range - 1 - lRange;
			var minPage = page - lRange;
			var maxPage = page + rRange;

			if (minPage < 1) {
				maxPage += -minPage + 1;
				minPage = 1;
			}
			if (maxPage > totalPages) {
				minPage -= maxPage - totalPages;
				maxPage = totalPages;
			}
			if (minPage < 1) minPage = 1;

			var $navItems = [];
			var index = 0;

			var _loop = function _loop(i) {
				$navItems.push(React.createElement(
					Button,
					{
						key: index,
						disabled: disabled,
						onClick: function onClick() {
							_this2.gotoPage(i);
						},
						type: page === i ? 'primary' : 'weak',
						transparent: true
					},
					i
				));

				index += 1;
			};

			for (var i = minPage; i <= maxPage; i += 1) {
				_loop(i);
			}

			return React.createElement(
				Group,
				_extends({ className: classnames('bmbo-pagination', className) }, props),
				React.createElement(
					Button,
					{ type: 'weak', transparent: true, onClick: this.goFirstPage, disabled: page <= 1 || disabled },
					'\xAB'
				),
				React.createElement(
					Button,
					{ type: 'weak', transparent: true, onClick: this.goPrevPage, disabled: page <= 1 || disabled },
					'\u2039'
				),
				$navItems,
				React.createElement(
					Button,
					{ type: 'weak', transparent: true, onClick: this.goNextPage, disabled: page >= totalPages || disabled },
					'\u203A'
				),
				React.createElement(
					Button,
					{ type: 'weak', transparent: true, onClick: this.goLastPage, disabled: page >= totalPages || disabled },
					'\xBB'
				)
			);
		}
	}]);
	return Pagination;
}(React.Component);

Pagination.propTypes = {
	gotoPage: propTypes.func,
	count: propTypes.number.isRequired,
	page: propTypes.number,
	pageSize: propTypes.number,
	range: propTypes.number,
	disabled: propTypes.bool,
	className: propTypes.string
};

var Table = function (_React$Component) {
	inherits(Table, _React$Component);

	function Table() {
		classCallCheck(this, Table);

		var _this = possibleConstructorReturn(this, (Table.__proto__ || Object.getPrototypeOf(Table)).call(this));

		_this.getPage = function () {
			var page = _this.props.page;

			return page === undefined ? _this.state.page : page;
		};

		_this.getCount = function () {
			var _this$props = _this.props,
			    count = _this$props.count,
			    data = _this$props.data;

			return count === undefined ? (data || []).length : count;
		};

		_this.getColumnList = function () {
			var children = _this.props.children;

			return mapChildrenByType(children, BAMBOO_TABLE_COLUMN);
		};

		_this.getList = function () {
			var _this$props2 = _this.props,
			    pageSize = _this$props2.pageSize,
			    page = _this$props2.page,
			    count = _this$props2.count,
			    data = _this$props2.data;
			var list = _this.state.list;


			if (page !== undefined && count !== undefined) {
				return data;
			}

			var myPage = _this.getPage();
			return list.slice((myPage - 1) * pageSize, myPage * pageSize);
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (prevProps.data === nextProps.data) return;

			_this.setState({
				list: (nextProps.data || []).concat()
			}, function () {
				// TODO: Support sort function
			});
		};

		_this.gotoPage = function (page) {
			var onPageChange = _this.props.onPageChange;

			_this.setState({ page: page });

			if (onPageChange) onPageChange(page);
		};

		_this.state = {
			page: 1,
			list: []
		};
		return _this;
	}

	createClass(Table, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _props = this.props,
			    className = _props.className,
			    pageSize = _props.pageSize,
			    bordered = _props.bordered,
			    hover = _props.hover,
			    onRowClick = _props.onRowClick,
			    props = objectWithoutProperties(_props, ['className', 'pageSize', 'bordered', 'hover', 'onRowClick']);

			delete props.page;
			delete props.count;
			delete props.data;
			delete props.onPageChange;

			var page = this.getPage();

			var columnList = this.getColumnList().map(function (c) {
				return c.props;
			});
			var list = this.getList();

			return React.createElement(
				'div',
				{ className: 'bmbo-table-container' },
				React.createElement(
					'table',
					_extends({
						className: classnames('bmbo-table', bordered && 'bmbo-bordered', className)

					}, props),
					React.createElement(
						'colgroup',
						null,
						columnList.map(function (_ref, colIndex) {
							var width = _ref.width;
							return React.createElement('col', { key: colIndex, width: width });
						})
					),
					React.createElement(
						'thead',
						null,
						React.createElement(
							'tr',
							null,
							columnList.map(function (_ref2, colIndex) {
								var title = _ref2.title,
								    name = _ref2.name;
								return React.createElement(
									'td',
									{ key: colIndex },
									title || name
								);
							})
						)
					),
					React.createElement(
						'tbody',
						{ className: classnames(hover && 'bmbo-hover') },
						list.map(function (item, rowIndex) {
							var trProps = {};

							if (onRowClick) {
								trProps.role = 'button';
								trProps.tabIndex = -1;
								trProps.onClick = function () {
									onRowClick(item, rowIndex);
								};
							}

							return React.createElement(
								'tr',
								_extends({ key: rowIndex }, trProps),
								columnList.map(function (column, colIndex) {
									var name = column.name,
									    render$$1 = column.render,
									    colProps = objectWithoutProperties(column, ['name', 'render']);

									delete colProps.width;

									return React.createElement(
										'td',
										_extends({ key: colIndex }, colProps),
										render$$1 ? render$$1(item) : getValue(item, name)
									);
								})
							);
						})
					)
				),
				React.createElement(
					'div',
					{ className: 'bmbo-table-pagination' },
					React.createElement(Pagination, {
						page: page,
						pageSize: pageSize,
						count: this.getCount(),
						gotoPage: this.gotoPage
					})
				)
			);
		}
	}]);
	return Table;
}(React.Component);

Table.propTypes = {
	className: propTypes.string,
	bordered: propTypes.bool,
	hover: propTypes.bool,
	children: propTypes.node,

	data: propTypes.array,
	page: propTypes.number,
	count: propTypes.number,
	pageSize: propTypes.number,

	onPageChange: propTypes.func,
	onRowClick: propTypes.func
};

Table.defaultProps = {
	pageSize: 10
};

Table.Column = Column;

var Label = function Label(_ref) {
	var type = _ref.type,
	    size = _ref.size,
	    className = _ref.className,
	    children = _ref.children,
	    props = objectWithoutProperties(_ref, ['type', 'size', 'className', 'children']);
	return React.createElement(
		'span',
		_extends({
			className: classnames('bmbo-label', 'bmbo-' + (type || 'primary'), 'bmbo-' + (size || 'md'), className)
		}, props),
		children
	);
};

Label.propTypes = {
	children: propTypes.node,

	type: propTypes.string,
	className: propTypes.string,
	size: propTypes.string,
	disabled: propTypes.bool,
	onClick: propTypes.func,
	transparent: propTypes.bool,
	checked: propTypes.bool
};

var FlatList = function (_React$Component) {
	inherits(FlatList, _React$Component);

	function FlatList() {
		classCallCheck(this, FlatList);

		var _this = possibleConstructorReturn(this, (FlatList.__proto__ || Object.getPrototypeOf(FlatList)).call(this));

		_this.onItemClick = function (event, level, itemValue) {
			var _this$props = _this.props,
			    value = _this$props.value,
			    onChange = _this$props.onChange;


			var myValue = toArray$1(value).slice(0, level);
			myValue[level] = itemValue;

			if (onChange) {
				onChange(wrapperEventValue(event, _this.$ele, myValue));
			}
		};

		_this.setRef = function (ele) {
			_this.$ele = ele;
		};

		_this.getMaxLevel = function () {
			return _this.props.columns || _this.state.maxLevel;
		};

		_this.getList = function () {
			var level = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
			var _this$props$list = _this.props.list,
			    list = _this$props$list === undefined ? [] : _this$props$list;
			var levelLists = _this.state.levelLists;


			if (level === 0) return list;

			return levelLists[level] || [];
		};

		_this.refreshList = function (prevProps, nextProps) {
			if (prevProps.value === nextProps.value && prevProps.list === nextProps.list) return;

			var _nextProps$list = nextProps.list,
			    list = _nextProps$list === undefined ? [] : _nextProps$list;

			var levelLists = {};
			var valueList = toArray$1(nextProps.value);

			var current = list;

			var _loop = function _loop(i) {
				var currentValue = valueList[i];
				current = current.find(function (_ref) {
					var value = _ref.value;
					return value === currentValue;
				});

				if (!current) {
					// Skip if path not match
					return 'break';
				}

				current = current.list || [];
				levelLists[i + 1] = current;
			};

			for (var i = 0; i < valueList.length; i += 1) {
				var _ret = _loop(i);

				if (_ret === 'break') break;
			}

			_this.setState({ levelLists: levelLists });
		};

		_this.state = {
			maxLevel: 1,
			levelLists: {}
		};
		return _this;
	}

	createClass(FlatList, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.refreshList({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.refreshList(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var _props = this.props,
			    size = _props.size,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['size', 'className']);


			delete props.value;
			delete props.list;
			delete props.multi;
			delete props.columns;
			delete props.onChange;

			var myValue = toArray$1(this.props.value);

			return React.createElement(
				'div',
				_extends({}, props, {
					className: classnames('bmbo-flat-list', 'bmbo-' + (size || 'md'), className),
					ref: this.setRef
				}),
				new Array(this.getMaxLevel()).fill(0).map(function (_, level) {
					var list = _this2.getList(level);
					var levelValue = myValue[level];

					return React.createElement(
						'ul',
						{
							key: level,
							className: 'bmbo-flat-list-column'
						},
						list.map(function (_ref2) {
							var title = _ref2.title,
							    value = _ref2.value;
							return React.createElement(
								'li',
								{
									className: classnames('bmbo-flat-list-item', levelValue === value && 'bmbo-active'),
									key: value
								},
								React.createElement(
									'span',
									{
										className: 'bmbo-padding',
										role: 'button',
										tabIndex: -1,
										onClick: function onClick(event) {
											_this2.onItemClick(event, level, value);
										}
									},
									title || value
								)
							);
						})
					);
				})
			);
		}
	}]);
	return FlatList;
}(React.Component);

FlatList.propTypes = {
	value: propTypes.node,
	list: propTypes.array,
	multi: propTypes.array,
	columns: propTypes.number,
	onChange: propTypes.func,

	size: propTypes.string,
	className: propTypes.string
};

var FUNC_LIST = ['getFullYear', 'getMonth', 'getDate', 'getDay', 'getHours', 'getMinutes', 'getSeconds', 'getTime', { name: 'setFullYear', returnThis: true }, { name: 'setMonth', returnThis: true }, { name: 'setDate', returnThis: true }, { name: 'setHours', returnThis: true }, { name: 'setMinutes', returnThis: true }, { name: 'setSeconds', returnThis: true }, 'toString', 'toISOString'];

function toDate(date) {
	if (date === undefined || date === null) return null;

	var myDate = date;
	var type = typeof date === 'undefined' ? 'undefined' : _typeof(date);

	switch (type) {
		case 'number':
		case 'string':
			if (date === '') {
				myDate = new Date();
			} else {
				myDate = new Date(date);
			}
			break;
	}

	if (myDate._isAMomentObject) {
		// Process if is moment object
		myDate = new Date();
		myDate.setFullYear(myDate.year(), myDate.month(), myDate.date());
		myDate.setHours(myDate.hour(), myDate.minute(), myDate.second());

		return myDate;
	} else if (myDate instanceof Date) {
		return myDate;
	}

	return null;
}

/**
 * Wrap a simple Date class like light version of moment.
 */

var BambooDate = function () {
	function BambooDate() {
		classCallCheck(this, BambooDate);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		var myArgs = args.length === 0 ? [''] : args;
		this.date = toDate.apply(undefined, myArgs);
	}

	createClass(BambooDate, [{
		key: 'setDay',
		value: function setDay(target) {
			var currentDay = this.date.getDay();
			var distance = target - currentDay;
			this.date.setDate(this.date.getDate() + distance);
			return this;
		}
	}, {
		key: 'clone',
		value: function clone() {
			return new BambooDate(this.getTime());
		}
	}, {
		key: 'format',
		value: function format() {
			var _format = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

			return _format.replace(/YYYY/g, this.getFullYear()).replace(/MM/g, String(this.getMonth() + 1).padStart(2, '0')).replace(/DD/g, String(this.getDate()).padStart(2, '0')).replace(/HH/g, String(this.getHours()).padStart(2, '0')).replace(/mm/g, String(this.getMinutes()).padStart(2, '0')).replace(/SS/g, String(this.getSeconds()).padStart(2, '0'));
		}
	}]);
	return BambooDate;
}();

FUNC_LIST.forEach(function (func) {
	var config = typeof func === 'string' ? { name: func } : func;

	BambooDate.prototype[config.name] = function dateFunc() {
		var _date;

		var returnValue = (_date = this.date)[config.name].apply(_date, arguments);

		if (config.returnThis) return this;
		return returnValue;
	};
});

var DatePanel = function DatePanel(_ref) {
	var date = _ref.date,
	    viewDate = _ref.viewDate,
	    startDate = _ref.startDate,
	    endDate = _ref.endDate,
	    triggerChangeEvent = _ref.triggerChangeEvent,
	    className = _ref.className,
	    setRef = _ref.setRef,
	    props = objectWithoutProperties(_ref, ['date', 'viewDate', 'startDate', 'endDate', 'triggerChangeEvent', 'className', 'setRef']);

	var $dateList = [];
	var currentDate = startDate.clone();

	var _loop = function _loop() {
		var pointDate = currentDate.clone();

		$dateList.push(React.createElement(
			'li',
			{
				key: pointDate.format('YYYY-MM-DD'),
				className: classnames({
					'bmbo-deprecated': viewDate && pointDate.getMonth() !== viewDate.getMonth(),
					'bmbo-active': date && pointDate.getFullYear() === date.getFullYear() && pointDate.getMonth() === date.getMonth() && pointDate.getDate() === date.getDate()
				}),
				role: 'button',
				tabIndex: -1,
				onClick: function onClick(event) {
					triggerChangeEvent(event, pointDate);
				}
			},
			pointDate.getDate()
		));

		currentDate.setDate(currentDate.getDate() + 1);
	};

	while (currentDate.getTime() <= endDate.getTime()) {
		_loop();
	}

	return React.createElement(
		'ul',
		_extends({ className: classnames('bmbo-date-picker-panel', className), ref: setRef }, props),
		$dateList
	);
};

DatePanel.propTypes = {
	date: propTypes.object,
	viewDate: propTypes.object,
	startDate: propTypes.object,
	endDate: propTypes.object,
	className: propTypes.string,
	triggerChangeEvent: propTypes.func,
	setRef: propTypes.func
};

var DATE_LIST = ['SUN', 'MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT'];

var DatePicker = function (_React$Component) {
	inherits(DatePicker, _React$Component);

	function DatePicker() {
		classCallCheck(this, DatePicker);

		var _this = possibleConstructorReturn(this, (DatePicker.__proto__ || Object.getPrototypeOf(DatePicker)).call(this));

		_this.onSwitchDateView = function (nextDate) {
			if (_this.state.animateStatus !== ANIMATE_STATUS_NONE) return;
			_this.switchDateView(nextDate);
		};

		_this.onTransitionEnd = function (event) {
			var _this$state = _this.state,
			    animateStatus = _this$state.animateStatus,
			    nextViewDate = _this$state.nextViewDate;


			if (event.target === _this.$nextView && animateStatus === ANIMATE_STATUS_SHOWN) {
				_this.seq.next(function () {
					_this.setState({
						animateStatus: ANIMATE_STATUS_NONE,
						viewDate: nextViewDate,
						nextViewDate: null
					});
				});
			}
		};

		_this.setRef = function (ele) {
			_this.$ele = ele;
		};

		_this.setViewRef = function (ele) {
			_this.$view = ele;
		};

		_this.setNextViewRef = function (ele) {
			_this.$nextView = ele;
		};

		_this.checkUpdate = function (prevProps, nextProps) {
			if (prevProps.value === nextProps.value) return;

			var viewDate = _this.state.viewDate;

			var nextDate = new BambooDate(nextProps.value);

			if (!prevProps.value || viewDate && viewDate.format('YYYY-MM') === nextDate.format('YYYY-MM')) {
				_this.setState({
					viewDate: nextDate
				});
			} else {
				var prevDate = new BambooDate(prevProps.value);
				if (prevDate.format('YYYY-MM') === nextDate.format('YYYY-MM')) return;

				_this.switchDateView(nextDate);
			}
		};

		_this.switchDateView = function (nextDate) {
			_this.seq.next(function (done) {
				_this.setState({
					animateStatus: ANIMATE_STATUS_SHOWING,
					nextViewDate: nextDate
				}, done);
			}).next(function (done) {
				_this.setState({
					animateStatus: ANIMATE_STATUS_SHOWN
				}, done);
			});
		};

		_this.triggerChangeEvent = function (event, date) {
			var onChange = _this.props.onChange;

			if (onChange) {
				onChange(wrapperEventValue(event, _this.$ele, date.format('YYYY-MM-DD')));
			}
		};

		_this.state = {
			animateStatus: ANIMATE_STATUS_NONE,

			viewDate: null, // This is used for show current year-month panel
			nextViewDate: null // This is used for animation panel
		};

		_this.seq = new Sequence();
		return _this;
	}

	createClass(DatePicker, [{
		key: 'componentWillMount',
		value: function componentWillMount() {
			this.checkUpdate({}, this.props);
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(nextProps) {
			this.checkUpdate(this.props, nextProps);
		}
	}, {
		key: 'render',
		value: function render$$1() {
			var _this2 = this;

			var _state = this.state,
			    animateStatus = _state.animateStatus,
			    viewDate = _state.viewDate,
			    nextViewDate = _state.nextViewDate;

			if (!viewDate) return null;

			var _props = this.props,
			    value = _props.value,
			    className = _props.className,
			    props = objectWithoutProperties(_props, ['value', 'className']);

			delete props.onChange;

			var date = new BambooDate(value);

			var isShowing = animateStatus === ANIMATE_STATUS_SHOWING;
			var isShown = animateStatus === ANIMATE_STATUS_SHOWN;

			// Current view
			var startDate = viewDate.clone().setDate(1);
			var endDate = viewDate.clone().setDate(1);
			startDate.setDay(0);
			endDate.setMonth(endDate.getMonth() + 1).setDate(0).setDay(6);
			var $view = React.createElement(DatePanel, {
				date: date,
				viewDate: viewDate,
				startDate: startDate,
				endDate: endDate,
				triggerChangeEvent: this.triggerChangeEvent,
				setRef: this.setViewRef,
				className: classnames({
					'bmbo-left': isShown && nextViewDate.getTime() > viewDate.getTime(),
					'bmbo-right': isShown && nextViewDate.getTime() < viewDate.getTime()
				})
			});

			// Next view
			var $nextView = void 0;
			if (animateStatus !== ANIMATE_STATUS_NONE) {
				var nextStartDate = nextViewDate.clone().setDate(1);
				var nextEndDate = nextViewDate.clone().setDate(1);
				nextStartDate.setDay(0);
				nextEndDate.setMonth(nextEndDate.getMonth() + 1).setDate(0).setDay(6);
				$nextView = React.createElement(DatePanel, {
					date: date,
					viewDate: nextViewDate,
					startDate: nextStartDate,
					endDate: nextEndDate,
					setRef: this.setNextViewRef,
					onTransitionEnd: this.onTransitionEnd,
					className: classnames('bmbo-target', {
						'bmbo-left': isShowing && nextViewDate.getTime() < viewDate.getTime(),
						'bmbo-right': isShowing && nextViewDate.getTime() > viewDate.getTime()
					})
				});
			}

			// Content style
			var contentStyle = {};
			if (canUseDOM) {
				if (isShowing && this.$view) {
					contentStyle.height = this.$view.clientHeight + 'px';
				} else if (!isShowing && this.$nextView) {
					contentStyle.height = this.$nextView.clientHeight + 'px';
				}
			}

			return React.createElement(
				'div',
				_extends({
					className: classnames('bmbo-date-picker', className),
					ref: this.setRef
				}, props),
				React.createElement(
					'div',
					{ className: 'bmbo-date-picker-header' },
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Prev Year',
						className: 'bmbo-date-picker-operation bmbo-year-prev',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setFullYear(viewDate.getFullYear() - 1));
						}
					}),
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Prev Month',
						className: 'bmbo-date-picker-operation bmbo-month-prev',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setMonth(viewDate.getMonth() - 1));
						}
					}),
					viewDate.format('YYYY-MM'),
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Next Year',
						className: 'bmbo-date-picker-operation bmbo-year-next',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setFullYear(viewDate.getFullYear() + 1));
						}
					}),
					React.createElement('a', {
						role: 'button',
						tabIndex: -1,
						'aria-label': 'Next Month',
						className: 'bmbo-date-picker-operation bmbo-month-next',
						onClick: function onClick() {
							_this2.onSwitchDateView(viewDate.clone().setMonth(viewDate.getMonth() + 1));
						}
					})
				),
				React.createElement(
					'ul',
					{ className: 'bmbo-date-picker-title' },
					DATE_LIST.map(function (str) {
						return React.createElement(
							'li',
							{ key: str },
							React.createElement(
								'strong',
								null,
								str
							)
						);
					})
				),
				React.createElement(
					'div',
					{
						className: classnames('bmbo-date-picker-content', {
							'bmbo-none': animateStatus === ANIMATE_STATUS_NONE,
							'bmbo-showing': isShowing
						}),
						style: contentStyle
					},
					$view,
					$nextView
				)
			);
		}
	}]);
	return DatePicker;
}(React.Component);

DatePicker.propTypes = {
	value: propTypes.oneOfType([propTypes.string, propTypes.number, propTypes.object]),
	className: propTypes.string,
	onChange: propTypes.func
};

var Picker = function (_React$Component) {
	inherits(Picker, _React$Component);

	function Picker() {
		var _ref;

		var _temp, _this, _ret;

		classCallCheck(this, Picker);

		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = Picker.__proto__ || Object.getPrototypeOf(Picker)).call.apply(_ref, [this].concat(args))), _this), _this.onChange = function (event) {
			var onChange = _this.props.onChange;

			if (onChange) {
				onChange(wrapperEventValue(event, _this.$input, event.target.value));
			}
		}, _this.setRef = function (ele) {
			_this.$input = ele;
		}, _temp), possibleConstructorReturn(_this, _ret);
	}

	createClass(Picker, [{
		key: 'render',
		value: function render$$1() {
			var value = this.props.value;


			var $pin = React.createElement(DatePicker, { value: value, onChange: this.onChange });

			return React.createElement(
				PinBox,
				{ pin: $pin, backdrop: true },
				React.createElement(Input, _extends({}, defineProperty({}, BAMBOO_INTERNAL_REF, this.setRef), this.props))
			);
		}
	}]);
	return Picker;
}(React.Component);

Picker.propTypes = {
	value: propTypes.string,
	onChange: propTypes.func
};

exports.Navigation = Nav;
exports.Row = Row;
exports.Col = Col;
exports.Button = Button;
exports.Icon = Icon;
exports.A = A;
exports.Form = Form;
exports.FormField = Field;
exports.Input = Input;
exports.Radio = Radio;
exports.Checkbox = Checkbox;
exports.Slider = Slider;
exports.Select = Select;
exports.Menu = Menu;
exports.Group = Group;
exports.GroupItem = Item;
exports.Modal = Modal$1;
exports.Curtain = Curtain;
exports.Progress = Progress;
exports.Tooltip = TooltipHolder;
exports.DynamicNumber = DynamicNumber;
exports.DynamicString = DynamicString;
exports.AutoComplete = AutoComplete;
exports.Table = Table;
exports.Label = Label;
exports.PinBox = PinBox;
exports.FlatList = FlatList;
exports.Picker = Picker;
exports.Pagination = Pagination;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=bamboo-ui.js.map
